#!/bin/bash

### SIDfm VM Server Manager ###

### Defines ###
CMD_NAME=$0
ARGS="$*"

# load profile
[ -f /etc/profile.d/lang.sh ] && source /etc/profile.d/lang.sh
[ -f /etc/profile.d/railsenv.sh ] && source /etc/profile.d/railsenv.sh
[ -f /etc/profile.d/rh-ruby23.sh ] && source /etc/profile.d/rh-ruby23.sh
[ -f /etc/profile.d/proxy.sh ] && source /etc/profile.d/proxy.sh
[ -f /etc/profile.d/timezone.sh ] && source /etc/profile.d/timezone.sh

# Paths
BASE_PATH=/var/lib/redmine
PLUGIN_PATH=${BASE_PATH}/plugins/sidfm-vm-server
HTTPDCONF_PATH=/etc/httpd/conf.d
LOGDIR=${BASE_PATH}/log
LOGFILE=${LOGDIR}/vm_server.log
SUPPORT_TMPDIR=/tmp/support
SUPPORT_FILE=/tmp/support.tar.gz

# status flags
FLG_PREFIX="/flag."
FLG_E_APACHE="enable-apache"
FLG_E_ISSUE_UPDATE="enable-issue-update"

# lock path
L_VM_DAEMON=/var/run/vm_daemon.lock

### functions ###
# logger
function log() {
    mes="$*"
    echo -n `date +'%Y/%m/%d %H:%M:%S'` >> $LOGFILE
    echo " [vm_manage] ${mes}" >> $LOGFILE
}
# usage
function usage() {
    echo "$0 <function> <args>"
    exit 1
}

### main ###
ARG_FUNC=$1
if [ "x$1" == "x" ]; then
    usage
fi
shift

case "$ARG_FUNC" in
    "server")
	case "$1" in
	    "status")
		pgrep -F /var/run/httpd/httpd.pid > /dev/null 2>&1
		if [ "x$?" == "x0" ]; then
		    echo "Server Running"
		else
		    echo "Server Stopped"
		fi
		;;
	    "enable")
		if [ ! -f "${FLG_PREFIX}${FLG_E_APACHE}" ]; then
		    touch "${FLG_PREFIX}${FLG_E_APACHE}"
		    echo "Server Enabled"
		else
		    echo "Server already Enabled"
		fi
		;;
	    "disable")
		if [ -f "${FLG_PREFIX}${FLG_E_APACHE}" ]; then
		    rm -f "${FLG_PREFIX}${FLG_E_APACHE}"
		    echo "Server Disabled"
		else
		    echo "Server already Disabled"
		fi
		;;
	    *)
		usage
		;;
	esac
	;;
    "issue_update")
	case "$1" in
	    "status")
		if [ -f "${FLG_PREFIX}${FLG_E_ISSUE_UPDATE}" ]; then
		    echo "Issue Update Enabled"
		else
		    echo "Issue Update Disabled"
		fi
		;;
	    "enable")
		if [ ! -f "${FLG_PREFIX}${FLG_E_ISSUE_UPDATE}" ]; then
		    touch "${FLG_PREFIX}${FLG_E_ISSUE_UPDATE}"
		    echo "Issue Update Enabled"
		else
		    echo "Issue Update already Enabled"
		fi
		;;
	    "disable")
		if [ -f "${FLG_PREFIX}${FLG_E_ISSUE_UPDATE}" ]; then
		    rm -f "${FLG_PREFIX}${FLG_E_ISSUE_UPDATE}"
		    echo "Issue Update Disabled"
		else
		    echo "Issue Update already Disabled"
		fi
		;;
	    *)
		usage
		;;
	esac
	;;
    "securecookie")
	case "$1" in
	    "status")
		if [ -f "${HTTPDCONF_PATH}/vm_securecookie.conf" ]; then
		    echo "Secure Cookie Enabled"
		else
		    echo "Secure Cookie Disabled"
		fi
		;;
	    "enable")
		if [ -f "${HTTPDCONF_PATH}/vm_securecookie.conf" ]; then
		    echo "Secure Cookie already Enabled"
		else
		    cp -f ${PLUGIN_PATH}/ROR/config/vm_securecookie.conf ${HTTPDCONF_PATH}/
		    echo "Secure Cookie Enabled"
		fi
		;;
	    "disable")
		if [ -f "${HTTPDCONF_PATH}/vm_securecookie.conf" ]; then
		    rm -f ${HTTPDCONF_PATH}/vm_securecookie.conf
		    echo "Secure Cookie Disabled"
		else
		    echo "Secure Cookie already Disabled"
		fi
		;;
	    *)
		usage
		;;
	esac
	;;
    "xforwardedhost")
	case "$1" in
	    "status")
		if [ -f "${HTTPDCONF_PATH}/vm_xforwardedhost.conf" ]; then
		    echo "unset X-Forwarded-Host Enabled"
		else
		    echo "unset X-Forwarded-Host Disabled"
		fi
		;;
	    "enable")
		if [ -f "${HTTPDCONF_PATH}/vm_xforwardedhost.conf" ]; then
		    echo "unset X-Forwarded-Host already Enabled"
		else
		    cp -f ${PLUGIN_PATH}/ROR/config/vm_xforwardedhost.conf ${HTTPDCONF_PATH}/
		    echo "unset X-Forwarded-Host Enabled"
		fi
		;;
	    "disable")
		if [ -f "${HTTPDCONF_PATH}/vm_xforwardedhost.conf" ]; then
		    rm -f ${HTTPDCONF_PATH}/vm_xforwardedhost.conf
		    echo "unset X-Forwarded-Host Disabled"
		else
		    echo "unset X-Forwarded-Host already Disabled"
		fi
		;;
	    *)
		usage
		;;
	esac
	;;
    "info")
	case "$1" in
	    "versions")
		cd ${BASE_PATH}
		bundle exec ruby bin/about 2>&1 | grep -v 'command not found'
		;;
	    "log")
		cat ${BASE_PATH}/log/production.log
		;;
	    "gen_support")
		rm -rf ${SUPPORT_TMPDIR}
		rm -f ${SUPPORT_FILE}
		mkdir -p ${SUPPORT_TMPDIR}
		date > ${SUPPORT_TMPDIR}/today.txt 2>&1
		pushd ${BASE_PATH}
		bundle exec ruby bin/about 2>&1 | grep -v 'command not found' > ${SUPPORT_TMPDIR}/rails_about.txt
		popd
		tar czf ${SUPPORT_FILE} ${SUPPORT_TMPDIR} ${LOGDIR} /var/log/httpd/error_log /var/log/httpd/ssl_error_log > /dev/null 2>&1
		;;
	esac
	;;
    *)
	usage
esac

