# -*- coding: utf-8 -*-

require 'sidfm_controller_support'

class SidfmCatGroupsController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport

  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles

  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session, only: [ :index ]

  before_action -> { get_summary_table_lines('cat-groups') }

  def check_roles
    if !@sidfm_user || !@sidfm_roles.include?(:host_admin) then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           User.current.categories.pluck(:id)
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host)
                                           .where(:sidfmd_category_id => @user_category_ids)
                                           .where(:sidfmd_hosts => { :flg_delete => false })
                                           .select(:sidfmd_host_id)
                                           .order(:sidfmd_host_id)
                                           .distinct
                                           .pluck(:sidfmd_host_id)
                     end
  end

  def get_user_categories
    categories = @role_admin ? SidfmdCategory : SidfmdCategory.where(:id => @user_category_ids)
    categories = categories.where(:is_system => false)
  end

  def get_all_content
    all_content = SidfmdCatGroup.all
    unless @role_admin then
      all_content = all_content.where(:id => User.current.sidfmd_cat_groups_users.select(:sidfmd_cat_group_id))
    end
    all_content
  end

  def parse_params
    f_params = params.fetch(:pages, {}).permit(:order, :offset, :per_page, :filters, :checked, :all_checked)
    @groups = {}
    @groups[:per_page] = (@summary_table_lines || f_params[:per_page] || 10).to_i
    @groups[:offset] = (f_params[:offset] || 0).to_i
    @groups[:order] = f_params[:order] || 'name_asc'
    @groups[:filters] = JSON.parse((f_params[:filters] || '{}'), symbolize_names: true)
    @groups[:checked] = JSON.parse(f_params[:checked] || '[]')
    @groups[:all_checked] = f_params[:all_checked] || false
  end

  FILTER_KEYS = {}

  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:name] = ''
      FILTER_KEYS[:category_id] = '0'
    end
  end

  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @groups[:filters].has_key?(key) then
        @groups[:filters][key] = val
      end
    end
    @default_filters = FILTER_KEYS
  end

  def filter_content
    if @groups[:filters][:name].size > 0 then
      @groups[:all_content] = @groups[:all_content].where("name ILIKE ?", "%#{ActiveRecord::Base.send(:sanitize_sql_like, @groups[:filters][:name])}%")
    end

    if @groups[:filters][:category_id].to_i != 0 then
      #if @groups[:filters][:category_id].to_i == -1 then
      #   @groups[:all_content] = @groups[:all_content].where.not(
      #     :id => SidfmdCatGroupsCategory.select(:sidfmd_cat_group_id)
      #   )
      #else
        @groups[:all_content] = @groups[:all_content].where(
          :id => SidfmdCatGroupsCategory.where(:sidfmd_category_id => @groups[:filters][:category_id].to_i).select(:sidfmd_cat_group_id)
        )
      #end
    end
  end

  def sort_content
    order_data = @groups[:order].split('_')
    col = order_data.first
    order = SidfmControllerSupport.check_order(order_data.last)
    
    case col
    when 'name' then
      @groups[:all_content] = @groups[:all_content].order(%|LOWER(name COLLATE "C") #{order.upcase}|)
    when 'categories' then
      @groups[:all_content] = @groups[:all_content].joins(%|
        LEFT OUTER JOIN(
          SELECT
            sidfmd_cat_groups_categories.sidfmd_cat_group_id AS order_min_category_cat_group_id,
            MIN(sidfmd_categories.name COLLATE "C") AS order_min_category_name
          FROM
            sidfmd_cat_groups_categories INNER JOIN sidfmd_categories ON sidfmd_cat_groups_categories.sidfmd_category_id = sidfmd_categories.id
          #{(@role_admin || @user_category_ids.size == 0) ? '' : "WHERE sidfmd_categories.id IN (SELECT unnest(ARRAY[#{@user_category_ids.join(', ')}]))"}
          GROUP BY sidfmd_cat_groups_categories.sidfmd_cat_group_id
        ) AS order_min_category_names
        ON sidfmd_cat_groups.id = order_min_category_names.order_min_category_cat_group_id 
      |)
      .order("order_min_category_name #{order.upcase}")
      .order(%|LOWER(name COLLATE "C") #{order.upcase}|)
    end
  end

  #グループのカテゴリ
  def create_cat_group_categories(group_ids)
    @group_categories = {}
    group_cats = SidfmdCatGroup.joins(:sidfmd_categories)
      .where(:id => group_ids)
      .order('LOWER(sidfmd_categories.name COLLATE "C")')
      .select(
        :id,
        'sidfmd_categories.id AS cat_id',
        'sidfmd_categories.name AS cat_name',
        'sidfmd_categories.desc AS cat_desc'
      )
    #unless @role_admin then
    #  group_cats = group_cats.where(:sidfmd_categories => { :id => @user_category_ids })
    #end
    group_cats.each do |cat_group|
      @group_categories[cat_group.id] = [] unless @group_categories[cat_group.id]
      @group_categories[cat_group.id] << {
        :id => cat_group.cat_id,
        :name => cat_group.cat_name,
        :desc => cat_group.cat_desc,
      }
    end
  end

  def create_summary_group_html(group, **opts)
    data = {
      :chkbox => view_context.create_summary_cat_group_chkbox(group.id, opts[:checked]),
      :id => view_context.create_summary_cat_group_id(group.id),
      :name => view_context.create_summary_cat_group_name(group.name),
      :categories => view_context.create_summary_cat_group_categories(
        (@group_categories[group.id] ? @group_categories[group.id].first : nil),
        (@group_categories[group.id] ? @group_categories[group.id].size : 0),
        group.id
      ),
      :edit => view_context.create_summary_cat_group_edit(
        group.id,
        group.name,
      )
    }
  end

  def process_related_rules(**opts)
    r = true
    param = {
      :role_admin => @role_admin,
      :user_category_ids => @user_category_ids,
      :messages_necessary => false
    }
    #ユーザのカテゴリの変更と考えて処理する
    case opts[:action]
    when 'delete' then
      param[:change_category_user_ids] = SidfmdCatGroup.user_cat_group_categories(
        :cat_group_ids => opts[:delete_ids],
        :change_category_ids => []
      ).stringify_keys
      param[:action] = 'bulk_category_change'
    when 'category_change' then
      param[:change_category_user_ids] = SidfmdCatGroup.user_cat_group_categories(
        :cat_group_ids => [ opts[:cat_group_category_ids].keys.first ].compact,
        :change_category_ids => opts[:cat_group_category_ids].values.first.compact
      ).stringify_keys
      param[:action] = 'category_change'
    else
      #invalid
    end
    ret = SidfmdRule.users_related_rule_info(param)
    if ret[:is_unchangeable_user] then
      r = false
    else
      SidfmdRuleChpUserSub.where(:id => ret[:delete_chp_user_sub_ids]).destroy_all if ret[:delete_chp_user_sub_ids].any?
      SidfmdRuleCond.where(:id => ret[:delete_cond_ids]).destroy_all if ret[:delete_cond_ids].any?
      SidfmdRuleAction.where(:id => ret[:delete_action_ids]).destroy_all if ret[:delete_action_ids].any?
      SidfmdRule.where(:id => ret[:delete_rule_ids]).destroy_all if ret[:delete_rule_ids].any?
    end
    r
  end

  public

  def index
    parse_params
    check_filters
    @page_name = 'カテゴリグループ設定'
    @user_categories = get_user_categories
  end

  def create
    f_params = params.permit(:name, :category_ids)
    name = f_params[:name]
    if !name || name.size == 0 || !f_params[:category_ids] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      category_ids = JSON.parse(f_params[:category_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end

    if category_ids.size == 0 then
      render :layout => false, :json => { :err_message => 'カテゴリを1つ以上選択してください。' } and return
    end

    begin
      SidfmdCatGroup.transaction do
        cat_group = SidfmdCatGroup.new(:name => name)
        category_ids = category_ids.map(&:to_i)
        categories = SidfmdCategory.where(:id => category_ids).where(:is_system => false)
        if !categories.exists? then
          render :layout => false, :json => { :err_message => 'カテゴリのないカテゴリグループは登録できません。' } and return
        else
          unless @role_admin then
            category_ids = category_ids & @user_category_ids
          end
          cat_group.sidfmd_categories = categories
          cat_group.save
          unless @role_admin then
            SidfmdCatGroupsUser.create(:user_id => User.current.id, :sidfmd_cat_group_id => cat_group.id)
          end
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '登録に失敗しました。' } and return
    end

    render :layout => false, :json => { :err_message => '' }
  end

  def modify
    f_params = params.permit(:id, :name, :category_ids)
    id = f_params[:id]
    name = f_params[:name]
    if !id || !name || name.size == 0 || !f_params[:category_ids] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      category_ids = JSON.parse(f_params[:category_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end

    if category_ids.size == 0 then
      render :layout => false, :json => { :err_message => 'カテゴリを1つ以上選択してください。' } and return
    end

    begin
      SidfmdCatGroup.transaction do
        cat_group = SidfmdCatGroup.find_by_id(id)
        if cat_group then
          unless cat_group.is_user_group?(:role_admin => @role_admin, :user_id => User.current.id) then
            raise Exception #不正
          end
          category_ids = category_ids.map(&:to_i)
          unless @role_admin then
            category_ids = category_ids & @user_category_ids
          end
          if !SidfmdCategory.where(:id => category_ids).where(:is_system => false).exists? then
            render :layout => false, :json => { :err_message => 'カテゴリのないカテゴリグループは登録できません。' } and return
          else
            rule_ret = process_related_rules( #ルールの処理
              :action => 'category_change',
              :cat_group_category_ids => { cat_group.id => category_ids }
            )
            unless rule_ret then
              raise Exception #不正
            end
            cat_group.update_attribute(:name, name)
            cat_group.update_categories(
              :role_admin => @role_admin,
              :user_category_ids => @user_category_ids,
              :user_id => User.current.id,
              :category_ids => category_ids
            )
          end
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '変更に失敗しました。' } and return
    end

    render :layout => false, :json => { :err_message => '' }
  end

  def delete
    f_params = params.permit(:id)
    if !f_params[:id] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    begin
      SidfmdCatGroup.transaction do
        cat_group = SidfmdCatGroup.find_by_id(f_params[:id])
        if cat_group then
          unless cat_group.is_user_group?(:role_admin => @role_admin, :user_id => User.current.id) then
            raise Exception #不正
          end
          rule_ret = process_related_rules( #ルールの処理
            :action => 'delete',
            :delete_ids => [cat_group.id]
          )
          unless rule_ret then
            raise Exception #不正
          end
          cat_group.destroy
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '削除に失敗しました。' } and return
    end

    render :layout => false, :json => { :err_message => '' }
  end

  def bulk_delete
    f_params = params.permit(:ids)
    if !f_params[:ids] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      ids = JSON.parse(f_params[:ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end

    begin
      ids = ids.map(&:to_i)
      unless @role_admin then
        all_content = get_all_content
        if (ids - all_content.pluck(:id)).size > 0 then
          raise Exception #不正
        end
      end
      SidfmdCatGroup.transaction do
        rule_ret =process_related_rules( #ルールの処理
          :action => 'delete',
          :delete_ids => ids
        )
        unless rule_ret then
          raise Exception #不正
        end
        SidfmdCatGroup.where(:id => ids).destroy_all
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end

    render :layout => false, :json => { :err_message => '' }
  end

  #JSON/TSV出力
  def read_download_data
    GC.start
    f_params = params.permit(:target, :executed_size, :process, :order, :filters, :ids)
    if !f_params[:ids] or !f_params[:target] or !f_params[:order] or !f_params[:filters] or
       !f_params[:executed_size] or !f_params[:process]
    then
      logger.info('params error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    format = f_params[:target]
    if ![ 'json', 'tsv' ].include?(format) then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    executed_size = f_params[:executed_size].to_i
    @groups = {}
    @groups[:order] = f_params[:order] || 'name_asc'
    @groups[:filters] = JSON.parse((f_params[:filters] || '{}'), symbolize_names: true)
    @groups[:all_content] = get_all_content
    check_filters
    sort_content
    filter_content
    @groups[:content_filtered] = @groups[:all_content]
    begin
      @groups[:ids] = JSON.parse(f_params[:ids] || '[]')
    rescue JSON::ParserError => e
      logger.error(e)
      render :layout => false, :json => { :err_message => 'エラーが発生しました。' } and return
    end

    case format
    when 'json' then
      if f_params[:process] == 'init' then
        @groups[:content_filtered] = @groups[:content_filtered].where(:id => @groups[:ids]) if @groups[:ids].any?
        temp_order = @groups[:order].to_s.split('_')
        init_json_summary_table_progress_download(
          :ids => @groups[:content_filtered].pluck(:id),
          :order_col_name => temp_order[0],
          :order_type => temp_order[1],
          :filters => @groups[:filters],
          :max_download_size => @groups[:content_filtered].count,
        )
      else
        create_cat_group_categories(@groups[:ids])
        create_json_data = Proc.new{|id|
          group = @groups[:content_filtered].find_by(:id => id)
          next unless group
          item = {}
          item[:id] = group.id
          item[:name] = group.name
          item[:categories] = (@group_categories[group.id] ? @group_categories[group.id].map{|grp| grp[:name]} : [])
          item
        }
        json_tsv_summary_table_progress_download(@groups[:ids], create_json_data, executed_size)
      end
    when 'tsv' then
      if f_params[:process] == 'init' then
        header = [ 'ID', 'グループ名', 'カテゴリ' ]
        @groups[:content_filtered] = @groups[:content_filtered].where(:id => @groups[:ids]) if @groups[:ids].any?
        init_tsv_summary_table_progress_download(@groups[:content_filtered].pluck(:id), header, @groups[:content_filtered].count)
      else
        create_cat_group_categories(@groups[:ids])
        create_tsv_data = Proc.new{|id|
          group = @groups[:content_filtered].find_by(:id => id)
          next unless group
          item = []
          item << group.id.to_s
          item << group.name
          item << (@group_categories[group.id] ? @group_categories[group.id].map{|grp| escape_category_tsv(grp[:name])}.join(',') : '')
          item
        }
        json_tsv_summary_table_progress_download(@groups[:ids], create_tsv_data, executed_size)
      end
    end
  end

  def download
    f_params = params.fetch(:download, {}).permit(:format)
    case f_params[:format]
    when 'json' then
      send_summary_table_json_file("vmlists-category-groups-json-#{Time.current.strftime('%Y%m%d%H%M%S')}.json")
    when 'tsv' then
      send_summary_table_tsv_file("vmlists-category-groups-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
    end
  end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    parse_params
    @groups[:per_page] = f_params[:per_page].to_i
    @groups[:all_content] = get_all_content
    check_filters
    sort_content
    filter_content
    @groups[:content_filtered] = @groups[:all_content]
    ids = @groups[:content_filtered].pluck(:id)
    create_cat_group_categories(ids)
    @groups[:content] = @groups[:content_filtered].limit(@groups[:per_page]).offset(@groups[:offset])
    user_id = User.current.id
    ret = {
      :rows => @groups[:content].map{|group|
        create_summary_group_html(
          group,
          {
            :user_id => user_id,
            :checked => @groups[:checked]
          }
        )
      },
      :total => ids.size,
      :all_ids => ids,
      :sidfm_roles => @sidfm_roles,
      :user_cats => @user_category_ids
    }
    render :layout => false, :json => ret
  end

  #グループのカテゴリを返す
  def api_get_cat_group_categories
    f_params = params.permit(:id)
    id = f_params[:id].to_i
    create_cat_group_categories([id])
    render :layout => false, :json => (@group_categories[id] || [])
  end

  #グループ名を返す
  def api_get_cat_group_names
    f_params = params.permit(:ids)
    ids = JSON.parse(f_params[:ids])
    all_content = get_all_content
    render :layout => false, :json => all_content.where(:id => ids).order('LOWER(name COLLATE "C")').pluck(:name)
  end

  #グループが設定されているユーザを返す
  def api_get_related_users
    f_params = params.permit(:ids)
    ids = JSON.parse(f_params[:ids])
    all_content = get_all_content
    users = all_content.joins(:users).where(:sidfmd_cat_groups => { :id => ids }).distinct.select(
      'users.login',
      'users.firstname',
      'users.lastname',
    )
    #unless @role_admin then
    #  users = users.where(:id => User.get_category_users(@user_category_ids).select(:id))
    #end
    users = users.sort_by{|user| user.login.downcase }
    render :layout => false, :json => users
  end

end
