# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmCvesController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport

  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles

  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session, only: [:index ]

  before_action -> { get_summary_table_lines('cves') }

  # 権限チェック
  def check_roles
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           User.current.categories.order(:id).pluck(:id)
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host)
                                           .where(:sidfmd_category_id => @user_category_ids)
                                           .where(:sidfmd_hosts => { :flg_delete => false })
                                           .select(:sidfmd_host_id)
                                           .order(:sidfmd_host_id)
                                           .distinct
                                           .pluck(:sidfmd_host_id)
                     end
  end

  def parse_params
    f_params = params.fetch(:pages, {}).permit(:order, :offset, :per_page, :filters, :checked, :all_checked)
    @cves = {}
    @cves[:per_page] = (@summary_table_lines || f_params[:per_page] || 10).to_i
    @cves[:offset] = (f_params[:offset] || 0).to_i
    @cves[:order] = f_params[:order] || 'cve_id_asc'
    @cves[:filters] = JSON.parse((f_params[:filters] || '{}'), symbolize_names: true)
    @cves[:checked] = JSON.parse(f_params[:checked] || '[]')
    @cves[:all_checked] = f_params[:all_checked] || false
  end

  #ソート
  #Group By前のデータをソートすること、表示分だけのデータのcve_idsを返すことに注意
  #ソートの結果は、CVEIDが一意になるようにする
  #返り値のcve_idsを使って、元データをソートする
  #引数はSidfmdVvCve
  def get_sorted_cve_ids(content, limit, offset)
    last_index = @cves[:order].rindex('_')
    col = @cves[:order].slice(0..last_index - 1)
    order = SidfmControllerSupport.check_order(@cves[:order].slice(last_index + 1..@cves[:order].size - 1))
    cve_ids = []
    q1 = "cve_id, cast(split_part(cve_id,'-',2) as integer) as cve_year, cast(split_part(cve_id,'-',3) as bigint) as cve_no"
    order2 = order == 'asc'? 'desc' : 'asc'
    q2 = "cve_year #{order2}, cve_no #{order2}"

    case col

    when 'cve_id' then
      content.distinct.select(Arel.sql(q1)).limit(limit).offset(offset).order(Arel.sql(q2)).all.map{|record| cve_ids.push(record.cve_id)} 

    when 'cvss' then
      #表示されるCVSSのデータに絞り込み、CVSS値順にソートしたcve_idを取得
      cve_ids = content.group(:cve_id)
        .select(:cve_id)
        .select("
          (
            SELECT COALESCE(c3_base_score,c2_base_score, 0.0) /* def create_contentと同じにする */ 
            FROM sidfmd_vv_cves AS cves_for_latest_cvss_score
            WHERE cves_for_latest_cvss_score.cve_id = sidfmd_vv_cves.cve_id
            ORDER BY
              CASE
              WHEN c3_update_datetime IS NOT NULL THEN
                100
              WHEN c2_update_datetime IS NOT NULL THEN
                200
              ELSE
                300
              END,
              c3_update_datetime DESC,
              c2_update_datetime DESC,
              id DESC
            LIMIT 1
          ) AS latest_cvss_score
        ")
        .order("latest_cvss_score #{(order == 'asc' ? 'desc' : 'asc').upcase}")
        .order(:cve_id => order.to_sym)
        .limit(limit)
        .offset(offset)
        .map(&:cve_id)
=begin
      cve_ids = content.where(
        :id => SidfmdVvCve.select("
                 (
                   SELECT id /* def create_contentを参照 */ 
                   FROM sidfmd_vv_cves AS cves_for_latest_cvss
                   WHERE cves_for_latest_cvss.cve_id = sidfmd_vv_cves.cve_id
                   ORDER BY
                     c3_update_datetime DESC,
                     c2_update_datetime DESC,
                     id DESC
                   LIMIT 1
                 )
              ")
      )
      .order("COALESCE(c3_base_score,c2_base_score, 0.0) #{(order == 'asc' ? 'desc' : 'asc').upcase}") #def create_contentを参照
      .order(:cve_id => order.to_sym)
      .limit(@cves[:per_page])
      .offset(@cves[:offset])
      .pluck(:cve_id)
=end
    when 'vuls' then
      #表示されるhtml_idのデータに絞り込み、html_id順にソートしたcve_idを取得
      cve_ids = content.where(
        :id => SidfmdVvCve.select("
                 (
                   SELECT cves_for_min_html_id.id
                   FROM sidfmd_vv_cves AS cves_for_min_html_id
                   INNER JOIN sidfmd_vul_views ON sidfmd_vul_views.id = cves_for_min_html_id.sidfmd_vul_view_id
                   WHERE cves_for_min_html_id.cve_id = sidfmd_vv_cves.cve_id #{(@role_admin || @user_host_ids.size == 0) ? "" : "AND sidfmd_vul_views.id IN (SELECT sidfmd_vul_view_id FROM sidfmd_vv_scsids WHERE sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}])))"}
                   ORDER BY abs(sidfmd_vul_views.html_id)
                   LIMIT 1
                 )
               ")
      )
      .order("
        (
          SELECT abs(html_id)
          FROM sidfmd_vul_views
          WHERE sidfmd_vul_views.id = sidfmd_vv_cves.sidfmd_vul_view_id
        ) #{order.upcase}
      ")
      .order(:cve_id => order.to_sym)
      .limit(limit)
      .offset(offset)
      .pluck(:cve_id)
    when 'hosts' then
      cve_ids = content.where(
        :id => SidfmdVvCve.select("
                 (
                   SELECT cves_for_min_host.id
                   FROM sidfmd_vv_cves AS cves_for_min_host
                   INNER JOIN sidfmd_vv_hosts ON sidfmd_vv_hosts.sidfmd_vul_view_id = cves_for_min_host.sidfmd_vul_view_id
                   WHERE cves_for_min_host.cve_id = sidfmd_vv_cves.cve_id #{(@role_admin || @user_host_ids.size == 0) ? "" : "AND sidfmd_vv_hosts.sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}]))"}
                   ORDER BY LOWER(display_name COLLATE \"C\")
                   LIMIT 1
                 )
               ")
      )
      .order("
        (
          SELECT display_name
          FROM sidfmd_vv_hosts
          WHERE sidfmd_vv_hosts.sidfmd_vul_view_id = sidfmd_vv_cves.sidfmd_vul_view_id
          ORDER BY LOWER(display_name COLLATE \"C\")
          LIMIT 1
        ) #{(order == 'asc' ? 'desc' : 'asc').upcase}
      ")
      .order(:cve_id => order.to_sym)
      .limit(limit)
      .offset(offset)
      .pluck(:cve_id)
    end
    cve_ids
  end

  FILTER_KEYS = {}

  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:cve_ids] = ''
      FILTER_KEYS[:cvsses] = []
      FILTER_KEYS[:cvss_vector] = {}
      FILTER_KEYS[:vuls] = ''
      FILTER_KEYS[:hosts] = ''
    end
  end

  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @cves[:filters].has_key?(key) then
        @cves[:filters][key] = val
      end
    end
    @default_filters = FILTER_KEYS
  end

  #フィルタ
  #Group By前のデータをフィルタすることに注意
  def filter_content
   
    #CVE ID検索
    if @cves[:filters][:cve_ids].size > 0 then
      @cves[:all_content] = @cves[:all_content].where('cve_id ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, @cves[:filters][:cve_ids].strip)}%")
    end

    #CVSS検索
    if @cves[:filters][:cvsses].size > 0 then
      label_values = {
        :notreg => nil,
        :d4 => { :top => 10.0 , :bottom => 9.0 },
        :d3 => { :top => 8.9, :bottom => 7.0 },
        :d2 => { :top => 6.9, :bottom => 4.0 },
        :d1 => { :top => 3.9, :bottom => 0.0 },
      }
      conds = []
      @cves[:filters][:cvsses].each do |label|
        values = label_values[label.to_sym]
        if values then
          conds << "COALESCE(c3_base_score,c2_base_score, 0.0) #{values[:bottom] == 0.0 ? '>' : '>='} #{values[:bottom]} AND COALESCE(c3_base_score,c2_base_score, 0.0) <= #{values[:top]}"
        else
          conds << "COALESCE(c3_base_score,c2_base_score, 0.0) = 0.0"
        end
      end
      #表示されるCVSSを対象に検索
      @cves[:all_content] = @cves[:all_content].where(%|
        cve_id IN (
          SELECT cve_id
          FROM sidfmd_vv_cves AS cves_for_cvss_filter
          WHERE
            id IN (
              SELECT id
              FROM sidfmd_vv_cves AS cves_for_cvss_filter2
              WHERE cves_for_cvss_filter.cve_id = cves_for_cvss_filter2.cve_id
              ORDER BY
                CASE
                WHEN c3_update_datetime IS NOT NULL THEN
                  100
                WHEN c2_update_datetime IS NOT NULL THEN
                  200
                ELSE
                  300
                END,
                c3_update_datetime DESC,
                c2_update_datetime DESC,
                id DESC
              LIMIT 1
            )
            AND
            id IN (
              SELECT id
              FROM sidfmd_vv_cves AS cves_for_cvss_filter3
              WHERE #{conds.join(' OR ')}
            )
        )
      |)
    end

    #CVSS指標検索
    if @cves[:filters][:cvss_vector].size > 0 then
      if @cves[:filters][:cvss_vector][:is_target_unregistered] then
        vector_condition = %|
          c2_av IS NULL AND
          c2_ac IS NULL AND
          c2_au IS NULL AND
          c2_c IS NULL AND
          c2_i IS NULL AND
          c2_a IS NULL AND
          c3_av IS NULL AND
          c3_ac IS NULL AND
          c3_pr IS NULL AND
          c3_ui IS NULL AND
          c3_s IS NULL AND
          c3_c IS NULL AND
          c3_i IS NULL AND
          c3_a IS NULL
        |
      else
        cvss_vector_items = []
        @cves[:filters][:cvss_vector][:cvss_v2].each do |vector, value|
          if value then
            cvss_vector_items << "c2_#{vector.to_s.downcase} = '#{value}'"
          end
        end
        @cves[:filters][:cvss_vector][:cvss_v3].each do |vector, value|
          if value then
            cvss_vector_items << "c3_#{vector.to_s.downcase} = '#{value}'"
          end
        end
        vector_condition = cvss_vector_items.join(' AND ')
      end
      #表示されるCVSS指標を対象に検索
      @cves[:all_content] = @cves[:all_content].where(%|
        cve_id IN (
          SELECT cve_id
          FROM sidfmd_vv_cves AS cves_for_vector_filter
          WHERE
            id IN (
              SELECT id
              FROM sidfmd_vv_cves AS cves_for_vector_filter2
              WHERE cves_for_vector_filter.cve_id = cves_for_vector_filter2.cve_id
              ORDER BY
                CASE
                WHEN c3_update_datetime IS NOT NULL THEN
                  100
                WHEN c2_update_datetime IS NOT NULL THEN
                  200
                ELSE
                  300
                END,
                c3_update_datetime DESC,
                c2_update_datetime DESC,
                id DESC
              LIMIT 1
            )
            AND
            id IN (
              SELECT id
              FROM sidfmd_vv_cves AS cves_for_vector_filter3
              WHERE #{vector_condition}
            )
        )
      |)
    end

    #脆弱性検索
    if @cves[:filters][:vuls].size > 0 then
      kwd = @cves[:filters][:vuls].gsub('/', '')
      if kwd.size > 0 then
        if /\A\s*(\d+)\s*\z/ =~ kwd then
          html_id = Regexp.last_match(1)
          @cves[:all_content] = @cves[:all_content].where(
            :cve_id => SidfmdVvCve.joins(:sidfmd_vul_view).where(%|
                         (sidfmd_vul_views.html_id = ? OR
                         sidfmd_vul_views.title ILIKE ?)#{(@role_admin || @user_host_ids.size == 0) ? "" : " AND sidfmd_vul_views.id IN (SELECT sidfmd_vul_view_id FROM sidfmd_vv_scsids WHERE sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}])))"}
                       |, html_id.to_i, "%#{ActiveRecord::Base.send(:sanitize_sql_like, html_id.to_s.strip)}%")
                       .select(:cve_id)
          )
        else
          @cves[:all_content] = @cves[:all_content].where(
            :cve_id => SidfmdVvCve.joins(:sidfmd_vul_view).where(%|
                         sidfmd_vul_views.title ILIKE ?#{(@role_admin || @user_host_ids.size == 0) ? "" : " AND sidfmd_vul_views.id IN (SELECT sidfmd_vul_view_id FROM sidfmd_vv_scsids WHERE sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}])))"}
                       |, "%#{ActiveRecord::Base.send(:sanitize_sql_like, kwd.strip)}%")
                       .select(:cve_id)
          )
        end
      else
        #/で検索すると、/でtitleが連結されているため、該当してしまう可能性がある
        #/は無効な文字として該当させない
        @cves[:all_content] = @cves[:all_content].where(:id => 0)
      end
    end

    #ホスト検索
    if @cves[:filters][:hosts].size > 0 then
      @cves[:all_content] = @cves[:all_content].where(
        :cve_id =>
          SidfmdVvCve.joins("
            INNER JOIN sidfmd_vv_scsids ON sidfmd_vv_cves.sidfmd_vul_view_id = sidfmd_vv_scsids.sidfmd_vul_view_id
            INNER JOIN sidfmd_host_views ON sidfmd_vv_scsids.sidfmd_host_id = sidfmd_host_views.sidfmd_host_id
          ")
          .where("(sidfmd_host_views.hostname ILIKE ? OR sidfmd_host_views.display_name ILIKE ?)#{(@role_admin || @user_host_ids.size == 0) ? "" : " AND sidfmd_vv_scsids.sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}]))"}",
            "%#{ActiveRecord::Base.send(:sanitize_sql_like, @cves[:filters][:hosts])}%", "%#{ActiveRecord::Base.send(:sanitize_sql_like, @cves[:filters][:hosts])}%"
          )
          .select(:cve_id)
      )
    end
  end

  QUERY_CVSS_VECTOR_ITEM_SPLITER = ':'
  QUERY_CVSS_VECTOR_COLUMN_SPLITER = '|'
  QUERY_CVSS_VECTORS_SPLITER = '&'

  def create_all_content
    @cves[:all_content] = SidfmdVvCve
    unless @role_admin then
      @cves[:all_content] = @cves[:all_content].where(
        :sidfmd_vul_view_id => SidfmdVvHost.where(:sidfmd_host_id => @user_host_ids).select(:sidfmd_vul_view_id)
      )
    end
  end

  def create_content(cve_ids, data_for = 'summary_table')
    cves = SidfmdVvCve.where(:cve_id => cve_ids)
    unless @role_admin then
      cves = cves.where(
        :sidfmd_vul_view_id => SidfmdVvHost.where(:sidfmd_host_id => @user_host_ids).select(:sidfmd_vul_view_id)
      )
    end
    cves = cves
    .group(:cve_id)
    .select(:cve_id)
    .select("
      (
        SELECT COALESCE(c3_base_score,c2_base_score, 0.0) /* SidfmdIssue#create_all_contentを参照 */ 
        FROM sidfmd_vv_cves AS cves_for_latest_cvss_score
        WHERE cves_for_latest_cvss_score.cve_id = sidfmd_vv_cves.cve_id
        ORDER BY
          CASE
          WHEN c3_update_datetime IS NOT NULL THEN
            100
          WHEN c2_update_datetime IS NOT NULL THEN
            200
          ELSE
            300
          END,
          c3_update_datetime DESC,
          c2_update_datetime DESC,
          id DESC
        LIMIT 1
      ) AS latest_cvss_score
    ")
    .select("
      (
        SELECT (
          CASE WHEN c2_av IS NOT NULL THEN
            ARRAY_TO_STRING(
              ARRAY['AV#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c2_av, ''), 'AC#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c2_ac, ''), 'Au#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c2_au, ''), 'C#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c2_c, ''), 'I#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c2_i, ''), 'A#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c2_a, '')], '#{QUERY_CVSS_VECTOR_COLUMN_SPLITER}'
            )
          ELSE
            ''
          END
        ) || '#{QUERY_CVSS_VECTORS_SPLITER}' ||
        (
          CASE WHEN c3_av IS NOT NULL THEN
            ARRAY_TO_STRING(
              ARRAY['AV#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c3_av, ''), 'AC#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c3_ac, ''), 'PR#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c3_pr, ''), 'UI#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c3_ui, ''), 'S#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c3_s, ''), 'C#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c3_c, ''), 'I#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c3_i, ''), 'A#{QUERY_CVSS_VECTOR_ITEM_SPLITER}' || COALESCE(c3_a, '')], '#{QUERY_CVSS_VECTOR_COLUMN_SPLITER}'
            )
          ELSE
            ''
          END
        )
        FROM sidfmd_vv_cves AS cves_for_latest_vectors
        WHERE cves_for_latest_vectors.cve_id = sidfmd_vv_cves.cve_id
        ORDER BY
          CASE
          WHEN c3_update_datetime IS NOT NULL THEN
            100
          WHEN c2_update_datetime IS NOT NULL THEN
            200
          ELSE
            300
          END,
          c3_update_datetime DESC,
          c2_update_datetime DESC,
          id DESC
        LIMIT 1
      ) AS latest_vectors
    ")

    case data_for
    when 'summary_table' then
      cves = cves.select("
        COUNT(DISTINCT sidfmd_vul_view_id) AS vuls_count
      ")
      .select("
        (
          SELECT ARRAY[CAST(sidfmd_vul_views.html_id AS text), sidfmd_vul_views.title, sidfmd_vul_views.short_description] /* 脆弱性タイトルと簡易説明の使用文字列に制限がないため(区切り文字が使えないため)、ARRAYで取得する */
          FROM sidfmd_vul_views INNER JOIN sidfmd_vv_cves AS cves_for_vuls ON sidfmd_vul_views.id = cves_for_vuls.sidfmd_vul_view_id
          WHERE cves_for_vuls.cve_id = sidfmd_vv_cves.cve_id #{(@role_admin || @user_host_ids.size == 0) ? "" : "AND sidfmd_vul_views.id IN (SELECT sidfmd_vul_view_id FROM sidfmd_vv_scsids WHERE sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}])))"}
          ORDER BY abs(sidfmd_vul_views.html_id)
          LIMIT 1
        ) AS first_vul
      ")
      .select("
        (
          SELECT ARRAY[CAST(sidfmd_vv_hosts.sidfmd_host_id AS text), sidfmd_vv_hosts.display_name, sidfmd_vv_hosts.hostname, COALESCE(sidfmd_vv_hosts.ext_name, '')]
          FROM sidfmd_vv_hosts INNER JOIN sidfmd_vv_cves AS cves_for_hosts ON sidfmd_vv_hosts.sidfmd_vul_view_id = cves_for_hosts.sidfmd_vul_view_id
          WHERE cves_for_hosts.cve_id = sidfmd_vv_cves.cve_id #{(@role_admin || @user_host_ids.size == 0) ? "" : "AND sidfmd_vv_hosts.sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}]))"}
          ORDER BY LOWER(sidfmd_vv_hosts.display_name COLLATE \"C\")
          LIMIT 1
        ) AS first_host
      ")
      .select("
        (
          SELECT COUNT(DISTINCT sidfmd_host_id)
          FROM sidfmd_vv_scsids INNER JOIN sidfmd_vv_cves AS cves_for_hosts_count ON sidfmd_vv_scsids.sidfmd_vul_view_id = cves_for_hosts_count.sidfmd_vul_view_id
          WHERE cves_for_hosts_count.cve_id = sidfmd_vv_cves.cve_id #{(@role_admin || @user_host_ids.size == 0) ? "" : "AND sidfmd_vv_scsids.sidfmd_host_id IN (SELECT unnest(ARRAY[#{@user_host_ids.join(', ')}]))"}
        ) AS hosts_count
      ")
    when 'download'
      # nothing to do 
    end
    cves
  end

  def create_cvss_vectors_data
    @cvss_v2_vectors = SidfmdIssueCve.get_cvss_v2_vectors
    @cvss_v3_vectors = SidfmdIssueCve.get_cvss_v3_vectors
  end

  def create_cve_cvss_vectors(cve)
    create_vector_values = -> (item){
      item.split("#{QUERY_CVSS_VECTOR_COLUMN_SPLITER}").map{|columns|
        vector_and_value = columns.split("#{QUERY_CVSS_VECTOR_ITEM_SPLITER}")
      }.to_h
    }
    latest_vectors = cve.latest_vectors.split("#{QUERY_CVSS_VECTORS_SPLITER}")
    cvss_v2_vectors = latest_vectors.index("#{QUERY_CVSS_VECTORS_SPLITER}") == 0 ? nil : latest_vectors[0] #ない場合は空文字なのでこのようにして判別
    cvss_v3_vectors = cvss_v2_vectors ? latest_vectors[1] : latest_vectors[0] #cvss_v2_vectorsがない場合はlatest_vectorsのsizeは1か0
    ret = {
      :cvss_v2_vectors => (cvss_v2_vectors ? create_vector_values.(cvss_v2_vectors) : nil),
      :cvss_v3_vectors => (cvss_v3_vectors ? create_vector_values.(cvss_v3_vectors) : nil)
    }
  end

  def create_summary_cve_html(cve, **opts)
    html_ids = SortedSet.new
    vuls = []
    vul = cve.first_vul
    vul = {
      :html_id => vul[0].to_i,
      :title => vul[1],
      :short_description => vul[2]
    }
    host = cve.first_host
    host = {
      :sidfmd_host_id => host[0].to_i,
      :display_name => host[1],
      :hostname => host[2],
      :ext_name => host[3].size > 0 ? host[3] : nil,
    }
    html_ids = html_ids.to_a

    cvss_vectors = create_cve_cvss_vectors(cve)
    cvss_v2_vector_texts = []
    cvss_v3_vector_texts = []
    if cvss_vectors[:cvss_v2_vectors] then
      cvss_v2_vector_texts = cvss_vectors[:cvss_v2_vectors].map{|vector, value|
        target_vector = @cvss_v2_vectors[vector]
        "#{target_vector['item_name']}: #{target_vector['values'][value]}"
      }
    end
    if cvss_vectors[:cvss_v3_vectors] then
      cvss_v3_vector_texts = cvss_vectors[:cvss_v3_vectors].map{|vector, value|
        target_vector = @cvss_v3_vectors[vector]
        "#{target_vector['item_name']}: #{target_vector['values'][value]}"
      }
    end
 
    data = {
      :chkbox => view_context.create_summary_cve_chkbox(cve.cve_id, opts[:checked]),
      :cve_id => view_context.create_summary_cve_id(cve.cve_id),
      :cvss => view_context.create_summary_cve_cvss(html_ids, (cve.latest_cvss_score.to_f > 0.0 ? cve.latest_cvss_score.to_f : nil), cvss_v2_vector_texts, cvss_v3_vector_texts),
      :vuls => view_context.create_summary_cve_vuls(vul, cve.vuls_count, cve.cve_id),
      :hosts => view_context.create_summary_cve_hosts(host, cve.hosts_count, cve.cve_id)
      #:sidfm_ids => view_context.create_summary_cve_sidfm_ids(html_ids)
    }
  end

  def create_cve_vuls(cve_ids)
    @cve_vuls = {}
    cves_vuls = SidfmdVvCve.joins(:sidfmd_vul_view)
      .where(:cve_id => cve_ids)
      .order("abs(sidfmd_vul_views.html_id)")
      .distinct
      .select(
        :cve_id,
        "sidfmd_vul_views.html_id AS orig_html_id",
        "abs(sidfmd_vul_views.html_id) AS html_id",
        "sidfmd_vul_views.title",
        "sidfmd_vul_views.short_description"
      )
    unless @role_admin then
      cves_vuls = cves_vuls.where(:sidfmd_vul_views => { :id => SidfmdVvScsid.where(:sidfmd_host_id => @user_host_ids).select(:sidfmd_vul_view_id) })
    end
    cves_vuls.each do |vul|
      @cve_vuls[vul.cve_id] = [] unless @cve_vuls[vul.cve_id]
      @cve_vuls[vul.cve_id] << {
        :html_id => vul.html_id,
        :title => SidfmdVulView.display_title(vul.title),
        :short_description => vul.short_description,
        :url => url_for(
          :controller => :sidfm_vulnerability,
          :action => :index,
          :id => vul.orig_html_id,
        )
      }
    end
  end

  def create_cve_hosts(cve_ids)
    @cve_hosts = {}
    cve_hosts = SidfmdVvCve.joins("
      INNER JOIN sidfmd_vv_hosts ON sidfmd_vv_hosts.sidfmd_vul_view_id = sidfmd_vv_cves.sidfmd_vul_view_id
    ")
    .where(:cve_id => cve_ids)
    .distinct
    .select(
      :cve_id,
      "sidfmd_vv_hosts.sidfmd_host_id",
      "sidfmd_vv_hosts.display_name",
      "sidfmd_vv_hosts.hostname",
      "sidfmd_vv_hosts.ext_name"
    )
    unless @role_admin then
      cve_hosts = cve_hosts.where(:sidfmd_vv_hosts => { :sidfmd_host_id => @user_host_ids })
    end
    cve_hosts.each do |host|
      @cve_hosts[host.cve_id] = [] unless @cve_hosts[host.cve_id]
      @cve_hosts[host.cve_id] << {
        :sidfmd_host_id => host.sidfmd_host_id,
        :hostname => host.hostname,
        :ext_name => host.ext_name,
        :display_name => host.display_name,
        :url => url_for(
          :controller => :sidfm_host,
          :action => :index,
          :id => host.sidfmd_host_id
        )
      }
    end
    @cve_hosts.each do |cve_id, hosts|
      hosts.uniq!{|host| host[:sidfmd_host_id] }
      hosts.sort_by!{|host| host[:display_name].downcase }
    end
  end

  public

  def index
    create_cvss_vectors_data
    parse_params
    check_filters
    @page_name = 'CVE一覧'
  end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    parse_params
    @cves[:per_page] = f_params[:per_page].to_i
    create_all_content
    check_filters
    filter_content
    sorted_cve_ids = get_sorted_cve_ids(@cves[:all_content], @cves[:per_page], @cves[:offset])
    @cves[:content] = create_content(sorted_cve_ids)
    cve_ids = @cves[:all_content].distinct.pluck(:cve_id)
    @cves[:content] = @cves[:content].sort_by{|cve| sorted_cve_ids.index(cve.cve_id)}
    create_cvss_vectors_data
 
    ret = {
      :rows => @cves[:content].map{|cve|
        create_summary_cve_html(
          cve,
          {
            :checked => @cves[:checked]
          }
        )
      },
      :total => cve_ids.size,
      :all_ids => cve_ids,
      :sidfm_roles => @sidfm_roles,
      :user_cats => @user_category_ids
    }
    render :layout => false, :json => ret
  end

  def read_download_data
    GC.start
    f_params = params.permit(:target, :ids, :executed_size, :process, :order, :filters)
    if !f_params[:ids] or !f_params[:target] or !f_params[:order] or !f_params[:filters] or !f_params[:executed_size] or !f_params[:process] then
      logger.error('params error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    executed_size = f_params[:executed_size].to_i

    begin
      cve_ids = JSON.parse(f_params[:ids])
      filters = JSON.parse(f_params[:filters], symbolize_names: true)
    rescue JSON::ParserError => e
      logger.error('json parser error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    if ![ 'json', 'tsv' ].include?(f_params[:target]) then
      logger.error('file target error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    max_download_size = Sidfm.get_cve_max_download_size
    @cves = {}
    @cves[:order] = f_params[:order] || 'cve_id_asc'
    @cves[:filters] = JSON.parse((f_params[:filters] || '{}'), symbolize_names: true)
    create_all_content
    
    #最初にデータをソートして、２回目以降(f_params[:process] == 'init' でない時)は
    #get_sorted_cve_idsを呼ばずに、最初にソートしたidsで順次データを処理する
    #２回目以降のidsは、jsの方でmax_download_sizeに指定した分のidsに限定される
    if f_params[:process] == 'init' then
      check_filters
      filter_content
    end
    cve_ids = @cves[:all_content].distinct.pluck(:cve_id) if cve_ids.size == 0 #サマリー表でチェックがない時
    @cves[:all_content] = @cves[:all_content].where(:cve_id => cve_ids)
    sorted_cve_ids = (f_params[:process] == 'init' ? get_sorted_cve_ids(@cves[:all_content], max_download_size, 0) : cve_ids)
    @cves[:content] = create_content(sorted_cve_ids)
    create_cve_vuls(sorted_cve_ids)    
    create_cve_hosts(sorted_cve_ids)

    case f_params[:target]
    when 'json' then
      if f_params[:process] == 'init' then
        separate_char_index = @cves[:order].rindex('_')
        col_name = @cves[:order].slice(0..separate_char_index - 1)
        order_type = @cves[:order].slice(separate_char_index + 1..@cves[:order].size - 1)
        init_json_summary_table_progress_download(
          :ids => cve_ids,
          :order_col_name => col_name,
          :order_type => order_type,
          :filters => filters,
          :max_download_size => max_download_size,
          :process_ids => sorted_cve_ids
        )
      else
        create_json_data = Proc.new{|cve_id|
          cve = @cves[:content].find_by(:cve_id => cve_id)
          next unless cve
          item = {}
          item[:cve_id] = cve.cve_id
          item[:cvss] = (cve.latest_cvss_score.to_f > 0.0 ? sprintf('%.1f', cve.latest_cvss_score.to_f) : '')
          item[:cvss_v2] = ''
          item[:cvss_v3] = ''
          cvss_vectors = create_cve_cvss_vectors(cve)
          if cvss_vectors[:cvss_v2_vectors] then
            item[:cvss_v2] = cvss_vectors[:cvss_v2_vectors].map{|vector, value|"#{vector}:#{value}"}.join('/')
          end
          if cvss_vectors[:cvss_v3_vectors] then
            item[:cvss_v3] = cvss_vectors[:cvss_v3_vectors].map{|vector, value| "#{vector}:#{value}"}.join('/')
          end
          item[:vulnerabilities] = @cve_vuls[cve.cve_id] || []
          item[:hosts] = @cve_hosts[cve.cve_id] || []
          item
        }
        json_tsv_summary_table_progress_download(sorted_cve_ids, create_json_data, executed_size)
      end
    when 'tsv' then
      header = [ 'CVE ID', 'CVSS', 'CVSS V2', 'CVSS V3', '関連する脆弱性', '関連するホスト' ]
      if f_params[:process] == 'init' then
        init_tsv_summary_table_progress_download(cve_ids, header, max_download_size, sorted_cve_ids)
      else
        create_tsv_data = Proc.new{|cve_id|
          cve = @cves[:content].find_by(:cve_id => cve_id)
          next unless cve
          item = []
          item << cve.cve_id
          item << (cve.latest_cvss_score.to_f > 0.0 ? sprintf('%.1f', cve.latest_cvss_score.to_f) : '')
          cvss_2 = ''
          cvss_3 = ''
          cvss_vectors = create_cve_cvss_vectors(cve)
          if cvss_vectors[:cvss_v2_vectors] then
            cvss_v2 = cvss_vectors[:cvss_v2_vectors].map{|vector, value|"#{vector}:#{value}"}.join('/')
          end
          if cvss_vectors[:cvss_v3_vectors] then
            cvss_v3 = cvss_vectors[:cvss_v3_vectors].map{|vector, value| "#{vector}:#{value}"}.join('/')
          end
          item << cvss_v2
          item << cvss_v3
          vuls = @cve_vuls[cve.cve_id] || []
          item << vuls.map{|vul| escape_category_tsv("[#{vul[:html_id]}]#{vul[:title]}") }.join(',')
          hosts = @cve_hosts[cve.cve_id] || []
          item << hosts.map{|host| escape_category_tsv(host[:display_name]) }.join(',')
          item
        }
        json_tsv_summary_table_progress_download(sorted_cve_ids, create_tsv_data, executed_size)
      end
    end
  end

  def download
    f_params = params.fetch(:download, {}).permit(:format)
    case f_params[:format]
    when 'json' then
      send_summary_table_json_file("vmlist-cves-json-#{Time.current.strftime('%Y%m%d%H%M%S')}.json")
    when 'tsv' then
      send_summary_table_tsv_file("vmlist-cves-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
    end
  end

end
