# -*- coding: utf-8 -*-
require 'sidfm_controller_support'
require 'sidfm_users_controller_support'

class SidfmUsersController < ApplicationController

  unloadable

  layout 'sidfm'

  self.main_menu = false

  before_action :check_roles

  protect_from_forgery with: :exception
  helper :sidfm

  private

  include SidfmControllerSupport, SidfmUsersControllerSupport
  before_action :update_session_updated_at
  before_action :create_common_data
  before_action :get_sidfm_roles
  before_action :prevent_access
  before_action :check_roles
  before_action :get_admin_ids
  before_action :get_vm_version
  before_action :clear_import_session, only: [ :read_import_data ]
  before_action :clear_export_session, only: [ :index ]
  before_action -> { get_summary_table_lines('users') }

  # 権限チェック
  def prevent_access
    get_sidfm_roles
    unless @sidfm_user 
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end 
  end

  def check_roles
    if !@sidfm_roles.include?(:view) then
      redirect_to :action => :index, :notice => '許可されていない操作です。'
    end
    if !@sidfm_roles.include?(:user_admin) and ![ 'index', 'modify', 'test_email', 'read_download_data', 'download', 'import_apply', 'api_get_display_summary_content' ].include?(action_name) then
      redirect_to :action => :index, :notice => '許可されていない操作です。'
    end
  end

  def get_admin_ids
    @sidfm_admin = Role.find_by(name: 'SIDfmVM管理者').id
    @admin = Role.find_by(name: '管理者').id
  end

  def export_import_executable?
    @sidfm_roles.include?(:user_admin)
  end

  FILTER_KEYS = {}

  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:search_login] = ''
      FILTER_KEYS[:search_name] = ''
      FILTER_KEYS[:search_auth] = ''
      FILTER_KEYS[:search_users_auth] = ''
      FILTER_KEYS[:search_category] = '0'
      FILTER_KEYS[:search_group] = '0'
      FILTER_KEYS[:search_last_login_on] = []
      FILTER_KEYS[:search_mail_notification] = ''
      FILTER_KEYS[:search_locked_users] = ''
    end
  end

  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @users_paginate[:filters].has_key?(key) then
        @users_paginate[:filters][key] = val
      end
    end
    @default_filters = FILTER_KEYS
  end

  #ソート処理
  def sort_for_users
    last_index = @users_paginate[:order].rindex('_')
    col = @users_paginate[:order].slice(0..last_index - 1)
    order = @users_paginate[:order].slice(last_index + 1..@users_paginate[:order].size - 1)
    case col
    when 'login' then
      col = 'login'
    when 'name' then
      col = 'lastname'
    when 'authentication' then
      col = 'auth_source_id'
    when 'mail_notification' then
      col = 'mail_notification'
    when 'last_login_on' then
      col = 'last_login_on'
    when 'lock' then
      col = 'lock'
    end

    @users_paginate[:content] = sort_users_content(
      :content => @users_paginate[:content],
      :order => "#{col}_#{order}",
      :sidfm_roles => @sidfm_roles
    )
  end
  
  #検索処理
  def search_items
    filters = {}
    filters[:login] = @users_paginate[:filters][:search_login]
    filters[:name] = @users_paginate[:filters][:search_name]
    filters[:auth] =  @users_paginate[:filters][:search_auth]
    filters[:role] = @users_paginate[:filters][:search_users_auth] 
    filters[:category] = @users_paginate[:filters][:search_category]
    filters[:group] = @users_paginate[:filters][:search_group]
    filters[:mail_notification] = @users_paginate[:filters][:search_mail_notification]
    filters[:last_login_on] = @users_paginate[:search_last_login_on]
    filters[:locked] = @users_paginate[:search_locked_users]

    @users_paginate[:content] = filter_users_content(
      :content => @users_paginate[:content],
      :filters => filters,
      :sidfm_project_id => @sidfm_project.id,
      :current_user => User.current,
      :sidfm_roles => @sidfm_roles,
    )

  end
  
  def create_all_content
    filtered_params = params.fetch(:pages, {}).permit(:offset, :per_page, :order, :filters, :checked, :all_checked)
    @users_paginate = {}
    @users_paginate[:per_page] = (@summary_table_lines || filtered_params[:per_page] || 10).to_i
    @users_paginate[:offset] = (filtered_params[:offset] || 0).to_i
    @users_paginate[:filters] = JSON.parse(filtered_params[:filters] || '{}').symbolize_keys
    @users_paginate[:order] = (filtered_params[:order] || 'login_asc')
    #ソート・検索機能
    @users_paginate[:search_login] = @users_paginate[:filters][:search_login] || ''
    @users_paginate[:search_name] = @users_paginate[:filters][:search_name] || ''
    @users_paginate[:search_auth] = @users_paginate[:filters][:search_auth] || ''
    @users_paginate[:search_users_auth] = @users_paginate[:filters][:search_users_auth] || ''
    @users_paginate[:search_category] = @users_paginate[:filters][:search_category] || '0'
    @users_paginate[:search_last_login_on] = @users_paginate[:filters][:search_last_login_on] || []
    @users_paginate[:search_mail_notification] = @users_paginate[:filters][:search_mail_notification] || ''
    @users_paginate[:search_locked_users] = @users_paginate[:filters][:search_locked_users] || ''
    @users_paginate[:search_group] = @users_paginate[:filters][:search_group] || '0'

    #チェックボックス
    @users_paginate[:checked] = JSON.parse(filtered_params[:checked] || '[]')
    @users_paginate[:all_checked] = ((filtered_params[:all_checked] || '') == 'true')

    init_filter_keys
    check_filters

    @users_categories = 
      if @sidfm_roles.include?(:admin) then
        SidfmdCategory.where(:is_system => false).order('LOWER(name COLLATE "C")')
      else
        if @sidfm_roles.include?(:use) then
          User.current.categories.order('LOWER(name COLLATE "C")')
        else
          nil
        end
      end
    @current_user_categories = @sidfm_roles.include?(:use) ? @users_categories.pluck(:id) : []
    @categories = @sidfm_roles.include?(:use) ? Hash[@users_categories.map{ |cat| [cat.id, cat.name]}] : {}
    @users_paginate[:content] = User.create_all_content(
      :sidfm_roles => @sidfm_roles,
      :current_user_id => User.current.id,
      :user_category_ids => @current_user_categories,
      :sidfm_project_id => @sidfm_project.id
    ) 
  end
  
  def create_summary_user_html(user, **opts)
    auth_source = user.auth_source
    role = user.roles.where(:members => { :project_id => @sidfm_project.id}).first
    role_id = (role ? role.id : -1)
    get_role(user)
    data = {
      :chkbox => view_context.create_summary_user_chkbox(user, opts[:checked]),
      :login => view_context.create_summary_user_login(user.login),
      :name => view_context.create_summary_user_name(user),
      :authentication => view_context.create_summary_user_authentication(auth_source ? auth_source.name : "パスワード認証"),
      :authority => view_context.create_summary_user_authority(
        role ? role.name : (user.admin == true ? 'Redmine管理者' : '')
      ),
      :categories => view_context.create_summary_user_categories(
        :user => user,
        :categories => @user_sidfmd_categories[user.id] || [],
        :user_sidfm_roles => @users_perms[user],
        :user_role_id => role_id,
        :current_user => opts[:current_user],
        :current_user_sidfm_roles => @sidfm_roles,
      ),
      :category_groups => (@sidfm_roles.include?(:user_admin) ?
        view_context.create_summary_user_category_groups(
          :user => user,
          :user_sidfm_roles => @users_perms[user],
          :user_role_id => role_id,
          :user_cat_groups => (@user_cat_groups[user.id] || []),
          :current_user => opts[:current_user],
          :current_user_sidfm_roles => @sidfm_roles,
        ) :
        ''
      ),
      :last_login_on => view_context.create_summary_user_last_login_on(user.last_login_on),
      :mail_notification => view_context.create_summary_user_mail_notification(user.mail_notification),
      :lock => view_context.create_summary_user_lock(user.status),
      :operation => view_context.create_summary_user_operation(
        :user => user,
        :user_sidfm_roles => @users_perms[user],
        :user_role_id => role_id,
        :user_auth_id => (auth_source ? auth_source.id : -1),
        :current_user => opts[:current_user],
        :current_user_sidfm_roles => @sidfm_roles,
      )
    }
  end

  def create_users_role_names(content)
    #ユーザの権限。カテゴリ一括更新の時、管理者権限のユーザを対象から除外するために使用する
    roles_and_users = Role.joins(:member_roles).joins(:members).joins("LEFT OUTER JOIN users ON users.id = members.user_id")
    @users_role_names = {}
    content.each do |user|
      user_role = roles_and_users.where("users.id = ?", user).first
      @users_role_names[user.id] = user_role.name if user_role
    end
  end

  #ユーザ毎のカテゴリ
  def create_user_sidfmd_categories(user_ids)
    @user_sidfmd_categories = {}
    user_cats = SidfmdCategoriesUser.joins(:sidfmd_category)
      .where(:user_id => user_ids)
      .select(
        :user_id,
        :sidfmd_category_id,
        'sidfmd_categories.name',
        'sidfmd_categories.desc'
      )
      .order('LOWER(sidfmd_categories.name COLLATE "C")')
    unless @sidfm_roles.include?(:admin) then
      user_cats = user_cats.where(:sidfmd_category_id => User.current.categories.select(:id))
    end

    user_cats.each do |user_cat|
      @user_sidfmd_categories[user_cat.user_id] = [] unless @user_sidfmd_categories[user_cat.user_id]
      @user_sidfmd_categories[user_cat.user_id] << {
        :id => user_cat.sidfmd_category_id,
        :name => user_cat.name,
        :desc => user_cat.desc
      }
    end
  end

  #ユーザ毎のカテゴリグループを生成する
  def create_user_cat_groups(user_ids)
    @user_cat_groups = {}
    user_cat_groups = SidfmdCatGroup.joins(:users).where(:users => { :id => user_ids }).select(
      :id,
      :name,
      'users.id as user_id'
    )
    .order('LOWER(sidfmd_cat_groups.name COLLATE "C")')
    unless @sidfm_roles.include?(:admin) then
      user_cat_groups = user_cat_groups.where(
        :sidfmd_cat_groups => { :id => SidfmdCatGroupsUser.where(:user_id => User.current.id).select(:sidfmd_cat_group_id) }
      )
    end
    user_cat_groups.each do |user_cat_group|
      @user_cat_groups[user_cat_group.user_id] = [] unless @user_cat_groups[user_cat_group.user_id]
      @user_cat_groups[user_cat_group.user_id] << {
        :id => user_cat_group.id,
        :name => user_cat_group.name,
      }
    end
  end

  # ユーザのカテゴリグループ
  def get_current_user_cat_groups
    cat_group = (
      @sidfm_roles.include?(:admin) ?
      SidfmdCatGroup.all :
      User.current.sidfmd_cat_groups
    ).order('LOWER(name COLLATE "C")')
  end

  public

  LASTNAME_LENGTH_LIMIT = User::LASTNAME_LENGTH_LIMIT 
  FIRSTNAME_LENGTH_LIMIT = User::FIRSTNAME_LENGTH_LIMIT
 
  def index
    if session[:clear_session] then
      clear_import_session
      redirect_to :action => :index
    end
    #インポートエラーがあればエラー内容を表示する
    @import_notices = session[:import_notices]
    clear_import_session
 
    create_all_content
    #ユーザのカテゴリの概要
    @categories_desc = @users_categories ? @users_categories.map{|cat| [ cat.id, cat.desc ] }.to_h : {}

    #権限とプロジェクトの取得
    @auths = AuthSource.all
    @roles_for_users = []
    Role.where(:name => ['SIDfmVM管理者','SIDfmVM報告者', 'SIDfmVM利用者', 'SIDfmVM閲覧者']).each do |role|
      if role.name == 'SIDfmVM管理者' then
        @roles_for_users[0] = role
      elsif role.name == 'SIDfmVM報告者' then
        @roles_for_users[1] = role
      elsif role.name == 'SIDfmVM利用者' then
        @roles_for_users[2] = role
      elsif role.name == 'SIDfmVM閲覧者' then
        @roles_for_users[3] = role    
      end
    end                  
    @sidfm_view_id = Role.find_by(name: 'SIDfmVM閲覧者').id
    #メール通知設定
    @mail_notification = {
      "無効" => "false",
      "有効" => "true"
    }

    @force_password_label = '次回ログイン時にパスワード変更を強制'

    @password_policy_settings = Sidfm.get_password_policy_settings
    @openid_connect_auth_id = get_openid_connect_auth_id
    @current_user_cat_groups = get_current_user_cat_groups
    @all_categories = (@sidfm_roles.include?(:admin) ?
      SidfmdCategory :
      User.current.categories
    )
    .where(:is_system => false)
    .order('LOWER(name COLLATE "C")')

    #ブラウザバックかどうか判断するための処理
    #if !session['chkbok_ids'] then
    #  session['chkbok_ids'] = {}
    #end
    #if !session['chkbok_ids']['user'] then
    #  session['chkbok_ids']['user'] = {}
    #end
    #session['chkbok_ids']['user']['is_browser_back'] = false
  end

  def create
    filtered_params = params.permit(:userid, :login, :pw, :pwconf, :lastname, :firstname, :email, :auth, :pj_auth, :mail_notification, :force_change_password, :user_lock, :category_ids, :cat_group_ids)
    begin
      category_ids = JSON.parse(filtered_params[:category_ids])
      cat_group_ids = JSON.parse(filtered_params[:cat_group_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '不正な操作です' } and return
    end

    category_ids = category_ids.map(&:to_i)
    cat_group_ids = cat_group_ids.map(&:to_i)
    unless @sidfm_roles.include?(:admin)
      category_ids = category_ids & User.current.categories.pluck(:id)
      cat_group_ids = cat_group_ids & User.current.sidfmd_cat_groups.pluck(:id)
      if category_ids.size == 0 && cat_group_ids.size == 0 then
         render :layout => false, :json => { :err_message => '不正な操作です' } and return
      end
    end

    begin
      checked = validate_fetch_params('cre', filtered_params)
      if !checked[:success] then
        render :layout => false, :json => { :err_message => checked[:notice] } and return
       end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '不正な操作です' } and return
    end

    res = create_user(
      :login => filtered_params[:login],
      :password => filtered_params[:pw],
      :lastname => filtered_params[:lastname],
      :firstname => filtered_params[:firstname],
      :email => filtered_params[:email],
      :auth_id => filtered_params[:auth].to_i,
      :role_id => filtered_params[:pj_auth].to_i,
      :mail_notification => filtered_params[:mail_notification],
      :must_change_password => filtered_params[:force_change_password],
      :user_lock => filtered_params[:user_lock],
      :category_ids => category_ids,
      :cat_group_ids => cat_group_ids,
      :sidfm_roles => @sidfm_roles,
      :sidfm_project_id => @sidfm_project.id,
      :avaliable_role_names => ROLES_SIDFM
    )
    render :layout => false, :json => { :err_message => res[:success] ? '' : res[:message] }
  end

  def modify
    filtered_params = params.permit(:userid, :login, :pw, :pwconf, :lastname, :firstname, :email, :auth, :pj_auth, :mail_notification, :force_change_password, :user_lock) #:per_page, :filters, :offset, :order 
    begin
      checked = validate_fetch_params('mod', filtered_params)
      if !checked[:success] then
        render :layout => false, :json => { :err_message => checked[:notice] } and return
       end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '不正な操作です' } and return
    end
    res = update_user(
      :user_id => filtered_params[:userid],
      :login => filtered_params[:login],
      :password => filtered_params[:pw],
      :lastname => filtered_params[:lastname],
      :firstname => filtered_params[:firstname],
      :email => filtered_params[:email],
      :auth_id => filtered_params[:auth],
      :role_id => filtered_params[:pj_auth],
      :mail_notification => filtered_params[:mail_notification],
      :must_change_password => filtered_params[:force_change_password],
      :user_lock => filtered_params[:user_lock],
      :sidfm_roles => @sidfm_roles,
      :sidfm_project_id => @sidfm_project.id,
      :avaliable_role_names => ROLES_SIDFM
    )
    render :layout => false, :json => { :err_message => res[:success] ? '' : res[:message] }
  end

  def modify_category 
    filtered_params = params.permit(:userid, :selected_category_ids)#, :offset, :per_page, :filters, :order)
    begin
      p_userid = filtered_params['userid']
      p_categories = JSON.parse(filtered_params['selected_category_ids']).map(&:to_i)
      mod_categories = SidfmdCategory.where(:id => p_categories)
      @users_perms = {}

      user = User.find_by_id(p_userid)
      if user then
        filtered_userids = filter_user_ids([user.id])
        raise Exception if filtered_userids.size == 0 #不正
        unless @sidfm_roles.include?(:admin) then
          unchangeable_categories = user.sidfmd_categories.where.not(:id => User.current.categories.select(:id))
          mod_categories = mod_categories + unchangeable_categories
          mod_categories.uniq!
        end
        User.transaction do
          get_role(user)
          unless user.members.where('project_id = ?', @sidfm_project.id).size == 0 || (!@sidfm_roles.include?(:admin) && user == User.current) then
            user.sidfmd_categories = mod_categories
            user.save
            #関連するルールの削除・変更
            process_rules = User.process_related_rules(
              :sidfm_roles => @sidfm_roles,
              :change_category_user_ids => { p_userid.to_s => (mod_categories.map{|cat| cat.id } + user.cat_group_categories.pluck(:id)).uniq },
              :action => 'category_change'
            )
            if !process_rules then
              logger.error('failed to process rules')
              raise Exception
            end
          else
            render :layout => false, :json => { :err_message => 'このユーザのカテゴリは変更できません。' } and return
          end
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'カテゴリー情報の更新に失敗しました。' } and return  
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def update_category_groups
    f_params = params.permit(:user_id, :group_ids)
    if !f_params[:user_id] || !f_params[:group_ids] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      group_ids = JSON.parse(f_params[:group_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    @users_perms = {}

    begin
      User.transaction do
        user = User.find_by_id(f_params[:user_id].to_i)
        if user then
          filtered_userids = filter_user_ids([user.id])
          raise Exception if filtered_userids.size == 0 #不正
          get_role(user)
          unless @users_perms[user].include?(:admin) || @users_perms[user].size == 0 || (!@sidfm_roles.include?(:admin) && user == User.current) then
            user.update_cat_groups(
              :current_user_id => User.current.id,
              :role_admin => @sidfm_roles.include?(:admin),
              :cat_group_ids => group_ids
            )
          end
          #関連するルールの削除・変更
          user_categoryids = { #グループ変更後のユーザ毎のカテゴリ
            user.id.to_s => User.after_cat_group_change_categories(
              :user_id => user.id,
              :cat_group_ids => group_ids
            )
          }
          unless @sidfm_roles.include?(:admin) then #ユーザに設定されていないのカテゴリグループのカテゴリを入れる
            user_categoryids[user.id.to_s] |= SidfmdCatGroup.limit_user_group_category_ids(user.id, User.current.id)
          end
          
          process_rules = User.process_related_rules(
            :sidfm_roles => @sidfm_roles,
            :change_category_user_ids => user_categoryids,
            :action => 'bulk_category_change'
          )
          if !process_rules then
            logger.error('failed to process rules')
            raise Exception
          end
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'カテゴリグループの更新に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def update_category_groups_bulk
    f_params = params.permit(:user_ids, :group_ids)
    if !f_params[:user_ids] || !f_params[:group_ids] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      user_ids = JSON.parse(f_params[:user_ids])
      group_ids = JSON.parse(f_params[:group_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    @users_perms = {}

    begin
      current_user_id = User.current.id
      user_ids = filter_user_ids(user_ids)
      User.transaction do
        user_categoryids = {} #グループ変更後のユーザ毎のカテゴリ
        user_ids.each do |user_id|
          user = User.find_by_id(user_id.to_i)
          if user then
            user_categoryids[user.id.to_s] = User.after_cat_group_change_categories(
              :user_id => user.id,
              :cat_group_ids => group_ids
            )
            unless @sidfm_roles.include?(:admin) then #ユーザに設定されていないのカテゴリグループのカテゴリを入れる
              user_categoryids[user.id.to_s] |= SidfmdCatGroup.limit_user_group_category_ids(user.id, current_user_id)
            end
            get_role(user)
            unless @users_perms[user].include?(:admin) || @users_perms[user].size == 0 || (!@sidfm_roles.include?(:admin) && user == User.current) then
              user.update_cat_groups(
                :current_user_id => User.current.id,
                :role_admin => @sidfm_roles.include?(:admin),
                :cat_group_ids => group_ids
              )
            end
          end
        end
        #関連するルールの削除・変更
        process_rules = User.process_related_rules(
          :sidfm_roles => @sidfm_roles,
          :change_category_user_ids => user_categoryids, 
          :action => 'bulk_category_change'
        )
        if !process_rules then
          logger.error('failed to process rules')
          raise Exception
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'カテゴリグループの更新に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def update_authenticate_bulk
    filtered_params = params.permit(:userids, :auth)#, :dump, :offset, :per_page, :filters, :order)
    userids = JSON.parse(filtered_params[:userids] || '[]')
    auth = filtered_params[:auth]
    update_users(userids, auth, "authentication")#, offset, per_page, filters, order)
  end

  def update_authority_bulk
    filtered_params = params.permit(:userids, :auth)# :dump, :offset, :per_page, :filters, :order)
    userids = JSON.parse(filtered_params[:userids] || '[]')
    auth = filtered_params[:auth]
    if (!@sidfm_roles.include?(:admin) && auth.to_i == @sidfm_admin) || auth.to_i == @admin then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    admin_users = User.joins("LEFT OUTER JOIN members ON members.project_id = #{@sidfm_project.id} AND members.user_id = users.id").joins("LEFT OUTER JOIN member_roles ON member_roles.member_id = members.id").where("member_roles.role_id = ?", @sidfm_admin)
    admin_users_size = admin_users.size
    bulk_admin_users_size = admin_users.where(id: userids).size
    if auth.to_i != @sidfm_admin && admin_users_size == bulk_admin_users_size then 
      render :layout => false, :json => { :err_message => '管理者を0にはできません。' } and return
    end
    update_users(userids, auth, "authority") #offset, per_page, filters, order)
  end
    
  def update_categories_bulk
    filtered_params = params.permit(:userids, :selected_category_ids)#, :dump, :offset, :per_page, :filters, :order)
    begin
      userids = JSON.parse(filtered_params[:userids])
      selected_category_ids = JSON.parse(filtered_params[:selected_category_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    unless userids or selected_category_ids then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    @users_perms = {}    
    begin
      #関連するルールの削除・変更
      user_categories = {}
      user_accessible_categories = {}
      selected_category_ids = selected_category_ids.map(&:to_i)
      unless @sidfm_roles.include?(:admin) then
        user_category_ids = User.current.categories.pluck(:id)
        userids = filter_user_ids(userids)
        User.where(:id => userids).each do |user|
          unchangeable_categories = user.sidfmd_categories.where.not(:id => user_category_ids).pluck(:id)
          user_categories[user.id.to_s] = (selected_category_ids + unchangeable_categories)
          user_accessible_categories[user.id.to_s] = (selected_category_ids + unchangeable_categories + user.cat_group_categories.pluck(:id))
        end
      else
        User.where(:id => userids).each do |user|
          user_categories[user.id.to_s] = selected_category_ids
          user_accessible_categories[user.id.to_s] = (selected_category_ids + user.cat_group_categories.pluck(:id))
        end
      end
      User.transaction do
        process_rules = User.process_related_rules(
          :sidfm_roles => @sidfm_roles,
          :change_category_user_ids => user_accessible_categories,
          :action => 'bulk_category_change'
        )
        if !process_rules then
          logger.error('failed to process rules')
          raise Exception
        end
        User.where(:id => userids).each{|user|
          get_role(user)
          unless @users_perms[user].size == 0 || (!@sidfm_roles.include?(:admin) && user == User.current) then
            update_categories = SidfmdCategory.where(:id => user_categories[user.id.to_s])
            user.sidfmd_categories = update_categories
          else 
            render :layout => false, :json => { :err_message => 'カテゴリを変更できないユーザが含まれています。' } and return
          end
          unless user.save then
            render :layout => false, :json => { :err_message => 'カテゴリの更新に失敗しました。' } and return
          end
        }
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'カテゴリの更新に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end
  
  def update_mail_notification_bulk
    filtered_params = params.permit(:userids, :notification)#, :dump, :offset, :per_page, :filters, :order)
    begin
      userids = JSON.parse(filtered_params[:userids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    unless userids then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    p_mail_notification = if filtered_params[:notification] == "false" then
                            "none"
                          elsif filtered_params[:notification] == "true" then
                            "only_my_events"
                          else
                            render :layout => false, :json => { :err_message => '不正な操作です' } and return
                          end
    @users_perms = {}
    begin
      userids = filter_user_ids(userids)
      User.transaction do
        users = User.where(id: userids.map(&:to_i))
        users.each do |user|
          get_role(user)
          if !@sidfm_roles.include?(:admin) && @users_perms[user].include?(:admin) then
            render :layout => false, :json => { :err_message => '不正な操作です' } and return
          end
        end
        users.update_all(mail_notification: p_mail_notification)
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => e } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def bulk_force_password_change
    if !@sidfm_roles.include?(:admin) then #管理者権限のみ操作可能
      render :layout => false, :json => { :err_message => '許可されていない操作です。' } and return
    end
    f_params = params.permit(:user_ids, :is_force_change_password) #:offset, :per_page, :filters, :order)
    is_force_change_password = f_params[:is_force_change_password]
    is_force_change_password = if is_force_change_password == "false" then
                                 false
                               elsif is_force_change_password == "true" then
                                 true
                               else
                                 render :layout => false, :json => { :err_message => '不正な操作です' } and return
                               end
    begin 
      user_ids = JSON.parse(f_params[:user_ids])
    rescue JSON::ParserError => e
      render :layout => false, :json => { :err_message => '不正な操作です' } and return
    end
    User.transaction do
      begin 
        User.where(:id => user_ids).update_all(:must_change_passwd => is_force_change_password)
      rescue Exception => e
        logger.info(e)
        render :layout => false, :json => { :err_message => '更新に失敗しました。' } and return
      end
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def delete
    filtered_params = params.permit(:id)
    filtered_userids = filter_user_ids([ filtered_params[:id].to_i ])
    if !filtered_userids.first then
      render :layout => false, :json => { :err_message => '不正な操作です' } and return
    end
    res = delete_user(user_id: filtered_params[:id], sidfm_roles: @sidfm_roles)
    render :layout => false, :json => { :err_message => res[:success] ? '' : res[:message] }
  end
  
  def test_email
    filtered_params = params.permit(:email_address, :subject, :message)#, :per_page, :filters, :order, :offset)
  begin 
    mail = {}
    #送信者はメール通知設定用のメールアドレス
    mail_sender_address = Setting.mail_from #User.find_by(:login => 'sidfmadm').mail
    mail[:from] = mail_sender_address
    mail[:to] = filtered_params[:email_address]
    mail[:subject] = filtered_params[:subject]
    mail[:message] = filtered_params[:message]
    SidfmdMailer.send_mail(mail).deliver_now 
  rescue Exception => e
    log_exception(e)
    render :layout => false, :json => { :err_message => 'テストメールの送信に失敗しました。※メールアドレスを確認して下さい。' } and return
  end
    render :layout => false, :json => { :err_message => '' }
  end

  def read_download_data
    unless export_import_executable? then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    GC.start
    f_params = params.permit(:target, :ids, :executed_size, :process, :order, :filters)
    if !f_params[:ids] or !f_params[:target] or !f_params[:order] or !f_params[:filters] or !f_params[:executed_size] or !f_params[:process] then
      logger.info('params error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
 
    executed_size = f_params[:executed_size].to_i

    @users_paginate = {}

    begin
      user_ids = JSON.parse(f_params[:ids])
      @users_paginate[:filters] = JSON.parse(f_params[:filters], symbolize_names: true)
      @users_paginate[:search_last_login_on] = @users_paginate[:filters][:search_last_login_on] || []
    rescue JSON::ParserError => e
      logger.info('json parser error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    if ![ 'tsv' ].include?(f_params[:target]) then
      logger.info('file target error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    @users_paginate[:per_page] = (f_params[:per_page] || 10).to_i
    @users_paginate[:offset] = (f_params[:offset] || 0).to_i
    @users_paginate[:search_login] = @users_paginate[:filters][:search_login] || ''
    @users_paginate[:search_name] = @users_paginate[:filters][:search_name] || ''
    @users_paginate[:search_auth] = @users_paginate[:filters][:search_auth] || ''
    @users_paginate[:search_users_auth] = @users_paginate[:filters][:search_users_auth] || ''
    @users_paginate[:search_category] = @users_paginate[:filters][:search_category] || ''
    @users_paginate[:search_mail_notification] = @users_paginate[:filters][:search_mail_notification] || ''
    @users_paginate[:search_locked_users] = @users_paginate[:filters][:search_locked_users] || ''

    @users_paginate[:order] = (f_params[:order] || 'login_asc')

    @users_paginate[:content] = User.where.not(:type => 'AnonymousUser')

    if !@sidfm_roles.include?(:admin) then
      if User.current.categories.exists? then
        @users_paginate[:content] = @users_paginate[:content].where(
          :id => Member.joins(:member_roles)
                       .joins("LEFT OUTER JOIN roles ON roles.id = member_roles.role_id")
                       .where.not(:roles => { :id => @sidfm_admin })
                       .where(
                         :members => { 
                           :user_id => User.get_category_users(User.current.categories.pluck(:id))
                         }
                       ).select('members.user_id')
        )
      else
        @users_paginate[:content] = @users_paginate[:content].where(:id => User.current.id)
      end
    end

    @users_paginate[:content] = @users_paginate[:content].where.not(:login => [ 'admin', 'sidfmadm' ]) #出力対象外

    search_items
    sort_for_users

    @user_category_ids = if @sidfm_roles.include?(:admin) then
                           nil
                         else
                           User.current.categories.pluck(:id)
                         end
 
    get_sidfm_project

    case f_params[:target]
    when 'tsv' then
      if f_params[:process] == 'init' then
        header = [ 'ユーザID', 'ログイン名', '姓', '名', 'メールアドレス', '認証方式', 'ユーザ権限', 'カテゴリ', 'カテゴリグループ', '最終ログイン日時', 'メール通知設定', 'ロック', '次回ログイン時にパスワード変更を強制', '削除', '新規ユーザのパスワード' ]
        @users_paginate[:content] = @users_paginate[:content].where(:id => user_ids) if user_ids.any?
        init_tsv_summary_table_progress_download(@users_paginate[:content].pluck(:id), header, @users_paginate[:content].count)
      else
        current_user_id = User.current.id
        @user_roles = Role.joins(:member_roles)
                          .joins(:members)
        create_tsv_data = Proc.new{|id|
          user = @users_paginate[:content].find_by(:id => id)
          next unless user
          item = []
          item << user.id.to_s
          item << user.login
          item << user.lastname
          item << user.firstname
          item << user.mail.to_s
          auth_source = user.auth_source
          item << (auth_source ? auth_source.name : 'パスワード認証')
          user_role = @user_roles.where(:members => { :user_id => user.id }).first
          item << (user_role ? user_role.name : '')
          is_admin_user = user.roles_for_project(@sidfm_project).map{|role| (role.permissions ? role.permissions.include?(:admin_sidfm) : false)}.include?(true)
          item << User.joins(:sidfmd_categories)
                      .where(%| 
                        sidfmd_categories_users.user_id = ? #{!@sidfm_roles.include?(:admin) ? "AND sidfmd_categories.id IN (SELECT unnest(ARRAY[#{@user_category_ids.join(',')}]))" : ''}
                      |, user.id)
                      .order('LOWER(sidfmd_categories.name COLLATE "C")')
                      .select('sidfmd_categories.name AS cat_name')
                      .map{|x| escape_category_tsv(x.cat_name) }
                      .uniq
                      .join(',') 
          item << if is_admin_user then
                    ''
                  else
                    (
                      @user_category_ids ?
                        user.sidfmd_cat_groups.where(
                          :id => SidfmdCatGroupsUser.where(:user_id => current_user_id).select(:sidfmd_cat_group_id)
                        ) :
                        user.sidfmd_cat_groups
                    ).order('LOWER(name COLLATE "C")').map{|cat_grp|
                      escape_category_tsv("#{cat_grp.id}:#{cat_grp.name}")
                    }.join(',')
                  end
          item << (user.last_login_on ? user.last_login_on.strftime('%Y/%m/%d %H:%M:%S') : '')
          item << (user.mail_notification == 'only_my_events').to_s
          item << (user.status == User::STATUS_LOCKED).to_s
          item << user.must_change_passwd.to_s
          item << ''
          item << ''
          item 
        }
        json_tsv_summary_table_progress_download(user_ids, create_tsv_data, executed_size, 50)
      end
    end    
  end

  def download
    unless export_import_executable? then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    f_params = params.fetch(:download, {}).permit(:format)
    case f_params[:format]
    when 'tsv' then
      send_summary_table_tsv_file("vmlist-users-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
    end
  end 

  def read_import_data
    GC.start
    f_params = params.permit(:userinfo_file)
    userinfo_file = f_params[:userinfo_file]
    if !export_import_executable? or !userinfo_file then
      render :layout => false, :json => { :err => true } and return
    end

    begin
      session[:import_notices] = []
      session[:import_check_data] = {
        :user_ids => {},
        :logins => {},
        :mails => {},
        :invalid_line_nums => [] #データが有効なもので且つユーザID・ログイン・メールアドレスのいずれかが重複したものは登録しない
      }
      session[:import_users] = {}
      session[:import_actions] = []
      lines = []
      userinfo_file = userinfo_file.read
      if userinfo_file.getbyte(0) == 255 and userinfo_file.getbyte(1) == 254 then
        userinfo_file = userinfo_file[2, userinfo_file.length - 2]
      end
      userinfo_file = userinfo_file.force_encoding('UTF-16LE').encode('UTF-8')
      cnt = 1
      CSV.parse(userinfo_file, :col_sep => "\t", :headers => :first_row).each do |line|
        cnt += 1
        item = {}
        item[:user_id] = (line['ユーザID'] || '').strip
        item[:login] = (line['ログイン名'] || '').strip
        item[:lastname] = (line['姓'] || '')
        item[:firstname] = (line['名'] || '')
        item[:mail] = (line['メールアドレス'] || '').strip
        item[:auth] = (line['認証方式'] || '')
        item[:role] = (line['ユーザ権限'] || '').strip
        begin
          item[:categories] = ((line['カテゴリ'] and line['カテゴリ'].size > 0) ? CSV.parse_line(line['カテゴリ']) : [])
        rescue Exception => e
          log_exception(e)
          session[:import_notices] << "ファイルの#{cnt}行目: カテゴリ列のフォーマットに誤りがあります。" 
          render :layout => false, :json => { :err => true } and return
        end
        begin
          item[:cat_groups] = ((line['カテゴリグループ'] and line['カテゴリグループ'].size > 0) ? CSV.parse_line(line['カテゴリグループ']) : [])
        rescue Exception => e
          log_exception(e)
          session[:import_notices] << "ファイルの#{cnt}行目: カテゴリグループ列のフォーマットに誤りがあります。"
          render :layout => false, :json => { :err => true } and return
        end
        item[:last_login_on] = (line['最終ログイン日時'] || '').strip
        item[:mail_notification] = (line['メール通知設定'] || '').strip
        item[:locked] = (line['ロック'] || '').strip
        item[:must_change_passwd] = (line['次回ログイン時にパスワード変更を強制'] || '').strip
        item[:delete_flg] = (line['削除'] || '').strip
        item[:new_user_passwd] = (line['新規ユーザのパスワード'] || '').strip
        item[:line_num] = cnt
        lines << item
      end
      session[:import_users][:lines] = lines
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err => true } and return
    end
    
    render :layout => false, :json => { :err => false, :total_size => lines.size }
  end

  def progress_import
    import_users = session[:import_users].symbolize_keys
    check_data = session[:import_check_data].symbolize_keys
    actions = session[:import_actions]
    lines = import_users[:lines].shift(50)
    executed_size = lines.size
    session[:import_users] = import_users
    is_admin = @sidfm_roles.include?(:admin)
    password_min_length = Setting.password_min_length.to_i
    user_category_ids = (is_admin ? nil : User.current.categories.pluck(:id))
    roles = Role.joins(:member_roles)
                .joins(:members)
                .distinct
                .pluck('roles.name')

    user_roles = Role.joins(:member_roles)
                     .joins(:members)
                     .where(:members => { :user_id => lines.map{|line| line.symbolize_keys[:user_id]}.compact })
    current_user_id = User.current.id
    get_group_ids = -> (cat_groups){
      grp_ids = cat_groups.map{|grp|
        grp_id = grp.slice(/\A[0-9]+:|\A[０-９]+:/)
        grp_id = (grp_id ? grp_id.tr('０-９', '0-9') : nil)
      }.compact
    }

    lines.each do |line|
      line = line.symbolize_keys
      #validation
      is_pass = true
      if line[:user_id].size == 0 then 
        if line[:new_user_passwd].size == 0 and line[:auth] != OPENID_CONNECT_AUTH_NAME then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 新規ユーザで認証方式が#{OPENID_CONNECT_AUTH_NAME}でない場合は、パスワードの入力が必須です。"
          is_pass = false
        elsif line[:new_user_passwd].size > 0 and line[:auth] == OPENID_CONNECT_AUTH_NAME then
          #OpenIDConnectの時は、パスワードは無効
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 認証方式が#{OPENID_CONNECT_AUTH_NAME}の場合は、パスワードは無効になります。"
          line[:new_user_passwd] = ''
        end
      elsif line[:user_id].size > 0 and ((line[:user_id] =~ /^[0-9０-９]+$/) == nil) then
        session[:import_notices] << "ファイルの#{line[:line_num]}行目: ユーザIDは数字で入力して下さい。"
        is_pass = false
      elsif line[:user_id].size > 0 and check_data[:user_ids][line[:user_id]]  then
        session[:import_notices] << "ファイルの#{check_data[:user_ids][line[:user_id]]}行目と#{line[:line_num]}行目: ユーザIDが重複しています。"
        check_data[:invalid_line_nums] << check_data[:user_ids][line[:user_id]]
        is_pass = false
      else
        line[:user_id] = line[:user_id].tr('０-９','0-9')
        check_data[:user_ids][line[:user_id]] = line[:line_num] if line[:user_id].size > 0
      end

      if line[:delete_flg].size > 0 then
        line[:delete_flg] = line[:delete_flg].downcase
        if line[:delete_flg] != 'false' and line[:delete_flg] != 'true' then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 削除列は、「TRUE」・「true」・「FALSE」・「false」のいずれかを入力して下さい。"
          is_pass = false
        elsif line[:delete_flg] == 'true' and line[:user_id].size == 0 then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 新規ユーザの場合、削除列の「TRUE」または「true」は無効です。"
        elsif line[:delete_flg] == 'true' and line[:user_id].to_i == User.current.id then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザを削除することはできません。"
          is_pass = false
        end
      end

      if line[:delete_flg] != 'true' or line[:user_id].size == 0 then #削除するものはチェックしない
        line[:login] = line[:login].tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z').gsub('：', ':')
        if line[:login].size == 0 then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログイン名が入力されておりません。"
          is_pass = false
        elsif line[:login].size > User::LOGIN_LENGTH_LIMIT then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログイン名は#{User::LOGIN_LENGTH_LIMIT}文字以内で入力して下さい。"
          is_pass = false
        elsif !login_valid?(line[:login]) then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログイン名に使用できない文字が含まれています。※半角英数字及び _.-@ のみ使用可能です。"
          is_pass = false
        elsif check_data[:logins][line[:login]] then
          session[:import_notices] << "ファイルの#{check_data[:logins][line[:login]]}行目と#{line[:line_num]}行目: ログイン名が重複しています。"
          check_data[:invalid_line_nums] << check_data[:logins][line[:login]]
          is_pass = false
        elsif (line[:user_id].size > 0 and User.where.not(:id => line[:user_id]).where('LOWER(login) = ?', line[:login].downcase).exists?) or 
              (line[:user_id].size == 0 and User.where('LOWER(login) = ?', line[:login].downcase).exists?) 
        then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 登録されているログイン名と同じログイン名は設定できません。"
          is_pass = false
        else
          check_data[:logins][line[:login]] = line[:line_num]
        end

        line[:mail] = line[:mail].tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z').gsub('：', ':')
        if line[:mail].size == 0 then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: メールアドレスが入力されておりません。"
          is_pass = false
        elsif line[:mail].size > User::MAIL_LENGTH_LIMIT then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: メールアドレスは#{User::MAIL_LENGTH_LIMIT}文字以内で入力して下さい。"
          is_pass = false
        elsif !mail_address_valid?(line[:mail]) then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: メールアドレスに誤りがあります。"
          is_pass = false
        elsif check_data[:mails][line[:mail]] then 
          session[:import_notices] << "ファイルの#{check_data[:mails][line[:mail]]}行目と#{line[:line_num]}行目: メールアドレスが重複しています。"
          check_data[:invalid_line_nums] << check_data[:mails][line[:mail]]
          is_pass = false
        elsif (line[:user_id].size > 0 and 
               EmailAddress.joins(:user)
                           .where.not(:users => { :id => line[:user_id] })
                           .where('LOWER(email_addresses.address) = ?', line[:mail].downcase)
                           .exists?
              ) or
              (line[:user_id].size == 0 and 
                EmailAddress.joins(:user)
                            .where('LOWER(email_addresses.address) = ?', line[:mail].downcase)
                            .exists?
              )
        then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 登録されているメールアドレスと同じメールアドレスは設定できません。"
          is_pass = false
        else
          check_data[:mails][line[:mail]] = line[:line_num]
        end

        if line[:lastname].size == 0 then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 姓が入力されておりません。"
          is_pass = false
        elsif line[:lastname].size > LASTNAME_LENGTH_LIMIT then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 姓は#{LASTNAME_LENGTH_LIMIT}文字以内で入力して下さい。"
          is_pass = false
        end

        if line[:firstname].size == 0 then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 名が入力されておりません。"
          is_pass = false
        elsif line[:firstname].size > FIRSTNAME_LENGTH_LIMIT then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 名は#{FIRSTNAME_LENGTH_LIMIT}文字以内で入力して下さい。"
          is_pass = false
        end

        if line[:auth].strip.size == 0 then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 認証方式が入力されておりません。"
          is_pass = false
        elsif line[:auth].strip != 'パスワード認証' and !AuthSource.where(:name => [ line[:auth], line[:auth].strip ]).exists? then
          #登録時は、先にstripしないもので検索して、なければstripしたもので登録するようにする。
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 存在しない認証方式です。"
          is_pass = false
        end
 
        if line[:role].size > 0 then
          is_role_pass = true
          if !is_admin then
            if line[:role] == 'SIDfmVM管理者' then
              session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザの権限は、SIDfmVM管理者権限を付与できません。"
              is_pass = false
              is_role_pass = false
            end
          end
          if is_role_pass then
            if line[:role] == '管理者' then
              session[:import_notices] << "ファイルの#{line[:line_num]}行目: 管理者権限は付与できません。#{is_admin ? '※SIDfmVM管理者権限を付与することは可能です。' : ''}"
              is_pass = false
            elsif !roles.include?(line[:role]) then
              session[:import_notices] << "ファイルの#{line[:line_num]}行目: 存在しない権限です。"
              is_pass = false
            end
          end
        else
          if line[:delete_flg].size == 0 then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: 権限を入力して下さい。"
            is_pass = false
          end
        end
     
        if line[:categories].include?(nil) then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 無効なカテゴリがあります。"
          is_pass = false
        else
          line[:categories] = line[:categories].map{|cat_name| cat_name.gsub("\n", '').strip}.uniq 
          #カテゴリをフィルタ
          if line[:role].size == 0 then
            #権限なしは不可。権限のチェックで弾く
            #if line[:categories].any? then
            #  session[:import_notices] << "ファイルの#{line[:line_num]}行目: 権限の無いユーザのカテゴリは設定できません。"
            #  is_pass = false
            #end
          else
            filtered_cats = if is_admin then
                              SidfmdCategory.where(:is_system => false, :name => line[:categories])
                            else
                              SidfmdCategory.where(:is_system => false, :name => line[:categories]).where(:id => user_category_ids)
                            end
            if filtered_cats.count != line[:categories].size then
              #無効なカテゴリがある場合は警告。(無効なカテゴリを除去して登録はする。)
              session[:import_notices] << "ファイルの#{line[:line_num]}行目: 無効なカテゴリ名が入力されています。"
            end
            line[:categories] = filtered_cats.map{|cat| { :id => cat.id, :name => cat.name, :desc => cat.desc }}
          end
        end

        if line[:cat_groups].include?(nil) then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 無効なカテゴリがあります。"
          is_pass = false
        else
          line[:cat_groups] = line[:cat_groups].map{|cat_grp| cat_grp.gsub("\n", '')}.uniq
          #カテゴリグループをフィルタ
          if line[:role].size == 0 then
            #権限なしは不可。権限のチェックで弾く
            #if line[:cat_groups].any? then
            #  session[:import_notices] << "ファイルの#{line[:line_num]}行目: 権限の無いユーザのカテゴリグループは設定できません。"
            #  is_pass = false
            #end
          else
            if line[:role] == 'SIDfmVM管理者' && line[:cat_groups].any? then
              session[:import_notices] << "ファイルの#{line[:line_num]}行目: SIDfmVM管理者権限ユーザのカテゴリグループは設定できません。"
              is_pass = false
            end
            #idで検索
            grp_ids = get_group_ids.(line[:cat_groups])
            filtered_cat_groups =
              if is_admin then
                SidfmdCatGroup.where(:id => grp_ids)
              else
                SidfmdCatGroup.where(:id => grp_ids)
                              .where(:id => SidfmdCatGroupsUser.where(:user_id => current_user_id).select(:sidfmd_cat_group_id))
              end
            if filtered_cat_groups.count != line[:cat_groups].size then
              #無効なグループがある場合は警告。(無効なグループを除去して登録はする。)
              session[:import_notices] << "ファイルの#{line[:line_num]}行目: 無効なカテゴリグループが入力されています。"
            end
            line[:cat_groups] = filtered_cat_groups.map{|grp| { :id => grp.id, :name => grp.name }}
          end
        end

        if line[:last_login_on].size > 0 then
          begin
            line[:last_login_on] = Time.parse(line[:last_login_on])
          rescue ArgumentError => e
            #logger.info(e)
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: 最終ログイン日時は、日時を入力して下さい。"
            is_pass = false
          end
        end
 
        if line[:mail_notification].size > 0 then
          line[:mail_notification] = line[:mail_notification].downcase
          if line[:mail_notification] != 'true' and line[:mail_notification] != 'false' then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: メール通知設定列は、「TRUE」・「true」・「FALSE」・「false」のいずれかを入力して下さい。"
            is_pass = false
          end
        else
          line[:mail_notification] = 'false' #入力がなければ無効を設定
        end

        if line[:user_id].size > 0 and line[:new_user_passwd].size > 0 then
          #既存ユーザでパスワード列に入力がある場合は、入力されたパスワードが無効になるだけにする
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: 新規ユーザのパスワード列は、新規ユーザのみ有効です。※登録済みユーザのパスワードは、インポートでは設定できません。"
        elsif line[:new_user_passwd].size > 0 then
          if line[:new_user_passwd].size < password_min_length then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: パスワードは#{password_min_length}文字以上で入力して下さい。"
            is_pass = false
          else
            temp_user = User.new
            temp_user.password = line[:new_user_passwd]
            temp_user.login = line[:login]
            is_password_valid = temp_user.check_password_with_password_policy
            unless is_password_valid then
              session[:import_notices] << "ファイルの#{line[:line_num]}行目: パスワードポリシーに違反しています。"
              is_pass = false
            end
          end
        end

        if line[:locked].size > 0 then
          line[:locked] = line[:locked].downcase
          if line[:locked] != 'true' and line[:locked] != 'false' then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: ロック列は、「TRUE」・「true」・「FALSE」・「false」のいずれかを入力して下さい。"
            is_pass = false
          elsif line[:locked] == 'true' and line[:user_id].size > 0 and line[:user_id].to_i == User.current.id then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザをロックすることはできません。"
            is_pass = false
          end
        else
          line[:locked] = 'false' #入力がなければ無効を設定
        end

        if line[:must_change_passwd].size > 0 then
          line[:must_change_passwd] = line[:must_change_passwd].downcase
          if line[:must_change_passwd] != 'true' and line[:must_change_passwd] != 'false' then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: 「次回ログイン時にパスワード変更を強制」列は、「TRUE」・「true」・「FALSE」・「false」のいずれかを入力して下さい。"
            is_pass = false
          end
        else
          line[:must_change_passwd] = 'false' #入力がなければ無効を設定
        end

      end
      if !is_pass then
        next
      end

      #新規登録
      if line[:user_id].size == 0 and (line[:new_user_passwd].size > 0 or line[:auth] == OPENID_CONNECT_AUTH_NAME) then
        if !is_admin then
          if line[:categories].size == 0 && line[:cat_groups].size == 0 then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザの権限は、カテゴリに属さないユーザを登録することはできません。"
            next
          end
          if line[:locked] == 'true' then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: ロックは、「FALSE」が設定されます。※ログインユーザがSIDfmVM管理者権限の場合のみ、任意の値を設定可能です。"
          end
          unless @sidfm_roles.include?(:user_admin) then
            if line[:must_change_passwd] == 'true' then
              session[:import_notices] << "ファイルの#{line[:line_num]}行目: 「次回ログイン時にパスワード変更を強制」は、「FALSE」が設定されます。※ログインユーザがSIDfmVM管理者権限またはSIDfmVM報告者権限の場合のみ、任意の値を設定可能です。"
            end
            line[:must_change_passwd] = 'false'
          end
          line[:locked] = 'false'
        end
        if line[:role] == 'SIDfmVM管理者' then
          if !@sidfm_roles.include?(:admin) && line[:categories].any? then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザの権限は、SIDfmVM管理者権限ユーザを登録できません。"
            line[:categories] = []
          end
          if line[:cat_groups].any? then
            session[:import_notices] << "ファイルの#{line[:line_num]}行目: SIDfmVM管理者権限ユーザのカテゴリグループは設定できません。"
            line[:cat_groups] = []
          end
        else
          line[:categories] = line[:categories].sort_by{|cat| cat[:name].downcase}
          line[:cat_groups] = line[:cat_groups].sort_by{|grp| grp[:name].downcase}
        end
        line[:last_login_on] = (line[:last_login_on].kind_of?(Time) ? line[:last_login_on].strftime('%Y/%m/%d %H:%M:%S') : '')
        actions << {
          :action => 'new',
          :user_name => line[:login],
          :user => line,
          :action_name => '新規登録',
          :line_num => line[:line_num]
        }
        next
      end

      target_user = User.find_by(:id => line[:user_id])
      if !target_user then
        session[:import_notices] << "ファイルの#{line[:line_num]}行目: ユーザが存在しません。"
        next
      elsif [ 'admin', 'sidfmadm' ].include?(target_user.login) then
        session[:import_notices] << "ファイルの#{line[:line_num]}行目: 操作できないユーザです。"
        next
      end

      if !is_admin then
        if !User.get_category_users(user_category_ids).where(:id => line[:user_id]).exists? then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: このユーザの操作は許可されておりません。"
          next
        end
      end

      #削除
      if line[:delete_flg] == 'true' then
        actions << {
          :action => 'delete',
          :user_name => target_user.login,
          :login => target_user.login,
          :user_id => target_user.id,
          :action_name => '削除',
          :line_num => line[:line_num]
        }
        next
      end

      #変更(チェック)
      options = []
      if target_user.login != line[:login] then
        options << {
          :action => 'login_change',
          :new_data => line[:login],
          :old_data => target_user.login,
          :action_name => 'ログイン名変更'
        }
      end
      if target_user.lastname != line[:lastname] then
        options << {
          :action => 'lastname_change',
          :new_data => line[:lastname],
          :old_data => target_user.lastname,
          :action_name => '姓変更'
        }
      end
      
      if target_user.firstname != line[:firstname] then
        options << {
          :action => 'firstname_change',
          :new_data => line[:firstname],
          :old_data => target_user.firstname,
          :action_name => '名変更'
        }
      end 
      
      user_mail = target_user.mail.to_s
      if user_mail != line[:mail] then
        options << {
          :action => 'mail_change',
          :new_data => line[:mail],
          :old_data => user_mail,
          :action_name => 'メールアドレス変更'
        }
      end
    
      auth_source = target_user.auth_source
      auth_name = (auth_source ? auth_source.name : 'パスワード認証')
      if auth_name != line[:auth] and auth_name != line[:auth].strip then
        options << {
          :action => 'auth_change',
          :new_data => line[:auth],
          :old_data => auth_name,
          :action_name => '認証方式変更' 
        }
      end

      user_role = user_roles.where(:members => { :user_id => line[:user_id] }).first
      user_rolename = (user_role ? user_role.name : '')
      if user_rolename != line[:role] then
        options << {
          :action => 'role_change',
          :new_data => line[:role],
          :old_data => user_rolename,
          :action_name => 'ユーザ権限変更'
        }
      end
      
      user_categories = target_user.sidfmd_categories
      user_categories = user_categories.where(:id => user_category_ids) unless is_admin
      if (user_categories.count != line[:categories].size) or user_categories.count != user_categories.where(:id => line[:categories].map{|cat| cat[:id]}).count then
        if line[:role] == 'SIDfmVM管理者' and !@sidfm_roles.include?(:admin) then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザの権限は、SIDfmVM管理者権限ユーザを変更できません。"
        elsif !is_admin and target_user == User.current then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザの権限は、ログインユーザのカテゴリを変更できません。"
        else
          if @sidfm_roles.include?(:admin) || line[:role] != 'SIDfmVM管理者' then
            options << {
              :action => 'categories_change',
              :new_data => line[:categories].sort_by{|cat| cat[:name].downcase},
              :old_data => user_categories.order('LOWER(name COLLATE "C")').map{|cat| { :id => cat.id, :name => cat.name, :desc => cat.desc }},
              :action_name => 'カテゴリ変更'
            }
          end
        end
      end

      user_cat_groups = target_user.sidfmd_cat_groups
      unless is_admin
        user_cat_groups = user_cat_groups.where(
          :id => SidfmdCatGroupsUser.where(:user_id => current_user_id).select(:sidfmd_cat_group_id)
        )
      end
      if (user_cat_groups.count != line[:cat_groups].size) or user_cat_groups.count != user_cat_groups.where(:id => line[:cat_groups].map{|grp| grp[:id]}).count then
        if line[:cat_groups].size > 0 and line[:role] == 'SIDfmVM管理者' then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: SIDfmVM管理者権限ユーザのカテゴリグループは変更できません。"
        elsif !is_admin and target_user.id == current_user_id then
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザの権限は、ログインユーザのカテゴリグループを変更できません。"
        else
          if line[:role] != 'SIDfmVM管理者' then
            options << {
              :action => 'cat_groups_change',
              :new_data => line[:cat_groups].sort_by{|grp| grp[:name].downcase},
              :old_data => user_cat_groups.order('LOWER(name COLLATE "C")').map{|grp| { :id => grp.id, :name => grp.name }},
              :action_name => 'カテゴリグループ変更'
            }
          end
        end
      end

      old_last_login_on = (target_user.last_login_on ? target_user.last_login_on.strftime('%Y/%m/%d %H:%M:%S') : '')
      new_last_login_on = (line[:last_login_on].kind_of?(Time) ? line[:last_login_on].strftime('%Y/%m/%d %H:%M:%S') : '')
      if old_last_login_on != new_last_login_on then
        options << {
          :action => 'last_login_on_change',
          :new_data => new_last_login_on,
          :old_data => old_last_login_on,
          :action_name => '最終ログイン日時変更'
        }
      end
     
      mail_notification = (target_user.mail_notification == 'only_my_events').to_s.downcase
      if mail_notification != line[:mail_notification] then
        options << {
          :action => 'mail_notification_change',
          :new_data => line[:mail_notification],
          :old_data => mail_notification,
          :action_name => 'メール通知設定変更'
        }
      end

      locked = (target_user.status == User::STATUS_LOCKED).to_s.downcase
      if locked != line[:locked] then
        if is_admin then
          options <<{
            :action => 'locked_change',
            :new_data => line[:locked],
            :old_data => locked,
            :action_name => 'ロック変更'
          }
        else
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザの権限は、ロックの変更を許可されていません。"
        end
      end

      must_change_passwd = target_user.must_change_passwd.to_s.downcase
      if must_change_passwd != line[:must_change_passwd] then
        if @sidfm_roles.include?(:user_admin) then
          options << {
            :action => 'must_change_passwd_change',
            :new_data => line[:must_change_passwd],
            :old_data => must_change_passwd,
            :action_name => '「次回ログイン時にパスワード変更を強制」変更'
          }
        else
          session[:import_notices] << "ファイルの#{line[:line_num]}行目: ログインユーザの権限は、「次回ログイン時にパスワード変更を強制」の変更を許可されていません。" 
        end
      end

      if options.any? then
        actions << {
          :action => 'modify',
          :user_name => target_user.login,
          :login => target_user.login,
          :user_id => target_user.id,
          :options => options,
          :action_name => '変更',
          :line_num => line[:line_num]
        }
      end

    end

    if executed_size == 0 then
      #ファイルで重複したデータを除去
      invalid_line_nums = check_data[:invalid_line_nums].map(&:to_i)
      filtered_actions = []
      session[:import_actions].each do |action|
        action = action.symbolize_keys
        if !invalid_line_nums.include?(action[:line_num].to_i) then
          filtered_actions << action
        end
      end
      session[:import_actions] = filtered_actions
    else
      session[:import_actions] = actions
      session[:import_check_data] = check_data
    end

    render :layout => false, :json => { :executed_size => executed_size }
  end

  def import
    @notices = session[:import_notices]
    actions = session[:import_actions]
    if !actions then
      redirect_to :action => :index, :notice => 'インポートに失敗しました。' and return
    elsif actions.empty? then
      redirect_to :action => :index, :notice => '変更がありません。' and return
    else
      session[:clear_session] = true #indexでclear_import_sessionする

      get_sort_value = ->(action){
        r = [ 'new', 'modify', 'delete' ].index(action[:action])
      }

      roles_name_id = {}
      Role.where('name LIKE ?', '%SIDfmVM%').each do |role|
        roles_name_id[role.name] = role.id
      end

      @actions = []
      @delete_user_ids = []
      @category_change_user_categoryids = {}
      @change_role_user_ids = {}
      @change_user_group_ids = {}

      actions.each do |action|
        action = action.deep_symbolize_keys
        @actions << action
        if action[:action] == 'delete' then
          @delete_user_ids << action[:user_id]
        elsif action[:action] == 'modify' then
          action[:options].each do |opt| 
            if opt[:action] == 'categories_change' then
              @category_change_user_categoryids[action[:user_id]] = opt[:new_data].map{|cat| cat[:id]}
            end
            if opt[:action] == 'role_change' then
              @change_role_user_ids[action[:user_id]] = (roles_name_id[opt[:new_data]] || -1)
            end
            if opt[:action] == 'cat_groups_change' then
              @change_user_group_ids[action[:user_id]] = opt[:new_data].map{|cat_grp| cat_grp[:id]}
            end
          end
        end
      end
      @actions.sort!{|a, b|
        #インポートの実行で削除を最初に行う
        r = get_sort_value.(a) <=> get_sort_value.(b)
        if r == 0 then
          r = a[:user_name].downcase <=> b[:user_name].downcase
        end
        r
      }
    end
  end

  def import_apply
    unless export_import_executable? then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    
    f_params = params.permit(:action_data, :process_rules, :delete_user_ids, :category_change_user_categoryids, :change_role_user_ids, :user_cat_group_ids)
    err_message = ''
    if !f_params[:action_data] then
      err_message = 'パラメータが不正です。'
    end
    begin 
      action = JSON.parse(f_params[:action_data], :symbolize_names => true)
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = 'パラメータが不正です。'
    end

    if err_message.size > 0 then
      render :layout => false, :json => { :err_message => err_message } and return
    end

    is_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = (is_admin ? nil : User.current.categories.pluck(:id))

    if f_params[:process_rules] then
      begin 
        delete_user_ids = JSON.parse(f_params[:delete_user_ids])
        change_category_user_ids = JSON.parse(f_params[:category_change_user_categoryids])
        change_role_user_ids = JSON.parse(f_params[:change_role_user_ids])
        user_cat_group_ids = JSON.parse(f_params[:user_cat_group_ids])
        unless is_admin then 
          SidfmdCategoriesUser.where(:user_id => change_category_user_ids.keys).group(:user_id).select(
            "ARRAY_AGG(sidfmd_category_id) AS category_ids",
            "CAST(user_id AS text) AS str_user_id"
          )
          .each do |cat_user|
            unchangeable_category_ids = (cat_user.category_ids - @user_category_ids)
            change_category_user_ids[cat_user.str_user_id] |= unchangeable_category_ids
          end
        end
        current_user_id = User.current.id
        admin_role_ids = Role.where(:name => 'SIDfmVM管理者').pluck(:id)
        (change_category_user_ids.keys + user_cat_group_ids.keys + change_role_user_ids.map{|k, v| admin_role_ids.include?(v) ? k : nil}.compact).uniq.each do |user_id|
          #(あれば)カテゴリグループ変更後と(あれば)カテゴリ変更後の全カテゴリを反映してルール処理する
          change_category_user_ids[user_id.to_s] = User.after_cat_group_change_categories(
            :user_id => user_id,
            :category_ids => change_category_user_ids[user_id],
            :cat_group_ids => (admin_role_ids.include?(change_role_user_ids[user_id.to_s].to_i) ? [] : user_cat_group_ids[user_id]) #管理者はグループなしになるため
          )
          unless is_admin then
            change_category_user_ids[user_id.to_s] |= SidfmdCatGroup.limit_user_group_category_ids(user_id, current_user_id)
          end
        end
      rescue Exception => e
        log_exception(e)
        render :layout => false, :json => { :err_message => 'パラメータが不正です。', :rules_error => true } and return
      end
      begin
        SidfmdRule.transaction do
          process_rules = User.process_related_rules(
            :sidfm_roles => @sidfm_roles,
            :delete_user_ids => delete_user_ids,
            :change_category_user_ids => change_category_user_ids,
            :change_role_user_ids => change_role_user_ids,
            :action => 'import'
          )
          if !process_rules then
            render :layout => false, :json => { :err_message => 'インポートに失敗しました。', :rules_error => true } and return
          end
        end
      end
    end

    password_min_length = Setting.password_min_length.to_i

    case action[:action]
    when 'new' then
      user = action[:user]
      columns_of_new = [ 
        :login, :lastname, :firstname, 
        :mail, :auth, :role, 
        :categories, :last_login_on, :mail_notification,
        :must_change_passwd, :locked, :cat_groups
      ]
      columns_of_new.each do |column|
        unless user[column] then
          err_message = "#{user[:login]}のパラメータが不正です。"
          break
        end
      end
      if err_message.size > 0 then
        render :layout => false, :json => { :err_message => err_message } and return
      end

      if user[:auth] != OPENID_CONNECT_AUTH_NAME and (!user[:new_user_passwd] or user[:new_user_passwd].size < password_min_length) then
        err_message = "#{user[:login]}の登録に失敗しました。※パスワードは、#{password_min_length}文字以上で入力して下さい。"
      end
      if err_message.size > 0 then
        render :layout => false, :json => { :err_message => err_message } and return
      end

      categories = []
      if user[:categories].size > 0 then
        if is_admin || user[:role].strip != 'SIDfmVM管理者' then
          categories = SidfmdCategory.where(:id => user[:categories].map{|cat| cat[:id]})
          unless is_admin then
            categories = categories.where(:id => @user_category_ids)
          end
        end
      end

      cat_groups = []
      if user[:cat_groups].size > 0 then
        if user[:role].strip != 'SIDfmVM管理者' then
          cat_groups = SidfmdCatGroup.where(:id => user[:cat_groups].map{|grp| grp[:id]})
          unless is_admin then
            cat_groups = cat_groups.where(
              :id => SidfmdCatGroupsUser.where(:user_id => User.current.id).select(:sidfmd_cat_group_id)
            )
          end
        end
      end

      if user[:last_login_on].size > 0
        begin 
          user[:last_login_on] = Time.parse(user[:last_login_on])
        rescue ArgumentError => e
          log_exception(e)
          err_message = "最終ログイン日時が不正なため、#{user[:login]}の登録に失敗しました。"
        end
      end
    
      User.transaction do               
        new_user = User.new
        new_user.login = user[:login]
        new_user.lastname = user[:lastname]
        new_user.firstname = user[:firstname]
        new_user.last_login_on =  user[:last_login_on] if user[:last_login_on].kind_of?(Time)
        new_user.mail_notification = (user[:mail_notification] == 'true' ? 'only_my_events' : 'none')
        new_user.must_change_passwd = (user[:must_change_passwd] == 'true' ? true : false)
        if user[:new_user_passwd] and user[:new_user_passwd].size > 0 then
          new_user.password = user[:new_user_passwd]
          new_user.password_confirmation = user[:new_user_passwd]
        end
        new_user.mail = user[:mail]
        new_user.sidfmd_categories = categories
        new_user.sidfmd_cat_groups = cat_groups

        if User.where(:login => user[:login]).exists? then
          err_message = "ログイン名が既に使用されているため、#{user[:login]}の登録に失敗しました。"
        end
        
        if EmailAddress.joins(:user)
                       .where.not(:users => { :type => 'AnonymousUser' })
                       .where(:email_addresses => { :address => user[:mail] })
                       .exists?
        then
          err_message = "メールアドレスが既に使用されているため、#{user[:login]}の登録に失敗しました。"
        end

        is_password_valid = new_user.check_password_with_password_policy
        unless is_password_valid then
          err_message = "パスワードポリシーに違反しているため、#{user[:login]}の登録に失敗しました。"
        end
        
        if user[:auth] != 'パスワード認証' then
          auth_source = AuthSource.find_by(:name => user[:auth])
          auth_source = AuthSource.find_by(:name => user[:auth].strip) unless auth_source

          if auth_source then
            new_user.auth_source_id = auth_source.id
          else
            err_message = "認証方式が存在しないため、#{user[:login]}の登録に失敗しました。"
          end
        end

        if @sidfm_roles.include?(:user_admin) then
          new_user.must_change_passwd = (user[:must_change_passwd] == 'true' ? true : false)
        end

        if is_admin then
          new_user.status = User::STATUS_LOCKED if user[:locked] == 'true'
        end

        
        if err_message.size == 0 then
          if new_user.save then
            if user[:role].size > 0 then
              role = Role.find_by(:name => user[:role].strip)
              if role then
                is_pass = true
                if !is_admin and role.name == 'SIDfmVM管理者' then
                  is_pass = false
                elsif role.id == @admin then
                  is_pass = false
                end
                if is_pass then
                  Member.create_principal_memberships(new_user, :project_ids => [ @sidfm_project.id ], :role_ids => [ role.id ])
                end
              else 
                err_message = "権限が存在しないため、#{user[:login]}の登録に失敗しました。"
                raise ActiveRecord::Rollback 
              end
            else
              err_message = "権限が存在しないため、#{user[:login]}の登録に失敗しました。"
              raise ActiveRecord::Rollback
            end
          else
            err_message = "#{user[:login]}の登録に失敗しました。"
          end
        end
      end

    when 'modify' then
      login_name = action[:login]
      target_user = User.find_by(:id => action[:user_id])
      if !target_user then
        err_message = "#{login_name}は削除されているため、変更できませんでした。"
      elsif [ 'admin', 'sidfmadm' ].include?(target_user.login) then
        err_message = "#{target_user.login}は変更できません。"
      end
      if err_message.size > 0 then
        render :layout => false, :json => { :err_message => err_message } and return
      end
    
      unless is_admin then
        unless User.get_category_users(@user_category_ids).where(:id => target_user.id).exists? then
          err_message = "ログインユーザのカテゴリ範囲外であるため、#{login_name}は変更できませんでした。"
          render :layout => false, :json => { :err_message => err_message } and return
        end
      end

      is_delete_session_necessary = false
      update_attrs = {}
      is_invalid = false #不正な操作をしない限りtrueにはならない
      update_err_messages = []
      action[:options].each do |opt|
        if !opt[:new_data] then
          is_invalid = true
          next
        end

        case opt[:action]
        when 'login_change' then
          if opt[:new_data].size == 0 or opt[:new_data].size > User::LOGIN_LENGTH_LIMIT then
            is_invalid = true
          elsif User.where(:login => opt[:new_data]).exists? then
            update_err_messages << "ログイン名が既に使用されているため、#{login_name}のログイン名の変更に失敗しました。"
          else
            update_attrs[:login] = opt[:new_data]
          end
        when 'lastname_change' then
          if opt[:new_data].size == 0 or opt[:new_data].size > LASTNAME_LENGTH_LIMIT then
            is_invalid = true
          else
            update_attrs[:lastname] = opt[:new_data]
          end
        when 'firstname_change' then
          if opt[:new_data].size == 0 or opt[:new_data].size > LASTNAME_LENGTH_LIMIT then
            is_invalid = true
          else
            update_attrs[:firstname] = opt[:new_data]
          end
        when 'mail_change' then
          if opt[:new_data].size == 0 or opt[:new_data].size > User::MAIL_LENGTH_LIMIT or !mail_address_valid?(opt[:new_data]) then
            is_invalid = true
          elsif EmailAddress.joins(:user).where(:email_addresses => { :address => opt[:new_data] }).exists? then
            update_err_messages << "メールアドレスが既に使用されているため、#{login_name}のメールアドレスの変更に失敗しました。"
          else
            update_attrs[:mail] = opt[:new_data]
          end
        when 'auth_change' then
          if opt[:new_data].size == 0 then
            is_invalid = true
          else
            auth = nil
            if opt[:new_data] != 'パスワード認証' then
              auth = AuthSource.find_by(:name => opt[:new_data])
              auth = AuthSource.find_by(:name => opt[:new_data].strip) if !auth
              if !auth then
                update_err_messages << "認証方式が存在しないため、#{login_name}の認証方式の変更に失敗しました。"
                next
              end
            end
            update_attrs[:auth_source] = auth
            is_delete_session_necessary = true
          end
        when 'role_change' then
          if opt[:new_data].size > 0 then
            role = Role.find_by(:name => opt[:new_data].strip)
            if role then
              is_pass = true
              if !is_admin and role.name == 'SIDfmVM管理者' then
                is_pass = false
                update_err_messages << "ログインユーザはSIDfmVM管理者を付与できないため、#{login_name}の権限の変更に失敗しました。"
              elsif role.id == @admin then
                is_pass = false
                is_invalid = true
              end
              if is_pass then
                update_attrs[:role] = role
              end
            else
              update_err_messages << "権限が存在しないため、#{login_name}の権限の変更に失敗しました。"
            end
          end
        when 'categories_change' then
          categories = []
          if opt[:new_data].size > 0 or !is_admin then
            categories = SidfmdCategory.where(:id => opt[:new_data].map{|cat| cat[:id]})
            unless is_admin then
              SidfmdCategoriesUser.where(:user_id => target_user.id).group(:user_id).select(
                "ARRAY_AGG(sidfmd_category_id) AS category_ids",
                "CAST(user_id AS text) AS str_user_id"
              )
              .each do |cat_user|
                unchangeable_category_ids = (cat_user.category_ids - @user_category_ids)
                categories = SidfmdCategory.where(%|
                  id IN (?) OR id IN (?)
                |,categories.where(:id => @user_category_ids).select(:id), unchangeable_category_ids)
              end
            end
          end
          update_attrs[:sidfmd_categories] = categories
        when 'cat_groups_change' then
          cat_groups = []
          if opt[:new_data].size > 0 or !is_admin then
            cat_groups = SidfmdCatGroup.where(:id => opt[:new_data].map{|cat| cat[:id]})
            unless is_admin then
              cat_groups = cat_groups.where( #フィルタ
                :id => User.current.sidfmd_cat_groups.select(:id)
              )
              unchangeable_cat_groups = target_user.sidfmd_cat_groups.where.not( #変更不可のグループ
                :id => User.current.sidfmd_cat_groups.select(:id)
              )
              cat_groups = cat_groups + unchangeable_cat_groups
            end
          end
          update_attrs[:sidfmd_cat_groups] = cat_groups
        when 'last_login_on_change' then
          if opt[:new_data].size > 0 then
            begin
              opt[:new_data] = Time.parse(opt[:new_data])
            rescue ArgumentError => e
              log_exception(e)
              is_invalid = true
              opt[:new_data] = nil
            end
          end
          update_attrs[:last_login_on] = (opt[:new_data].kind_of?(Time) ? opt[:new_data] : nil)
        when 'mail_notification_change' then
          update_attrs[:mail_notification] = (opt[:new_data] == 'true' ? 'only_my_events' : 'none')
        when 'locked_change'
          if !is_admin then
            update_err_messages << "ログインユーザはロックを変更できないため、#{login_name}のロックの変更に失敗しました。"
          else
            update_attrs[:status] = (opt[:new_data] == 'true' ? User::STATUS_LOCKED : User::STATUS_ACTIVE)
            is_delete_session_necessary = true
          end
        when 'must_change_passwd_change'
          if !@sidfm_roles.include?(:user_admin) then
            update_err_messages << "ログインユーザは「次回ログイン時にパスワード変更を強制」を変更できないため、#{login_name}の「次回ログイン時にパスワード変更を強制」の変更に失敗しました。"
          else
            update_attrs[:must_change_passwd] = opt[:new_data] == 'true'
          end
        end
      end
      if is_invalid then
        err_message = "不正な操作のため、#{login_name}の変更に失敗しました。"
      else
        User.transaction do
          is_err = false
          if update_attrs.has_key?(:role) then
            #roleだけは単純にupdateできない
            role = update_attrs[:role]
            update_attrs.delete(:role)
            member = Member.where(:project_id => @sidfm_project.id, user_id: target_user.id).first
            if role then
              if member then
                member_role = MemberRole.where(:member_id => member.id).first
                unless member_role.update(:role_id => role.id) then
                  is_err = true
                end
              else
                Member.create_principal_memberships(target_user, :project_ids => [ @sidfm_project.id ], :role_ids => [ role.id ])
              end
              if Role.where(:id => role.id, :name => ROLE_SIDFM_USR).exists? then
                #閲覧者に変更なら、VmApikeyをチェックして更新許可がtrueをfalseに変更
                SidfmdVmApikey.update_user_apikeys_allow_update target_user.id
              end
              if role.id == @sidfm_admin then
                update_attrs[:sidfmd_cat_groups] = [] #管理者はカテゴリグループの紐付けなし
              end
            else
              if member then
                update_attrs[:sidfmd_categories] = []
                update_attrs[:sidfmd_cat_groups] = []
                unless member.destroy then
                  is_err = true
                end
                SidfmdVmApikey.delete_user_apikeys [target_user.id] #権限削除ならAPIキー削除
              end            
            end
          end
          if is_err or !target_user.update(update_attrs) then
            err_message = "#{login_name}の変更に失敗しました。"
          else
            if target_user.id == User.current.id then
              session[:tk] = target_user.generate_session_token
            else
              Session.delete_session(target_user.id) if is_delete_session_necessary
            end
          end
        end
      end
      if update_err_messages.any? then
        err_message = update_err_messages.join("\n")
        if action[:options].size != update_err_messages.size then
          err_message += "\n※他のデータの変更は完了しています。"
        end
      end
    when 'delete' then
      target_user = User.find_by(:id => action[:user_id])
      if target_user then
        if [ 'admin', 'sidfmadm' ].include?(target_user.login) then
          err_message += "#{target_user.login}は削除できません。"
        elsif target_user.id == User.current.id then
          err_message += "ログインユーザは削除できません。"
        else
          User.transaction do
            unless target_user.destroy then
              err_message = "#{action[:login]}の削除に失敗しました。"
            else
              Session.delete_session(action[:user_id].to_i)
            end
          end
        end
      end
    end
    render :layout => false, :json => { :err_message => err_message  }
  end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    create_all_content
    @users_paginate[:per_page] = f_params[:per_page].to_i

    search_items
    sort_for_users

    create_users_role_names(@users_paginate[:content])
    userids_names = @users_paginate[:content].where(:admin => false).map{|x| [x.id, x.login] }.to_h
    @users_paginate[:total_size] = @users_paginate[:content].count
    @users_paginate[:all_ids] = @users_paginate[:content].where(:admin => false).pluck(:id)

    @users_paginate[:content] = @users_paginate[:content].offset(@users_paginate[:offset]).limit(@users_paginate[:per_page])
    create_user_cat_groups(@users_paginate[:content].pluck(:id))
    create_user_sidfmd_categories(@users_paginate[:content].pluck(:id))
    @users_perms = {}

    current_user = User.current
    current_user_category_ids = 
      @sidfm_roles.include?(:admin) ?
      nil :
      current_user.categories.pluck(:id)
    ret = {
      :rows => @users_paginate[:content].map{|user|
        create_summary_user_html(
          user,
          {
            :checked => @users_paginate[:checked],
            :current_user => current_user,
            :current_user_category_ids => current_user_category_ids
          }
        )
      },
      :total => @users_paginate[:total_size],
      :all_ids => @users_paginate[:all_ids],
      :users_role_names => @users_role_names,
      :userids_names => userids_names,
      :sidfm_roles => @sidfm_roles,
      :user_cats => current_user_category_ids
    }
    render :layout => false, :json => ret
  end

  def api_get_vm_apikey_related_users
    f_params = params.permit(:user_role_ids, :delete_user_ids)
    user_role_ids = JSON.parse(f_params[:user_role_ids] || '{}')
    delete_user_ids = JSON.parse(f_params[:delete_user_ids] || '[]')
    delete_user_ids = delete_user_ids.map(&:to_i)
    view_role_ids = Role.where(:name => "SIDfmVM閲覧者").pluck(:id)
    not_allowed_allow_update_true_users = []
    delete_role_apikey_users = []
    delete_apikey_users = []
    if user_role_ids.size > 0 || delete_user_ids.size > 0 then
      user_apikeys = User.joins(:sidfmd_vm_apikeys)
      if !@sidfm_roles.include?(:admin) then
        user_apikeys = user_apikeys.where(:users => {
          :id => User.get_category_users(User.current.categories.pluck(:id))
                     .select(:id)
        })
      end
      user_apikeys.where(%|
        (sidfmd_vm_apikeys.user_id IN (?) AND allow_update IS TRUE) OR sidfmd_vm_apikeys.user_id IN (?) 
      |, user_role_ids.keys, (user_role_ids.map{|k, v| v.to_i < 0 ? k : nil}.compact + delete_user_ids))
      .select(
        :id,
        :login,
        :firstname,
        :lastname
      )
      .order('LOWER(users.login COLLATE "C")')
      .each do |user|
        if delete_user_ids.include?(user.id) then
          delete_apikey_users << user
          next
        end
        role_id = user_role_ids[user.id.to_s].to_i
        if role_id < 0 then
          delete_role_apikey_users << user #権限削除はAPIキーも削除
        elsif view_role_ids.include?(role_id) then
          not_allowed_allow_update_true_users << user #閲覧者は更新許可をtrueにできない
        end
      end
    end
    not_allowed_allow_update_true_users = not_allowed_allow_update_true_users.uniq{|user| user.id}
    delete_apikey_users = delete_apikey_users.uniq{|user| user.id}
    delete_role_apikey_users = delete_role_apikey_users.uniq{|user| user.id}
    render :layout => false, :json => {
      :not_allowed_allow_update_true_users => not_allowed_allow_update_true_users,
      :delete_apikey_users => delete_apikey_users,
      :delete_role_apikey_users => delete_role_apikey_users,
      :all_related_users => (not_allowed_allow_update_true_users + delete_apikey_users + delete_role_apikey_users).uniq{|user| user.id}.sort_by{|user| user.login.downcase}
    }
  end

  #ユーザ毎のカテゴリグループを返す
  def api_get_user_cat_groups
    f_params = params.permit(:user_ids)
    user_ids = JSON.parse(f_params[:user_ids])
    ret = {}
    @users_perms = {}
    User.where(:id => user_ids).each do |user|
      get_role(user)
      if @users_perms[user].include?(:admin) then
        ret[user.id] = []
      else
        ret[user.id] = user.sidfmd_cat_groups
      end
    end
    render :layout => false, :json => ret
  end

  # ユーザ毎のカテゴリを返す
  # 管理者でない場合は、ログインユーザのカテゴリの範囲内のカテゴリのみ返すことに注意
  def api_get_users_categories
    f_params = params.permit(:user_ids, :target)
    user_ids = JSON.parse(f_params[:user_ids])
    current_user_category_ids = User.current.categories.pluck(:id)
    user_category_ids = {}
    case f_params[:target]
    when 'category_ids_without_group_categories' then
      cat_users = SidfmdCategoriesUser.where(:user_id => user_ids)
      unless @sidfm_roles.include?(:admin) then
        cat_users = cat_users.where(:sidfmd_category_id => current_user_category_ids)
      end
      cat_users.each do |c_user|
        user_category_ids[c_user.user_id] = Set.new unless user_category_ids[c_user.user_id]
        user_category_ids[c_user.user_id].add(c_user.sidfmd_category_id)
      end
      unless @sidfm_roles.include?(:admin) then #管理者でない時は、ログインユーザに設定されていないカテゴリグループのカテゴリも対象ユーザのカテゴリとする
        current_user_id = User.current.id
        user_ids.each do |user_id|
          user_category_ids[user_id.to_i] = Set.new unless user_category_ids[user_id.to_i]
          unchangeable_cat_group_category_ids = SidfmdCatGroup.limit_user_group_category_ids(user_id.to_i, current_user_id)
          unchangeable_cat_group_category_ids.each do |cat_id|
            user_category_ids[user_id.to_i].add(cat_id)
          end
        end
      end
    when 'group_category_ids' then
      cat_group_users = SidfmdCatGroupsUser.joins(:sidfmd_cat_group).joins("
        LEFT OUTER JOIN sidfmd_cat_groups_categories ON sidfmd_cat_groups.id = sidfmd_cat_groups_categories.sidfmd_cat_group_id
      ")
      .where(:user_id =>  user_ids)
      .select(
        'sidfmd_cat_groups_categories.sidfmd_category_id',
         :user_id
      )
      unless @sidfm_roles.include?(:admin) then
        cat_group_users = cat_group_users.where(:sidfmd_cat_groups_categories => { :sidfmd_category_id => current_user_category_ids })
      end
      cat_group_users.each do |grp_usr|
        user_category_ids[grp_usr.user_id] = Set.new unless user_category_ids[grp_usr.user_id]
        user_category_ids[grp_usr.user_id].add(grp_usr.sidfmd_category_id)
      end
    end
    user_category_ids.each do |user_id, cat_ids|
      user_category_ids[user_id] = cat_ids.to_a.compact
    end
    render :layout => false, :json => user_category_ids
  end

  private

  def get_role(user)
    roles = user.roles_for_project(get_sidfm_project)
    temp = []
    roles.each do |role|
      perms = role.permissions
      if perms then
        temp << :admin if perms.include?(:admin_sidfm)
        temp << :use if perms.include?(:use_sidfm)
        temp << :view if perms.include?(:view_sidfm)
        temp << :user_admin if perms.include?(:user_admin_sidfm)
        temp << :host_admin if perms.include?(:host_admin_sidfm)
        temp << :host_limited_admin if perms.include?(:host_limited_admin_sidfm)
        temp << :rule_admin if perms.include?(:rule_admin_sidfm)
      end
    end
    temp.uniq!
    @users_perms[user] = temp
  end


  def update_users(userids, auth, method) # offset, per_page, filters, order)
    auth = auth.to_i
    unless userids or auth then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if auth.nil? || auth == 0 then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if auth < 0 then
      auth = nil
    end
    begin
      userids = filter_user_ids(userids)
      User.transaction do
        userids.each_with_index do |i, idx|
          id = i.to_i
          user = User.find(id)
          if user then 
            if method == "authentication" then
              user.update_attribute :auth_source_id, auth
              if id == User.current.id then
                session[:tk] = user.generate_session_token
              else
                Session.delete_session(id)
              end
            elsif method == "authority" then 
              if idx == 0 then
                if User.where(:id => userids).where(:login => 'sidfmadm').exists? then
                  render :layout => false, :json => { :err_message => 'ログイン名が「sidfmadm」のユーザの権限は変更できません。' } and return
                end
                #ルールの処理
                change_role_user_ids = {}
                userids.each do |user_id|
                  change_role_user_ids[user_id.to_s] = auth
                end
                process_rules = User.process_related_rules(
                  :sidfm_roles => @sidfm_roles,
                  :change_role_user_ids => change_role_user_ids,
                  :action => 'bulk_auth_change'
                )
                if !process_rules then
                  logger.error('failed to process rules')
                  raise Exception
                end
              end
              roleid =  role_check(id)
              if roleid then
                if roleid == -2 then
                  Member.create_principal_memberships(user, :project_ids => [ @sidfm_project.id ], :role_ids => auth)
                  SidfmdVmApikey.delete_user_apikeys [user.id] #権限削除ならAPIキー削除
                else
                  memrole = MemberRole.find(roleid)
                  memrole.update_attribute :role_id, auth
                  if Role.where(:id => auth, :name => ROLE_SIDFM_USR).exists? then
                    #閲覧者に変更なら、VmApikeyをチェックして更新許可がtrueをfalseに変更
                    SidfmdVmApikey.update_user_apikeys_allow_update user.id
                  end
                  if auth == @sidfm_admin then
                    user.sidfmd_cat_groups = [] #管理者に変更なら、カテゴリグループの紐付けはなし
                  end
                end        
              else
                raise Exception
              end
            end
          end
        end
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '権限の更新に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def role_check(p_id)
    memofuser = Member.where(:project_id => @sidfm_project.id).find_by(user_id: p_id)
    unless memofuser then
      return -2
    end
    MemberRole.joins(:member).where("member_id = ?", memofuser).first.id
  end
end
