#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmHelper

  RISKS = [ 'Critical', 'High', 'Medium', 'Low' ].map(&:freeze).freeze
  TARGETS = [ 'リモート', 'ローカル', 'クライアント' ].map(&:freeze).freeze
  FORM_PARAMS = 'queries'.freeze

  ### FIXME: RedmineのIssueに合わせて要修正
  STATUSES = [ '未読', '影響有り', '保留', '影響無し', '対策済み' ].map(&:freeze).freeze

  ### FIXME: SIDfmのデータに合わせて要修正
  CATEGORIES = [ '公開サーバ', '内部サーバ', 'モバイル', 'クライアント', '共用サーバ' ].map(&:freeze).freeze


  # 危険度を数値から表示用の文字列に変換
  def risk_to_string(v)
    number_to_string(RISKS, v)
  end

  # 対象を数値から表示用の文字列に変換
  def target_to_string(v)
    number_to_string(TARGETS, v)
  end

  # ステータスを数値から表示用の文字列に変換
  def status_to_string(v)
    number_to_string(STATUSES, v)
  end

  # カテゴリを数値から表示用の文字列に変換
  def category_to_string(v)
    number_to_string(CATEGORIES, v)
  end

  # 表の列名のタイトル出力
  #   sortnameとsort,orderでソート状態とリンクへのソートパラメータ挿入を行う。
  def table_title(name, sortname, sort, order, target = {}, options = {}, sort_param_name: 'sort', order_param_name: 'order')
    # ソートオプションのチェックと設定
    sort = '' unless sort
    if sort == sortname then
      # sortが自分のsortnameと一致している場合
      target[sort_param_name] = sortname
      target[order_param_name] = order == 'asc' ? 'desc' : 'asc'
      options[:class] = '' unless options.has_key?(:class)
      options[:class] << ' ' if options[:class].size > 0 
      options[:class] << 'sort ' << order
    else
      # その他全ての場合
      target[sort_param_name] = sortname
      target[order_param_name] = 'asc'
    end
    content_tag(:th, title: "並べ替え \"#{name}\"") do
      concat link_to name, target, options
    end
  end

  # 現在のURLにパラメータを追加したURLを返す。
  def url_for_with_new_param(k, v)
    url_for(params.merge(k => v))
  end

  # 現在のURLからパラメータを削除したURLを返す。
  def url_for_without_param(k)
    url_for(params.merge(k => nil))
  end

  # ブロックの返り値が"true"ならtrue、それ以外は全てdefault_valueで返す。
  # ブロックの処理で例外が出た場合はrescue_valueを返す。
  # viewでのパラメータ値の判定用。エラーにならずに値を判定できる。
  def query_true?(default_value = false, rescue_value = false)
    raise ArgumentError, 'must have a block' unless block_given?
    ret = default_value
    begin
      ret = ("true" == yield)
    rescue Exception
      ret = rescue_value
    end
    ret
  end

  # ブロックの返り値がnilでなければその値を文字列にして返す。
  # それ以外は全て(例外含む)defaut_valueで返す。
  # viewでのパラメータ値の取得用。エラーにならずに値を取得できる。
  def query_value(default_value = '')
    raise ArgumentError, 'must have a block' unless block_given?
    ret = default_value
    begin
      ret = yield
    rescue Exception
    end
    ret
  end

  # buttonと対応するhiddenを同時に生成する
  #  idはbuttonは{id}_btn, hiddenは{id}になる。
  def button_hidden_tag(id, btn_name, hdn_name, hdn_value, btn_class: '')
    concat(button_tag(btn_name, :id => id + '_btn', :class => btn_class))
    concat(hidden_field_tag(hdn_name, hdn_value, :id => id))
  end

  # SIDfmDigitalのmenuを出力する
  def sid_main_menu(current_menu_item)
    concat(content_tag(:div, render_menu(:sidfm_menu), :id => 'sidfm-menu', :class => 'main-menu tabs'))
    concat(content_tag(:div, render_menu(:sidfm_menu), :id => 'sidfm-menu-mobile', :class => 'flyout-menu'))
    concat(hidden_field_tag('current_menu_item', current_menu_item.to_s.tr('_', '-'), :id => 'current_menu_item'))
    # 選択されているメニューを選択状態にする処理と、PC/モバイルでメニューの有無を切り換える処理を出力する。
    concat(javascript_tag('$(function(){$("#sidfm-menu ."+$("#current_menu_item").val()).addClass("selected");(function(){var sidfm_menu_resize={timer:false,action:function(){if(this.timer!==false){clearTimeout(this.timer);};this.timer=setTimeout(function(){if(isMobile()){$("#sidfm-menu").css("display", "none");}else{$("#sidfm-menu").css("display", "block");}},200);}};$(window).resize(sidfm_menu_resize.action);})();});', :type => 'text/javascript'));
  end

  # 左向き三角を出力する。
  # sizeの単位はpt。
  def right_triangle(size = 10)
    content_tag(:svg, width: "#{size}px", height: "#{size}px") do
      content_tag(:path, nil, d: "M0 0 L#{size} #{size/2} L0 #{size} Z")
    end
  end

  # FIXME: under construction.
  def sidfm_menu(opts = {})
    concat(content_tag(:div, :id => 'sidfm-menu') do
             content_tag(:ul, :class => 'top') do
               concat(content_tag(:li, link_to('ダッシュボード', :controller => :sidfm, :action => :index)))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('ダッシュボード', :controller => :sidfm, :action => :index)))
                        concat(content_tag(:li, link_to('レイアウト・表示内容変更', '#')))
                      end)
               concat(content_tag(:li, link_to('ホスト情報', :controller => :sidfm_hosts, :action => :index)))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('対応が必要なホスト', '#')))
                        concat(content_tag(:li, link_to('#') do
                                             concat('カテゴリ')
                                             concat(right_triangle)
                                           end))
                        concat(content_tag(:li, link_to('#') do
                                             concat('ホスト')
                                             concat(right_triangle)
                                           end))
                        concat(content_tag(:li, link_to('統計・一覧', '#')))
                        concat(content_tag(:li, link_to('カテゴリ管理', '#')))
                      end)
               concat(content_tag(:li, link_to('脆弱性情報', :controller => :sidfm_vulnerabilities, :action => :index)))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('対応が必要な脆弱性', '#')))
                        concat(content_tag(:li, link_to('対応が必要な脆弱性 条件設定', '#')))
                        concat(content_tag(:li, link_to('#') do
                                             concat('指標')
                                             concat(right_triangle)
                                           end))
                        concat(content_tag(:li, link_to('#') do
                                             concat('検出時期')
                                             concat(right_triangle)
                                           end))
                        concat(content_tag(:li, link_to('#') do
                                             concat('脆弱性詳細')
                                             concat(right_triangle)
                                           end))
                      end)
               concat(content_tag(:li, link_to('SIDfm脆弱性情報', '#')))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('セキュリティホール情報')))
                        concat(content_tag(:li, link_to('アドバイザリ情報')))
                        concat(content_tag(:li, link_to('リリース・EOL情報')))
                        concat(content_tag(:li, link_to('セキュリティニュース情報')))
                      end)
               concat(content_tag(:li, link_to('test(development)', '#')))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('settings', :controller => :sidfm_settings, :action => :index)))
                      end)
             end
           end)
  end

  # FIXME
  #   ここより下が新しいシステム用。上は削除予定。

  COLOR = {
    # Bootstrapより
    :success => '#5cb85c'.freeze,
    :info => '#5bc0de'.freeze,
    :warning => '#f0ad4e'.freeze,
    :danger => '#d9534f'.freeze,


    :sri_none => 'hsl(200, 80%, 48%)'.freeze,
    :sri_low => 'hsl( 55, 80%, 48%)'.freeze, 
    :sri_medium => 'hsl( 30, 80%, 48%)'.freeze,
    :sri_high => 'hsl(  5, 80%, 48%)'.freeze,
    :sri_critical => 'hsl(340, 65%, 48%)'.freeze,
    :sri_fixed => '#4dcc33'.freeze,

    :cvss_notreg => '#f7d6b6'.freeze,
    :cvss_d4 => '#ca2b60'.freeze,
    :cvss_d3 => '#dc2918'.freeze,
    :cvss_d2 => '#dc7a18'.freeze,
    :cvss_d1 => '#dccc18'.freeze,
    :cvss_fixed => '#4dcc33'.freeze,

    :risk_low => '#c5c958'.freeze,
    :risk_medium => '#f8c036'.freeze,
    :risk_high => '#fa2121'.freeze,
    :risk_critical => '#c42161'.freeze,
    :risk_fixed => '#4dcc33'.freeze,

    :status_notaffect => '#3399cc'.freeze,
    :status_fixed => '#4dcc33'.freeze,
    :status_ongoing => '#e6c319'.freeze,
    :status_notconfirmed => '#e62a19'.freeze,

    :status_gradient_start => '#d9534f'.freeze,
    :status_gradient_end => '#5cb85c'.freeze,
  }.freeze

  # viewからCOLORにアクセスするために使用する。
  def colors
    COLOR
  end

  # colorの値が :success のようなシンボルだった場合、そのシンボルに
  # 対応する色を示す文字列を返す。
  # それ以外はそのま（シンボルは文字列化）ま返す。
  def symbol_color(color)
    COLOR[color] || color.to_s
  end

  # status barを生成する。width, heightから大きさを自動調節する。
  # width: px
  # height: px
  # data = [
  #  { :value => 値, :name => '名称', :color => '#aaa', :placement => 'top' },
  #  { :value => 値, :name => '名称', :color => 'blue', :placement => 'bottom' }, ...
  # ]
  #  省略可能(:placement(bottom))
  # opts = { :id, ... }
  def status_bar(width, height, data, opts = {})
    unit = opts[:unit] || ''
    opts.delete(:unit)
    content_tag(:div, :class => 'text-nowrap') do
      concat(content_tag(:svg,
                         opts.merge(:width => "#{width}px",
                                    :height => "#{height}px",
                                    :xmlns => 'http://www.w3.org/2000/svg',
                                    :version => '1.1')) do
               #data_size = data.size
               value_sum = data.inject(0) {|r,x| r += x[:value] }
               return if value_sum == 0
               unit_width = width.to_f / value_sum.to_f
               current_x = 0
               data.each_with_index do |item, i|
                 w = (unit_width * item[:value].to_f).to_i
                 concat(content_tag(:a,
                                    :'data-toggle' => 'tooltip',
                                    :'data-container'=> 'body',
                                    :'data-placement' => item[:placement] || 'bottom', 
                                    :'data-html' => 'true',
                                    :title => "<svg xmlns='http://www.w3.org/2000/svg' version='1.1' width='12px' height='12px'><rect x='0' y='0' width='10' height='12' fill='#{symbol_color(item[:color])}' stroke='#{symbol_color(item[:stroke])}' /></svg>#{item[:name]}:#{item[:value]}#{unit}")  do
                          rect_opt = {
                            :x => "#{current_x}",
                            :y => '0',
                            :width => "#{w}",
                            :height => "#{height}",
                            :'fill-opacity' => "1.0",
                            :fill => "#{symbol_color(item[:color] || 'white')}",
                          }
                          rect_opt[:stroke] = "#{symbol_color(item[:stroke])}" if item[:stroke]
                          concat(content_tag(:rect, '', rect_opt))
                        end)
                 current_x += w
               end
             end)
      concat(content_tag(:div, :class => 'row statusbar') do
               data.each do |i|
                 concat(content_tag(:div, :class => 'col-6') do
                          concat(content_tag(:span, '■', :style => "color:#{COLOR[i[:color]]};padding-right:2px;"))
                          concat(i[:name])
                        end)
                 concat(content_tag(:div, i[:value], :class => 'col-6 num_align'))
               end
             end)
    end
  end

  # パラメータnoticeに値が設定されていれば、アラートとして出力する。
  def notice_output(params, level: :warning, dismissible: true)
    if params[:notice] then
      concat(content_tag(:div, :class => 'row') do
               k = [ 'alert ']
               case level
               when :success then k << 'alert-success'
               when :info    then k << 'alert-info'
               when :warning then k << 'alert-warning'
               when :danger  then k << 'alert-danger'
               else               k << 'alert-info'
               end
               k << 'alert-dismissible' if dismissible
               content_tag(:div, :class => k.join(' ')) do
                 if dismissible then
                   concat(content_tag(:button, :type => 'button', :class => 'close', :'data-dismiss' => 'alert', :role => 'alert') do
                            content_tag(:span, '&times;'.html_safe, :'aria-hidden' => 'true')
                          end)
                 end
                 concat(h(params[:notice]))
               end
             end)
    end
  end

  # 各種アラートを出力する。
  # アラートの内容は、コントローラで@alertsに設定しておく。
  #   :no_categorized_hosts - カテゴリを設定していないホストが存在。
  def alerts_output
    if @alerts then
      if @alerts[:no_categorized_hosts] then
        concat(content_tag(:div, :class => 'alert alert-info mb-0 s-shadow-v6 font-weight-bold text-gray-555', :role => 'alert') do
                 concat(content_tag(:span, '', :class => 'fas fa-exclamation-circle g-font-size-1_14x text-info', :'aria-hidden' => 'true'))
                 concat("\n" + 'カテゴリを設定していないホストが存在しています。カテゴリを設定していない場合、対象ホストの情報が出力されないため、少なくとも「システムカテゴリ」の設定は行ってください。')
                 if controller.controller_name != 'sidfm_hostcategories' or controller.action_name != 'index' then
                   concat(link_to('ホストカテゴリ設定', { :controller => :sidfm_hostcategories, :action => :index}, {:class => 'btn btn-default btn-xs', :role => 'button' }))
                 end
                 
               end)
      end
    end
  end

  # alerts_output用のデータを返す。controllerではこの値を@alertに設定しておく。
  # FIXME: 本当はconcern辺りに入れるべき処理だが、alerts_outputと近い場所に
  #   あった方がよいという判断でここに設置。controllerからview_context経由で
  #   アクセスする。
  def prepare_alerts
    ret = {}
    ret[:no_categorized_hosts] = SidfmdHost.enable_hosts.not_categorized.count > 0
    ret
  end

  # paginator
  def paginator2(base: 'pages', content: nil, method: :get, anchor: 'SummaryTable')
    raise ArgumentError, 'need "content"' unless content
    concat(content_tag(:div, :class => 'mr-2') do
             concat(content_tag(:label, '表示行数:', :class => 'g-font-size-13'))
             concat(content_tag(:select, :class => base + '_change_per_page g-font-size-14', :style => 'border:1px solid #bbb;') do
                      [ 10, 25, 50, 100, 200 ].each do |i|
                        if content[:per_page] == i then
                          concat(content_tag(:option, i, :selected => 'selected'))
                        else
                          concat(content_tag(:option, i))
                        end
                      end
                    end)
           end)

    c_start = content[:offset] + 1 
    f_start_edge = (content[:offset] == 0)
    c_end = content[:offset] + content[:per_page]
    c_end = content[:total_size] if c_end > content[:total_size]
    f_end_edge = (content[:total_size] == c_end)
    offset_prev = content[:offset] - content[:per_page]
    offset_prev = 0 if offset_prev < 0
    offset_next = content[:offset] + content[:per_page]
    offset_next = content[:total_size] - 1 if content[:total_size] <= offset_next

    concat(content_tag(:div, :class => 'mr-2') do
             concat(content_tag(:label, '移動:', :class => 'g-font-size-13'))
             concat(text_field_tag('dummy', c_start, :maxlength => '5', :class => base + '_change_offset_direct g-font-size-14 pagination-input-box', :autocomplete => 'off'))
           end)

    concat(content_tag(:div, :class => 'mx-1') do
             concat(content_tag(:span, "#{c_start}-#{c_end}/#{content[:total_size]}", :class => 'g-font-size-14'))
           end)
    concat(content_tag(:nav, :class => 'ml-1', :'aria-label' => 'Page navigation') do
             concat(content_tag(:ul, :class => 'pagination pagination-sm') do
                      concat(content_tag(:li, :class => 'page-item' + ( f_start_edge ? ' disabled' : '')) do
                               concat(link_to('#', :class => base + '_change_offset_paging page-link', :data => { :offset => offset_prev.to_s }) do
                                        concat(content_tag(:span, :'aria-hidden' => 'true') do
                                                 concat(content_tag(:i, '', :class => 'fa fa-angle-double-left'))
                                               end)
                                      end)
                             end)
                      concat(content_tag(:li, :class => 'page-item' + (f_end_edge ? ' disabled' : '')) do
                               concat(link_to('#', :class => base + '_change_offset_paging page-link', :data => { :offset => offset_next.to_s }, :'aria-label' => '次のページへ') do
                                        concat(content_tag(:span, :'aria-hidden' => 'true') do
                                                 concat(content_tag(:i, '', :class => 'fa fa-angle-double-right'))
                                               end)
                                      end)
                             end)
                    end)
           end)
    target_action = controller.action_name
    if 'index_r' == target_action then
      target_action = 'index'
    end
    concat(form_tag(url_for(:controller => controller.controller_name, :action => target_action, :anchor => anchor), :method => method, :id => base + '_form') do
             concat(hidden_field_tag base + '[offset]', content[:offset])
             concat(hidden_field_tag base + '[per_page]', content[:per_page])
             concat(yield base, content) if block_given?
           end)
  end

  def create_dynamic_table_pages_params_form(opts)
    pages_params = []
    if opts[:category_id] then
      pages_params << hidden_field_tag('pages[category_id]', opts[:category_id], :id => 'pages_category_id')
    end
    if opts[:grp] then
      pages_params << hidden_field_tag('pages[grp]', opts[:grp], :id => 'pages_grp')
    end
    if opts[:idx] then
      pages_params << hidden_field_tag('pages[idx]', opts[:idx], :id => 'pages_idx')
    end
    if opts[:hostname] then
      pages_params << hidden_field_tag('pages[hostname]', opts[:hostname], :id => 'pages_hostname')
    end
    if opts[:timeline_risk_period] then
      pages_params << hidden_field_tag('pages[timeline_risk_period]', opts[:timeline_risk_period], :id => 'pages_timeline_risk_period')
    end
    if opts[:timeline_status_period] then
      pages_params << hidden_field_tag('pages[timeline_status_period]', opts[:timeline_status_period], :id => 'pages_timeline_status_period')
    end
    if ([ :order, :filters, :checked, :all_checked, :offset, :per_page ] - opts.keys).any? then
      logger.info(opts)
      logger.error('check args of def create_dynamic_table_pages_params_form')
      raise Exception
    end
    pages_params << hidden_field_tag('pages[order]', opts[:order], :id => 'pages_order')
    pages_params << hidden_field_tag('pages[offset]', opts[:offset], :id => 'pages_offset')
    pages_params << hidden_field_tag('pages[per_page]', opts[:per_page], :id => 'pages_per_page')
    pages_params << hidden_field_tag('pages[filters]', opts[:filters].to_json, :id => 'pages_filters')
    pages_params << hidden_field_tag('pages[checked]', opts[:checked].to_json, :id => 'pages_checked')
    pages_params << hidden_field_tag('pages[all_checked]', opts[:all_checked], :id => 'pages_all_checked')

    target_action = controller.action_name
    if 'index_r' == target_action then
      target_action = 'index'
    end
    content = form_tag(
      url_for(
        :controller => controller.controller_name,
        :action => target_action,
        :anchor => 'SummaryTable'
      ),
      :method => :post,
      :id => 'pages_form'
    ) do
      pages_params.each do |prm|
        concat(prm)
      end
    end
  end

  # 例外を発生しないJSON.parse
  # 例外になるような不正な文字列を入力した場合、nilを返す。
  def force_json_parse(s)
    ret = nil
    begin
      ret = JSON.parse(s)
    rescue Exception => e
      logger.error(e.message)
      if e.backtrace then
        e.backtrace.each do |i|
          logger.error(i)
        end
      end
    end
    ret
  end

  # テーブルの列ソートの状態表示
  # 使用しなくなったため、削除予定。
  def table_sort_th(order: nil, title: nil, target: nil, type: 'attribute')
    content_tag(:th,
                :class => 'sort',
                :data => {
                  :target => target,
                  :order => order,
                  :type => type,
                  :status => (case order when target + '_asc' then 'asc' when target + '_desc' then 'desc' else 'asc' end)}) do
      h(title) +
        content_tag(:span, '',
                    :class => 'glyphicon glyphicon-sort' + (case order when target + '_asc' then '-by-' + type when target + '_desc' then '-by-' + type + '-alt' else '' end),
                    :aria => { :hidden => 'true' })
    end
  end


  # 数値を配列の添字と見なして文字列に変換する。
  # zero=true にすると、添字＝パラメータv
  # zero=false(default) にすると、添字-1＝パラメータv
  def number_to_string(ary, v, zero = false)
    vi = v.to_i
    (vi >= (zero ? 0 : 1) and vi <= (zero ? ary.size - 1 : ary.size)) ? ary[vi - (zero ? 0 : 1)] : '---'
  end

  SRIS = [ '無', '低', '中', '高', '重大' ].map(&:freeze).freeze
  # COLORSは削除予定
  SRI_COLORS = [ 'danger', 'warning', 'notice', 'info' ].map(&:freeze).freeze

  def display_percent(v)
    v ? "#{sprintf('%.1f', v)}%" : '---'
  end

  def display_sri(v)
    return unless v
    content_tag(:span, :class => "bgc_risk_d#{v} d-block mx-auto sri-btn text-white b_radius") do
      number_to_string(SRIS, v + 1)
    end
  end

  # --- の時に、condの条件でn/aに値を差し替える。
  def na_check(v, cond, str = '---', l = 'n/a')
    if v then
      if str == v and cond then
        l
      else
        v
      end
    else
      v
    end
  end

  # CVSS値の出力。小数点第一位まで必ず出力。
  def cvss(v)
    sprintf('%.1f', (v ? v : 0).to_f)
  end

  RISK_COLORS = [ 'info', 'notice', 'warning', 'danger' ].map(&:freeze).freeze

  def display_risk(v)
    if v then
      content_tag(:span, :class => 'text-' + number_to_string(RISK_COLORS, v.risk_id)) do
        v.name_en
      end
    end
  end

  AREA_COLORS = [ 'notice', 'info', 'warning' ].map(&:freeze).freeze

  def display_area(v)
    if v then
      content_tag(:span, :class => 'text-' + number_to_string(AREA_COLORS, v.area_id)) do
        v.name
      end
    end
  end

  # カラーマップをJavaScriptから参照できる形で出力する。
  def color_map
    COLOR.keys.each do |k|
      concat hidden_field_tag "color[#{k}]", symbol_color(k), :id => "color_#{k}", :disabled => 'disabled'
    end
  end

  # 指標のパラメータ値を表示用の文字列に変換
  def idx2str(idx)
    selector = @hosts ? @hosts : @vuls
    case selector[:idx]
    when :sri then 'SRI'
    when :cvss then 'CVSS'
    when :softek then 'SofTek'
    end
  end

  # 各ページに出力する本日日付
  def show_today
    Time.current.localtime.strftime('%Y/%m/%d')
  end

  # 「対応が必要なホスト数」等で使用する円グラフの描画
  ### FIXME: percentによる制御はまだ未対応
  def circle_graph(percent, opts = {})
    cx = 0.0
    cy = 0.0
    r = 85.0
    content_tag(:svg, opts.update(:width => '150px', :height => '150px',
                :viewBox => '-105 -105 210 210',
                :xmlns => 'http://www.w3.org/2000/svg',
                :version => '1.1')) do
      if percent != 0 and percent != 100 then
        d = 360.0 * percent.to_f / 100.0
        start = -90.0
        
        s = -90.0
        e = -90.0 + d

        x1 = cx + r * Math.cos(s * Math::PI / 180.0)
        y1 = cy + r * Math.sin(s * Math::PI / 180.0)
        x2 = cx + r * Math.cos(e * Math::PI / 180.0)
        y2 = cy + r * Math.sin(e * Math::PI / 180.0)
        dx = x2 - x1
        dy = y2 - y1

        f1 = d > 180.0 ? 1 : 0
        f2 = 1

        concat(content_tag(:path, '',
                           :d => "M #{x1.to_i},#{y1.to_i} a #{r.to_i} #{r.to_i} #{start.to_i} #{f1} #{f2} #{dx.to_i},#{dy.to_i}",
                           :'fill-opacity' => '0',
                           :stroke => '#c12e2a',
                           :'stroke-width' => '30'))

        start = -90.0 + d
        s = -90.0 + d
        e = 270.0

        x1 = cx + r * Math.cos(s * Math::PI / 180.0)
        y1 = cy + r * Math.sin(s * Math::PI / 180.0)
        x2 = cx + r * Math.cos(e * Math::PI / 180.0)
        y2 = cy + r * Math.sin(e * Math::PI / 180.0)
        dx = x2 - x1
        dy = y2 - y1

        f1 = d > 180.0 ? 0 : 1
        f2 = 1

        concat(content_tag(:path, '',
                           :d => "M #{x1.to_i},#{y1.to_i} a #{r.to_i} #{r.to_i} #{start.to_i} #{f1} #{f2} #{dx.to_i},#{dy.to_i}",
                           :'fill-opacity' => '0',
                           :stroke => '#e6e6e6',
                           :'stroke-width' => '30'))
      else
        # 100%と0%は円弧では描画できないので、circleで描画するよう
        # 特別扱いする。
        if percent == 100 then
          concat(content_tag(:circle, '',
                             :cx => cx.to_i.to_s,
                             :cy => cy.to_i.to_s,
                             :r => r.to_i.to_s,
                             :'fill-opacity' => '0',
                             :stroke => '#c12e2a',
                             :'stroke-width' => '30'))
        elsif percent == 0 then
          concat(content_tag(:circle, '',
                             :cx => cx.to_i.to_s,
                             :cy => cy.to_i.to_s,
                             :r => r.to_i.to_s,
                             :'fill-opacity' => '0',
                             :stroke => '#e6e6e6',
                             :'stroke-width' => '30'))
        end
      end
      concat(content_tag(:text, percent.to_s, :y => '20', :x => '-10',
                         :'text-anchor' => 'middle',
                         :'font-weight' => 'bold',
                         :'font-size' => '56'))
      concat(content_tag(:text, '%', :y => '20', :x => '50',
                         :'text-anchor' => 'middle',
                         :'font-size' => '30'))
    end
  end

  # bがtrueの場合' selected'、falseの場合''を返す。
  def selected(b)
    b ? ' selected' : ''
  end

  # 対策予定日設定済表示用
  def display_due_dates_percent(due_dates)
    return '---' unless due_dates
    f_cnt = due_dates.size
    return display_percent(0.0) if f_cnt == 0
    return display_percent(0.0) if due_dates.compact.size == 0
    cnt = due_dates.inject(0) {|r, x| r += 1 if (x and x.size > 0); r }
    display_percent(cnt.to_f / f_cnt.to_f * 100.0)
  end

  def display_due_dates_percent_value(due_dates)
    return nil unless due_dates
    f_cnt = due_dates.size
    return nil if f_cnt == 0
    return 0.0 if due_dates.compact.size == 0
    cnt = due_dates.inject(0) {|r, x| r += 1 if (x and x.size > 0); r }
    cnt.to_f / f_cnt.to_f * 100.0
  end

  # 対応状況表示用
  def display_finished_percent(status_ids)
    return '---' unless status_ids
    f_cnt = status_ids.size
    cnt = status_ids.inject(0) {|r, x|
      r += 1 if SidfmdIssue::STATUS_FINISHED.include?(x)
      r
    }
    display_percent(cnt.to_f / f_cnt.to_f * 100.0)
  end

  def display_finished_percent_value(status_ids)
    return nil unless status_ids
    f_cnt = status_ids.size
    cnt = status_ids.inject(0) {|r, x|
      r += 1 if SidfmdIssue::STATUS_FINISHED.include?(x)
      r
    }
    cnt.to_f / f_cnt.to_f * 100.0
  end

  # ソートアイコンの表示
  def sort_icon(name, value)
    case value
    when name + '_asc' then ' sort-up'
    when name + '_desc' then ' sort-down'
    else ''
    end
  end

  # ユーザの名前表示
  def display_user_name(user)
    user ? "#{user.lastname} #{user.firstname}" : '（存在しないユーザ）'
  end

  #ローカルプロダクト設定画面で使う、dateの表示
  def display_lproduct_date(str_date)
    date = if str_date.kind_of?(Date) then
             SidfmdLproduct.date_to_string(str_date)
           else
             ((str_date.nil? or str_date.size == 0) ? '---' : str_date)
           end
  end

  # CVEのリンクを生成
  def create_cve_link(cve_id)
    #次のリンクから以下に変更 "https://cve.mitre.org/cgi-bin/cvename.cgi?name=#{cve_id.to_s}"
    cve_link = "https://www.cve.org/CVERecord?id=#{cve_id.to_s}"
  end

  # 脆弱性のサマリー表のチェックボックス
  def create_summary_vul_chkbox(html_id, checked)
    content = check_box_tag(
      'table[selected]',
      html_id,
      checked.include?(html_id),
      :class => 'vulSelect',
      :id => 'table_selected' + html_id.to_s
    )
  end

  #ホストのサマリー表のチェックボックス
  def create_summary_host_chkbox(host_id, checked)
    content = check_box_tag(
      'table[selected]',
      host_id,
      checked.include?(host_id),
      :class => 'hostSelect',
      :id => 'table_selected' + host_id.to_s
    )
  end
 
  # サマリー表の対応状況
  def create_summary_taiou(is_warning)
    content_tag(
      :span,
      nil,
      :class => "fas fa-hand-point-right#{is_warning ? '' : ' d-none' }",
      :'aria-hidden' => "true"
    )
  end

  # サマリー表のステータス
  def create_summary_status(status_ids, min_status_id, is_status_bar_necessary = true)
    status_elm = content_tag(
      :span,
      "#{
        case min_status_id
          when SidfmdIssue::STATUS_NOTCONFIRMED then '未対策'
          when SidfmdIssue::STATUS_ONGOING then '対策中'
          when SidfmdIssue::STATUS_FIXED then '対策済'
          when SidfmdIssue::STATUS_NOTAFFECT then '影響無'
        end
      }",
      :class => "status-btn btn_st_#{
        case min_status_id
          when SidfmdIssue::STATUS_NOTCONFIRMED then 'not-confirmed'
          when SidfmdIssue::STATUS_ONGOING then 'ongoing'
          when SidfmdIssue::STATUS_FIXED then 'fixed'
          when SidfmdIssue::STATUS_NOTAFFECT then 'notaffect'
        end
      }"
    )
    content = is_status_bar_necessary ? 
      (content_tag(
        :div,
        :data => { :toggle => 'tooltip', :html => 'true', :boundary => 'window' },
        :title => (
          status_ids.size > 0 ? status_bar(
            140, 12, SidfmdIssue.statusids2count(status_ids), :unit => '件'
          ) : '---'
        )
      ) do
        concat(
          status_elm
        )
      end) :
      status_elm
  end

  # サマリー表のsri
  def create_summary_vul_sri(max_sri, html_id)
    content = (max_sri ? display_sri(max_sri) : '---')
    if html_id < 0 then
      content = content.concat(
        content_tag(
          :a,
          :class => 'win_agent_info_mark',  
          :data => {
            :toggle => 'tooltip',
            :placement => 'right',
            :html => 'true',
            :title => 'Windows 用 SIDfm VM Agent で検出した脆弱性については、SRI ではなく Microsoft のパッチ情報に付与されている Severity を SRI として出力しております'
          }
        ) do
          concat(
            content_tag(
              :i,
              nil,
              :class => 'fas fa-info-circle c_gray_70 g-font-size-0_93x'
            )
          )
        end
      )
    end
    content
  end

  def create_summary_host_sri(max_sri, status_ids, kb)
    content = status_ids.size > 0 ? display_sri(max_sri) : display_sri(SidfmdIssueSri::VALUE_NONE)
    if kb then
      content = content.concat(
        content_tag(
          :a,
          :data => {
            :toggle => "tooltip",
            :placement => "right",
            :html => "true",
            :title => "Windows用SIDfm VM Agentで検出した脆弱性については、SRIではなくMicrosoftのパッチ情報に付与されているSeverityをSRIとして出力しております"
          }
        ) do
          concat(
            content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70")
          )
        end
      )
    end
    content
  end

  # サマリー表のcvss
  def create_summary_vul_win_cvss_info
    content_tag(
      :a,
      :class => 'win_agent_info_mark',
      :data => {
        :toggle => 'tooltip',
        :placement => 'right',
        :html => 'true',
        :title => 'Windows 用 SIDfm VM Agent で検出した脆弱性については、Microsoft のパッチ情報に CVSS を示す情報が含まれていないため、SIDfm VM でも CVSS は出力されません'
      }
    ) do
      concat(
        content_tag(:i, nil, :class => 'fas fa-info-circle c_gray_70 g-font-size-0_93x')
      )
    end
  end

  def create_summary_vul_cvss(vul_view_id, html_id, cvss_score, cvss_label, is_cves, page_name = '')
    content = content_tag(
      :span,
      :class => "show-summary-cves bgc_cvss_#{cvss_label} d-block mx-auto cvss-btn text-white b_radius #{is_cves ? 'cursor-pointer' : 'no-cves'}",
      :data => {
        :vulid => "#{vul_view_id}",
        :getdata => 'true',
        :pagename => "#{page_name}"
      }
    ) do
      if cvss_label == 'notreg' then
        if html_id < 0 then
          concat(
            content_tag(:span, 'n/a', :class => 'text-gray-999').concat(
              is_cves ? content_tag(:i, nil, :class => 'fas fa-caret-right ml-1 show-summary-cves-inner') : ''
            )
          )
        else
          if is_cves then
            concat(
               content_tag(:span, :class => 'ml-2 show-summary-cves-inner') do
                 concat( '---')
                 .concat(
                    content_tag(:i, nil, :class => 'fas fa-caret-right ml-1 show-summary-cves-inner')
                 )
               end
            )
          else
            concat('---')
          end
        end
      else
        concat(cvss(cvss_score)).concat(
          content_tag(:i, nil, :class => 'fas fa-caret-right ml-1 show-summary-cves-inner')
        )
      end
    end
    
    if html_id < 0 then
      content = content.concat(create_summary_vul_win_cvss_info)
    end
    content
  end

  def create_summary_host_cvss(max_cvss_label, max_cvss_score)
    display_max_cvss_score = (max_cvss_label == 'notreg' ? nil : sprintf('%.1f', (max_cvss_score ? max_cvss_score : 0)).to_f)
    content = content_tag(
      :span,
      "#{display_max_cvss_score ? display_max_cvss_score : '---'}",
      :class => "bgc_cvss_#{SidfmdIssueCve.labelling(display_max_cvss_score).to_s} d-block mx-auto sri-btn text-white b_radius"
    )
  end

  def create_summary_vul_detail_cvss(cvss_label, cvss_score, html_id)
    score_text =
      if cvss_label == 'notreg' then
        if html_id < 0 then
          content_tag(
            :span,
            "n/a",
            :class => "text-gray-999"
          )
        else
          "---"
        end
      else
        cvss(cvss_score)
      end
    content = content_tag(
      :span,
      :class => "bgc_cvss_#{cvss_label} d-block mx-auto sri-btn text-white b_radius"
    ) do
      concat(score_text)
    end
    if html_id < 0 then
      content = content.concat(create_summary_vul_win_cvss_info)
    end
    content
  end

  # サマリー表の影響範囲
  def create_summary_vul_area(area_id, html_id)
    content = content_tag(
      :span,
      "#{SidfmdArea.where(:area_id => area_id).first.name}",
      :class => "badge bgc_area_#{area_id == 1 ? 'd1' : 'd3'} text-white g-text-shadow-888"
    )
    if html_id < 0 then
      content = content.concat(
        content_tag(
          :a,
          :class => "win_agent_info_mark",
          :data => {
            :toggle => "tooltip",
            :placement => "right",
            :html => "true",
            :title => "Windows 用 SIDfm VM Agent で検出した脆弱性については、Microsoft のパッチ情報に影響範囲を示す情報が含まれていないため、SIDfm VM でも影響範囲は出力されません"
           }
         ) do
           concat( content_tag(:i, nil, :class => "fas fa-info-circle c_gray_70 g-font-size-0_93x") )
         end
      )
    end
    content
  end

  def create_summary_host_area(area_id, status_ids, kb)
    content =
      if status_ids.size > 0 then
        content_tag(
          :span, 
          "#{SidfmdArea.where(:area_id => area_id).first.name}",
          :class => "badge bgc_area_#{area_id == 1 ? 'd1' : 'd3'} text-white g-text-shadow-888"
        )
      else
        content_tag(
          :span, 
          "#{SidfmdArea.where(:area_id => 0).first.name}",
          :class => "badge bgc_area_d3 text-white g-text-shadow-888"
        )
      end
    if kb then
      content = content.concat(
        content_tag(
          :a,
          :data => {
            :toggle => "tooltip",
            :placement => "right",
            :html => "true",
            :title => "Windows用SIDfm VM Agentで検出した脆弱性>については、Microsoftのパッチ情報に影響範囲を示す情報が含まれていないため、SIDfm VMでも影響範囲は出力されません"
          }
        ) do
          concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70") )
        end
      )
    end
    content
  end

  # サマリー表の攻撃コード
  def create_summary_vul_exploit(exploitable, cvss_label, html_id)
    content =
      if cvss_label != 'notreg' then
        if exploitable then
          content_tag(
            :span,
            "あり",
            :class => "badge badge-danger text-white g-text-shadow-888"
          )
        elsif exploitable.nil? then
          "---"
        else
          content_tag(
            :span,
            "なし",
            :class => "badge badge-success text-white g-text-shadow-888"
          )      
        end
      else
        if html_id < 0 then
          content_tag(:span, "n/a", :class => "text-gray-999")
        else
          "---"
        end
      end
    if html_id < 0 then
      content = content.concat(
        content_tag(
          :a,
          :class => "win_agent_info_mark",
          :data => {
            :toggle => "tooltip",
            :placement => "right",
            :html => "true",
            :title => "Windows 用 SIDfm VM Agent で検出した脆弱性については、Microsoft のパッチ情報に攻撃コードの有無を示す情報が含まれていないため、SIDfm VM でも攻撃コードの有無は出力されません"
          }
        ) do
          concat( content_tag(:i, nil, :class => "fas fa-info-circle c_gray_70 g-font-size-0_93x") )
        end
      )
    end
    content
  end

  def create_summary_host_exploit(exploitable, status_ids, cvss_label, kb)
    content = 
      if status_ids.any? then
        if cvss_label != 'notreg' then
          if exploitable then
            content_tag(:span, "あり", :class => "badge badge-danger text-white g-text-shadow-888")
          else
            content_tag(:span, "なし", :class => "badge badge-success text-white g-text-shadow-888")
          end
        else
          "---"
        end
      else
        "---"
      end
    if kb then
      content = content.concat(  
        content_tag(
          :a,
          :data => {
            :toggle => "tooltip",
            :placement => "right",
            :html => "true",
            :title => "Windows用SIDfm VM Agentで検出した脆弱性については、Microsoftのパッチ情報に攻撃コードの有無を示す情報が含まれていないため、SIDfm VMでも攻撃コードの有無は出力されません"
          }
        ) do
          concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70") )
        end
      )
    end
    content
  end

  def create_summary_detail_exploit(vul_view, status_id, cvss_label)
    content =
      if cvss_label != 'notreg' then
        if SidfmdIssue::STATUS_UNFINISHED.include?(status_id) then
          if vul_view.exploitable then
            content_tag(:span, "あり", :class => "badge badge-danger text-white g-text-shadow-888")
          elsif vul_view.exploitable.nil? then
            "---"
          else
            content_tag(:span, "なし", :class => "badge badge-success text-white g-text-shadow-888")
          end
        else
          "---"
        end
      else
        if vul_view.html_id > 0 then
          "---"
        else
          content_tag(:span, "n/a", :class => "text-gray-999")
        end
      end
    if vul_view.html_id < 0 then
      content = content.concat(
        content_tag(
          :a,
          :class => "win_agent_info_mark",
          :data => {
            :toggle => "tooltip",
            :placement => "right",
            :html => "true",
            :title => "Windows 用 SIDfm VM Agent で検出した脆弱性については、Microsoft のパッチ情報に攻撃コードの有無を示す情報が含まれていないため、SIDfm VM でも攻撃コードの有無は出力されません"
          }
        ) do
          concat(
            content_tag(:i, nil, :class => "fas fa-info-circle c_gray_70 g-font-size-0_93x")
          )
        end
      )
    end
    content
  end

  # サマリー表の経過日数
  def create_summary_elapsed_days(min_created_on_display, min_elapsed_created_on_display)
    content = content_tag(
      :a,
      "#{min_elapsed_created_on_display >= 0 ? "#{min_elapsed_created_on_display}" : "---" }",
      :data => {
        :toggle => "tooltip",
        :title => "検出日 #{min_created_on_display}"
      }
    )
  end

  def create_summary_vul_detail_elapsed_days(created_on, status_id)
    elapsed_created_on = created_on ? created_on : Time.current
    elapsed_days = SidfmdIssue::STATUS_UNFINISHED.include?(status_id) ? 
      (((Time.current - elapsed_created_on) / 24.0 / 60.0 / 60.0).to_i) :
      -1
    content = content_tag(
      :a,
      "#{elapsed_days > 0 ? elapsed_days : "---" }",
      :data => {
        :toggle => "tooltip",
        :title => "#{created_on ? "検出日 #{created_on.localtime.strftime('%Y/%m/%d')}" : '---'}"
      }
    )
  end

  # サマリー表の脆弱性名
  def create_summary_vul_title(title:, short_description:, html_id:, create_datetime:, update_datetime:, pages_info: {}, link_opts: {})
    created_date = "#{create_datetime ? create_datetime.strftime('%Y/%m/%d') : '-'}"
    updated_date = "#{update_datetime ? update_datetime.strftime('%Y/%m/%d') : '-'}"
    if created_date == updated_date then
      updated_date = '-'
    end
    created_date =  "登録日: #{created_date}"
    updated_date =  "最終更新日: #{updated_date}"

    content = content_tag(
      :div,
      :class => "d-flex"
    ) do
      concat(
        content_tag(
          :div,
          :class => "text-truncate"
        ) do
          concat(
            link_to(
              pages_info.merge(
                { :controller => :sidfm_vulnerability, :action => :index, :id => html_id }
              ),
              link_opts
            ) do 
              concat(
                content_tag(
                  :div, 
                  :class => "inline-block max-width-100 text-truncate",
                  :data => {
                    :html => true,
                    :toggle => "tooltip",
                    :title => "#{created_date}<br>#{updated_date}#{(short_description && short_description.length > 0) ? "<br>#{html_escape(short_description)}" : ''}",
                    :boundary => "window"
                  }
                ) do
                  concat(
                    content_tag(
                      :span, 
                      "[#{html_id > 0 ? "#{html_id}" : "KB#{-html_id}"}]#{title}",
                      :class => "align-middle",
                      :title => "#{title}"
                    )
                  )
                end
              )
            end
          )
        end
      )
      .concat(
        content_tag(
          :div,
          :class => "ml-auto content-info-btn show-information-content",
          :data => {
            :local => (html_id >= SidfmdLocalVul::MIN_HTMLID and html_id <= SidfmdLocalVul::MAX_HTMLID),
            :htmlid => html_id,
            :toggle => "tooltip",
            :title => "",
            :html => true,
            :boundary => "window",
            :trigger => "manual",
            :template => '<div class="tooltip max-width-fit-content" role="tooltip" style="opacity: 1 !important; z-index:10;"><div class="arrow"></div><div class="tooltip-inner max-width-fit-content p-0"></div></div>'
          }
        ) do
          concat(
            content_tag(:span, "詳細")
          )
          .concat(
            content_tag(
              :div,
              :class => "spinner-border text-secondary show-information-content-loading loading-content-mark",
              :role => "status",
              :style => "display: none;"
            ) do
              concat(
                content_tag(:span, "Loading...", :class => "sr-only")
              )
            end
          )
        end
      ) #render 'common/vuln_information_content_button', html_id: html_id)
    end
  end

  # サマリー表の脆弱性詳細情報へのリンク
  def create_summary_link_to_vul_detail(html_id)
    content = content_tag(
      :span,
    ) do
      concat(
        link_to(
          {
            :controller => :sidfm_vulnerability,
            :action => :index,
            :id => html_id
          },
          :title => '脆弱性の詳細説明ページへ',
          :target => '_blank'
        ) do
          concat( content_tag(:i, nil, :class => "far fa-caret-square-right") )
        end
      )
    end
  end

  # サマリー表のホスト
  def create_summary_hosts(hosts, html_id, category_id)
    content =
      if hosts.size > 1 then
        content_tag(
          :div,
          :class => "d-inline-block balloon-dropdown open-hosts-list",
          :data => { :htmlid => "#{html_id}" }
        ) do
          concat(
            button_tag(
              :type => "button",
              :class => "create-hosts-list-ballon dropdown-without-arrow btn btn-xs btn-light-nb text-nowrap text-left",
              :data => { :htmlid => "#{html_id}", :filtercategoryid => "#{category_id}" },
              :style => "padding-left:0.3em!important; padding-right:0.3em!important; "
            ) do
              concat(
                content_tag(
                  :span,
                  :class => "d-inline-flex",
                  :style => "width:7.5em!important;"
                ) do
                  concat(
                    content_tag(
                      :span,
                      "#{hosts.first[:display_name]}",
                      :id => "hosts-first-#{html_id}",
                      :style => "overflow:hidden;"
                    )
                  )
                end
              )
              .concat(
                content_tag(
                  :span,
                  "#{hosts.size}",
                  :id => "hosts-size-#{html_id}",
                  :class => "badge badge-pill badge-gray-999",
                  :style => "margin-left: 1px;"
                )
              )
            end
          )
          .concat(
            content_tag(
              :div, :class => "dropdown-menu"
            ) do
              concat(
                content_tag(
                  :div, :class => "balloon-menu"
                ) do
                  concat(
                    content_tag(:div, nil, :class => "balloon-arrow")
                  )
                  .concat(
                    content_tag(
                      :div, :class => "dropdown-item", :data => { :itemslist => true }
                    ) do
                      concat( content_tag(:div, nil, :id => "hosts-list-#{html_id}", :class => "list-group") )
                    end
                  )
                end
              )
            end
          )
        end
      elsif hosts.size == 1 then
        content_tag(
          :div,
          :class => "text-truncate d-inline-block max-width-100",
          :data => {
            :toggle => "tooltip",
            :placement => "right",
            :boundary => "window",
            :html => "true",
            :title => "ホスト名: #{hosts.first[:hostname]}<br/>表示名: #{hosts.first[:ext_name] ? hosts.first[:ext_name] : '(未設定)'}"
          }
        ) do
          concat(
            content_tag(
              :span,
              :class => "align-middle d-inline-block",
              :style => "max-width:125px;"
           ) do
             concat(
               link_to(hosts.first[:display_name], { :controller => :sidfm_host, :action => :index, :id => hosts.first[:sidfmd_host_id] })
             )
           end
         )
        end
      end
  end

  def create_summary_hostname(host_view, pages_info = {})
    sidfmd_host = SidfmdHost.enable_hosts.where(:id => host_view.sidfmd_host_id).first
    content = content_tag(
      :div,
      :class => "text-truncate"
    ) do
      concat(
        if sidfmd_host then
          link_to(
            pages_info.merge({ :controller => :sidfm_host, :action => :index, :id => host_view.sidfmd_host_id })
          ) do
            concat(
              content_tag(
                :span,
                "#{host_view.display_name}",
                :class => "align-middle",
                :data => {
                  :toggle => "tooltip",
                  :placement => "right",
                  :html => "true",
                  :boundary => "window",
                  :title => "ホスト名: #{host_view.hostname}<br/>表示名: #{host_view.ext_name ? host_view.ext_name : '(未設定)'}"
                }
              )
            )
          end
        else
          content_tag(
            :span,
            "#{host_view.display_name ? host_view.display_name : '(未登録)'}",
            :class => "d-inline-block text-truncate g-width-12em align-middle",
            :data => {
              :toggle => "tooltip",
              :placement => "top",
              :title => "同期待機中"
            }
          )
        end
      )
    end
  end
 
  def create_summary_vul_detail_hostname(host_view)
    content = content_tag(
      :div,
      :class => "text-truncate w-100"
    ) do
      concat(
        link_to(
          { :controller => :sidfm_host, :action => :index, :id => host_view.sidfmd_host_id },
          { :class => 'link-to-detail', 'data-hostid' => host_view.sidfmd_host_id }
        ) do
          concat(
            content_tag(
              :span,
              "#{host_view.display_name}",
              :class => "align-middle",
              :data => {
                :toggle => "tooltip",
                :html => "true",
                :placement => "right",
                :title => "ホスト名: #{host_view.hostname}<br/>表示名: #{host_view.ext_name ? host_view.ext_name : '(未設定)'}"
              }
            )
          )
        end
      )
    end
  end

  # サマリー表のカテゴリ
  def create_summary_vul_categories(system_categories, custom_categories, html_id, category_id, prior_category)
    cat_size = system_categories.size + custom_categories.size
    sc = system_categories.first
    content = ''

    if sc then
      if cat_size > 1 then
        content = content_tag(
          :div, :class => "d-inline-block balloon-dropdown"
        ) do
          concat(
            button_tag(
              :type => "button",
              :class => "create-categories-list-balloon dropdown-without-arrow btn btn-xs btn-light-nb text-nowrap text-left",
              :data => { :htmlid => html_id, :filtercategoryid => category_id },
              :style => "padding-left:0.3em!important; padding-right:0.3em!important;"
            ) do
              concat(
                content_tag(
                  :span,
                  :class => "d-inline-flex",
                  :style => "width:6.0em!important;"
                ) do
                  concat(
                    content_tag(
                      :span,
                      ((prior_category == 'custom' and custom_categories.any?) ? custom_categories.first[:name] : sc[:name]),
                      :id => "categories-first-#{html_id}",
                      :style => "overflow:hidden;",
                    )
                  )
                end
              )
              .concat(
                content_tag(
                  :span,
                  "#{cat_size}",
                  :id => "categories-size-#{html_id}",
                  :class => "badge badge-pill badge-gray-999",
                  :style => "margin-left: 1px;"
                )
              )
            end
          )
          .concat(
            content_tag(
              :div, :class => "dropdown-menu"
            ) do
              concat(
                content_tag(
                  :div, :class => "balloon-menu"
                ) do
                  concat(
                    content_tag(
                      :div, nil, :class => "balloon-arrow"
                    )
                  )
                  .concat(
                    content_tag(
                      :div, :class => "dropdown-item", :data => { :itemslist => true }
                    ) do
                      concat(
                        content_tag(:div, nil, :id => "categories-list-#{html_id}", :class => "list-group")
                      )
                    end
                  )
                end
              )
            end
          )
        end
      else
        t_cat = SidfmdCategory.find_by(:id => sc[:sidfmd_category_id])
        content = content_tag(
          :span,
          :class => "align-middle text-left d-inline-block text-truncate g-width-8em g-font-size-0_86x",
          :data => {
            :toggle => "tooltip",
            :title => "#{t_cat ? t_cat.desc : ''}",
            :placement => "right"
          }
        ) do
          concat(
            link_to sc[:name], { :action => :index, 'pages[category_id]' => sc[:sidfmd_category_id] }
          )
        end
      end
    end
    content
  end

  def create_summary_host_categories(sidfmd_host_id, system_category_id, system_category_name, custom_categories, prior_category, opts = {})
    content = ''
    data = { :hostid => sidfmd_host_id }
    data[:htmlid] = opts[:html_id] if opts[:html_id]

    if custom_categories.any? then
      content = content_tag(
        :div,
        :class => "d-inline-block balloon-dropdown"
      ) do
        concat(
          button_tag(
            :type => "button",
            :class => "create-categories-list-balloon dropdown-without-arrow " + (opts[:button_class] ? opts[:button_class] : "btn btn-xs btn-light-nb text-nowrap text-left"),
            :data => data,
            #:style => "padding-left:0.4em!important;"
          ) do
            concat(
              content_tag(:span, :class => "d-inline-flex", :style => "width:6.0em!important;") do
                concat(
                  content_tag(
                    :span,
                    "#{prior_category == 'custom' ? custom_categories.first[:name] : system_category_name}",
                    :id => "categories-first-#{sidfmd_host_id}",
                    :style => "overflow:hidden;"
                  )
                )
              end
            )
            .concat(
              content_tag(
                :span,
                "#{custom_categories.size + 1}",
                :id => "categories-size-#{sidfmd_host_id}",
                :class => "badge badge-pill badge-gray-999",
                :style => "margin-left: 1px;"
              )
            )
          end
        ).concat(
          content_tag(:div, :class => "dropdown-menu") do
            concat(
              content_tag(:div, :class => "balloon-menu") do
                concat(
                  content_tag(:div, nil, :class => "balloon-arrow")
                ).concat(
                  content_tag(:div, :class => "dropdown-item", :data => { :itemslist => true }) do
                    concat(
                      content_tag(:div, nil, :id => "categories-list-#{sidfmd_host_id}", :class => "list-group")
                    )
                  end
                )
              end
            )
          end
        )
      end
    else
      s_cat = SidfmdCategory.find_by(:id => system_category_id)
      content = content_tag(:span, :class => "align-middle text-left g-font-size-0_86x") do
        concat(
          link_to(
            system_category_name,
            { :action => :index, 'pages[category_id]' =>  system_category_id },
            :data => {
              :toggle => 'tooltip',
              :title => (s_cat ? s_cat.desc : ''),
              :boundary => 'window',
              :placement => 'right'
            }
          )
        )
      end
    end
    content
  end

  # サマリー表のOS名
  def create_summary_host_osname(host_view, undef_text = '---')
    is_os = (
      (host_view.queue_dist_id or host_view.product_id or host_view.version_id) and host_view.osname and host_view.osname.size > 0
    )
    content = content_tag(:div, :class => "text-truncate") do
      concat(
        content_tag(:span, (is_os ? host_view.osname : undef_text), :title => (is_os ? host_view.osname : undef_text))
      )
    end
  end

  # サマリー表の対策予定日設定済率
  def create_summary_duedates_percent(duedates_percent)
    content = content_tag(
      :span,
      "#{display_percent(duedates_percent)}",
      :class => "align-middle"
    )
  end
  
  # サマリー表の対策予定日
  def create_summary_duedate(due_date, status_id)
    is_over_time = (due_date and due_date.strftime('%Y/%m/%d').to_date < Date.current.strftime('%Y/%m/%d').to_date)
    content = content_tag(
      :span,
      "#{due_date ? due_date.strftime('%Y/%m/%d') : "---"}",
      :class => "align-middle#{
        case
        when (is_over_time and SidfmdIssue::STATUS_UNFINISHED.include?(status_id)) then
          ' text-red timeover'
        when (is_over_time and SidfmdIssue::STATUS_FINISHED.include?(status_id)) then
          ' text-cyan g-text-ul'
        else ''
        end
      }"
    )
  end

  # サマリー表の対処率
  def create_summary_vul_fixed_percent(fixed_percent)
    content = content_tag(
      :span, 
      "#{display_percent(fixed_percent)}",
      :class => "align-middle"
    )
  end

  def create_summary_host_fixed_percent(status_ids)
    content = content_tag(
      :span,
      "#{status_ids.any? ? display_finished_percent(status_ids) : '100.0%'}",
      :class => "align-middle"
    )
  end

  # サマリー表のカスタムタグ
  def create_summary_edit_custom_tag_btn(html_id, vul_title, elm_id, sidfmd_host_id)
    data = { :edit => 'ctag', :id => "#{elm_id}", :htmlid => html_id, :title => vul_title }
    data[:hostid] = sidfmd_host_id if sidfmd_host_id
    button_tag(
      '編集',
      :type => 'button',
      :class => 'edit-vul-ctags-btn ml-1',
      :data => data
    )
  end

  # ctagには、サマリー表の列に表示する(最も優先される)カスタムタグのデータを渡す
  def create_summary_custom_tag(ctag, html_id, vul_title, sidfm_roles, **opts)
    content = ''
    elm_id = (opts[:elm_id] || "#{html_id}-#{opts[:sidfmd_host_id]}")
    if ctag then
      data = { :htmlid => html_id, :id => elm_id }
      data[:hostid] = opts[:sidfmd_host_id] if opts[:sidfmd_host_id]
      data[:hostscategoryid] = opts[:hosts_category_id] if opts[:hosts_category_id]
      content = content_tag(
        :div,
        :class => 'd-flex justify-content-center'
      ) do
        concat(
          content_tag(
            :div,
            :class => "d-inline-block balloon-dropdown"
          ) do
            concat(
              button_tag(
                :type => "button",
                :class => "create-ctags-list-balloon dropdown-without-arrow btn btn-xs btn-light-nb text-nowrap text-left",
                :data => data,
                :style => "#{
                  (ctag[:bg_color] && ctag[:bg_color].size > 0) ? " background-color: #{ctag[:bg_color]};" : ""
                }"
              ) do
                concat(
                  content_tag(
                    :span,
                    :class => "d-inline-flex",
                    :style => "width:6.0em!important;#{
                      (ctag[:text_color] && ctag[:text_color].size > 0) ? " color: #{ctag[:text_color]};" : ""
                    }"
                  ) do
                    concat(
                      content_tag(
                        :span,
                        "#{ctag[:keyword]}",
                        :id => "ctags-first-#{elm_id}",
                        :style => "overflow:hidden;"
                      )
                    )
                  end
                )
              end
            ).concat(
              content_tag(:div, :class => "dropdown-menu") do
                concat(
                  content_tag(:div, :class => "balloon-menu") do
                    concat(
                      content_tag(:div, nil, :class => "balloon-arrow")
                    ).concat(
                      content_tag(:div, :class => "dropdown-item", :data => { :itemslist => true }) do
                        concat(
                          content_tag(:div, nil, :id => "ctags-list-#{elm_id}", :class => "list-group")
                        )
                      end
                    )
                  end
                )
              end
            )
          end
        )
        .concat(
          (sidfm_roles.include?(:use) && !opts[:read_only]) ? create_summary_edit_custom_tag_btn(html_id, vul_title, elm_id, opts[:sidfmd_host_id]) : ""
        )
      end
    else
      content = content_tag(
        :div,
        :class => 'd-flex justify-content-center'
      ) do
        concat(
          content_tag(:span, '---', :class => 'summary-vul-ctag')
        )
        .concat(
          (sidfm_roles.include?(:use) && !opts[:read_only]) ? create_summary_edit_custom_tag_btn(html_id, vul_title, elm_id, opts[:sidfmd_host_id]) : ""
        )
      end
    end
    content
  end

  # サマリー表の影響しているパッケージ・プロダクト
  def create_summary_affected_pkgs(pkgs)
    content = content_tag(
      :div,
      :class => "dropdown dropleft"
    ) do
      concat(
        link_to('影響しているパッケージ一覧',
          '#',
          :class => 'dropdown-toggle show-affitems',
          :data => { :affitem => 'pkgs' },
          :style => "text-decoration:none; color:black;",
        )
      )
      .concat(
        content_tag(
          :div,
          :class => "dropdown-menu mr-2 affitems-menu",
          :style => "top:-25px; display:none;"
        ) do
          concat(
            content_tag(:div, :class => "copy-target-pkg-list") do
              concat(
                content_tag(:ul, :class => "dropdown-item-text list-group list-scrollable") do
                  pkgs.each do|pkg|
                    concat( content_tag(:li, pkg[:fullname], :class => "list-group-item") )
                  end
                end
              )
            end
          )
          .concat(
            content_tag(:div, :class => "text-center") do
              concat(
                button_tag(
                  "コピー",
                  :type => "button",
                  :class => "copybtn g-font-size-0_74x copy-pkg-list btn btn-primary btn-xs text-nowrap",
                  :data => { :toggle => "tooltip", :title => "コピーしました", :trigger => "manual" }
                )
              )
            end
          )
        end
      )
    end
  end

  def create_summary_affected_products(products, lproducts)
    content = content_tag(
      :div,
      :class => "dropdown dropleft"
    ) do
      concat(
        link_to(
          '影響しているプロダクト一覧',
          '#',
          :class => 'dropdown-toggle show-affitems',
          :data => { :affitem => 'prds' },
          :style => "text-decoration:none; color:black;"
        )
      )
      .concat(
        content_tag(
          :div,
          :class => "dropdown-menu mr-2 affitems-menu",
          :style => "top:-25px; display:none;"
        ) do
          concat(
            content_tag(:div, :class => "copy-target-prd-list") do
              concat(
                content_tag(:ul, :class => "dropdown-item-text list-group list-scrollable") do
                  (products + lproducts).each do |prd|
                    concat( content_tag(:li, prd[:display_name], :class => "list-group-item") )
                  end
                end
              )
            end
          )
          .concat(
            content_tag(:div, :class => "text-center mt-2") do
              concat(
                button_tag(
                  "コピー",
                  :type => "button",
                  :class => "copybtn g-mb-3 g-font-size-0_74x copy-prd-list btn btn-primary btn-xs text-nowrap",
                  :data => { :toggle => "tooltip", :title => "コピーしました", :trigger => "manual" }
                )
              )
            end
          )
        end
      )
    end
  end

  def create_summary_vul_detail_affected_items(vul, sidfmd_issue)
    pkgs = []
    products = []
    lproducts = []
    if sidfmd_issue then
      pkgs = sidfmd_issue.affect_packages('')
      products = sidfmd_issue.affect_products
      lproducts = sidfmd_issue.affect_lproducts
    end
    content = content_tag(
      :span
    ) do
      concat(
        content_tag(
          :div,
          :class => "dropdown"
        ) do
          concat(
            content_tag(
              :span,
              :class => "detail-info-tooltip-target",
              :data => { :toggle => "tooltip", :title => "コピーしました", :placement => "bottom", :trigger => "manual" }
            ) do
              concat(
                button_tag(
                  "",
                  :type => "button",
                  :class => "btn btn-outline-primary btn-xs dropdown-toggle show-affitems-dropdown-btn",
                  :data => { :toggle => "dropdown" }
                )
              )
              .concat(
                content_tag(:div, :class => "dropdown-menu") do
                  concat(
                    vul.html_id < 0 ?
                    link_to(
                      'Microsoft Update カタログへ',
                      "https://www.catalog.update.microsoft.com/Search.aspx?q=#{"KB#{-vul.html_id}"}",
                      :class => 'dropdown-item',
                      :target => '_blank',
                      :title => '別ウィンドウが開きます'
                    ) :
                    ""
                  )
                  .concat(
                    if vul.html_id > 0 and sidfmd_issue then
                      if pkgs.size > 0 then
                        content_tag(:div, :class => "dropdown-item") do
                          concat( create_summary_affected_pkgs(pkgs) )
                        end
                      else
                        ""
                      end
                    else
                      ""
                    end
                  )
                  .concat(
                    if products.any? || lproducts.any? then
                      content_tag(:div, :class => "dropdown-item") do
                        concat( create_summary_affected_products(products, lproducts) )
                      end
                    elsif sidfmd_issue and sidfmd_issue.html_id >= SidfmdLocalVul::MIN_HTMLID then
                      content_tag(:div, :class => "dropdown-item") do
                        concat(
                          content_tag(
                            :div,
                            "影響するプロダクトとホストの関連が削除されました。次回の脆弱性同期時に、影響するプロダクトの情報は更新されます。", 
                            :class => "dropdown dropleft"
                          )
                        )
                      end
                    else
                      ""
                    end
                  )
                end
              )
            end
          )
        end
      )
    end 
  end

  def create_summary_host_detail_affected_items(vul, sidfmd_issue)
    pkgs = []
    products = []
    lproducts = []
    if sidfmd_issue then
      pkgs = sidfmd_issue.affect_packages('')
      products = sidfmd_issue.affect_products
      lproducts = sidfmd_issue.affect_lproducts
    end
    content = content_tag(
      :span
    ) do
      concat(
        content_tag(
          :div,
          :class => "dropdown"
        ) do
          concat(
            content_tag(
              :span,
              :class => "detail-info-tooltip-target",
              :data => { :toggle => "tooltip", :title => "コピーしました", :placement => "bottom", :trigger => "manual" }
            ) do
              concat(
                button_tag(
                  "",
                  :type => "button",
                  :class => "btn btn-outline-primary btn-xs dropdown-toggle show-affitems-dropdown-btn",
                  :data => { :toggle => "dropdown" }
                )
              )
              .concat(
                content_tag(:div, :class => "dropdown-menu") do
                  concat(
                    if vul.html_id > 0 then
                      link_to(
                        '脆弱性情報の詳細ページへ',
                        { :controller => :sidfm_informations, :action => :vuln, :id => vul.html_id },
                        :class => 'dropdown-item pl-4 dropdown-item-link-to-page',
                        :title => '脆弱性情報の詳細ページへ',
                        :target => '_blank'
                      )
                    else
                      link_to(
                        '脆弱性詳細ページへ',
                        { :controller => :sidfm_vulnerability, :action => :index, :id => vul.html_id },
                        :class => 'dropdown-item dropdown-item-link-to-page',
                        :title => '脆弱性詳細ページへ'
                      )
                    end
                  )
                  .concat(
                    if vul.html_id > 0 and sidfmd_issue then
                      if pkgs.size > 0 then
                        content_tag(:div, :class => "dropdown-item") do
                          concat( create_summary_affected_pkgs(pkgs) )
                        end
                      else
                        ""
                      end
                    else
                      ""
                    end
                  )
                  .concat(
                    if products.any? || lproducts.any? then
                      content_tag(:div, :class => "dropdown-item") do
                        concat( create_summary_affected_products(products, lproducts) )
                      end
                    else
                      ""
                    end
                  )
                end
              )
            end
          )
        end
      )
    end
  end

  # サマリー表の履歴
  def create_summary_vul_detail_journal(vul, host_view, journals_num, user_id)
    content = content_tag(
      :div,
      :class => "action-open-list"
    ) do
      concat(
        button_tag(
          :type => "button",
          :class => "btn-xs btn-light-nb text-nowrap g-width-5em mx-auto d-block action-open-history",
          :data => {
            :hostid => host_view.sidfmd_host_id,
            :hostname => host_view.hostname,
            :displayname => host_view.display_name,
            :journaltitle => SidfmdVulView.display_title(vul.title),
            :userid => user_id
          }
        ) do
          concat(
            content_tag(
              :span,
              :class => "tooltip-target",
              :data => { :toggle => "tooltip", :placement => "bottom", :title => "コピーしました", :trigger => "manual" }
            ) do
              concat(
                content_tag(
                  :span,
                  "#{journals_num}",
                  :class => "badge badge-pill badge-gray-999",
                  :id => "journal-amount-host#{host_view.sidfmd_host_id}",
                  :data => { :journalnum => journals_num }
                )
              )
            end
          )
        end
      )
    end
  end

  def create_summary_host_detail_history(vul, journals_num, user_id)
    content = content_tag(
      :div,
      :class => "action-open-list"
    ) do
      concat(
        button_tag(
          :type => "button",
          :class => "btn-xs btn-light-nb text-nowrap g-width-5em mx-auto d-block action-open-history",
          :data => {
            :journaltitle => SidfmdVulView.display_title(vul.title),
            :htmlid => vul.html_id,
            :userid => user_id
          }
        ) do
          concat(
            content_tag(
              :span,
              :class => "tooltip-target",
              :data => { :toggle => "tooltip", :placement => "bottom", :title => "コピーしました", :trigger => "manual" }
            ) do
              concat(
                content_tag(
                  :span,
                  "#{journals_num}",
                  :class => "badge badge-pill badge-gray-999",
                  :id => "journal-amount-html#{vul.html_id}",
                  :data => { :journalnum => journals_num }
                )
              )
            end
          )
        end
      )
    end
  end

  # サマリー表のステータス・対策予定日(単体)変更
  def create_summary_detail_action(view, status_id, modal_edit_status_selector, modal_edit_duedate_selector, target)
    status_data = {
      :action => "single",
      :toggle => "modal",
      :target => "#{modal_edit_status_selector}",
    }
    duedate_data = {
      :action => "single",
      :target => "#{modal_edit_duedate_selector}",
    }

    if target == 'host' then
      status_data[:hostid] = view.sidfmd_host_id
      duedate_data[:hostid] = view.sidfmd_host_id
    else
      status_data[:htmlid] = view.html_id
      duedate_data[:htmlid] = view.html_id
    end
    content = content_tag(
      :div,
      :class => "d-flex justify-content-between align-items-center"
    ) do
      concat(
        button_tag(
          "ステータス変更",
          :class => "btn btn-primary btn-xs mx-auto d-block text-nowrap",
          :data => status_data
        )
      )
      .concat(
        if SidfmdIssue::STATUS_UNFINISHED.include?(status_id) then
          button_tag(
            "対策予定日変更",
            :class => "btn btn-primary btn-xs mx-auto d-block text-nowrap",
            :data => duedate_data
          )
        else
          button_tag(
            "対策予定日変更",
            :class => "btn btn-primary btn-xs mx-auto d-block text-nowrap",
            :disabled => "disabled"
          )
        end
      )
    end
  end
  
  # ホスト一覧のdropdown(データは非同期で取得しない)
  def create_hosts_balloon(hosts)
    content = content_tag(
      :div,
      :class => "d-block balloon-dropdown"
    ) do
      concat(
        button_tag(
          :type => "button",
          :class => "create-balloon dropdown-without-arrow btn btn-xs btn-light-nb2 g-font-size-1x text-nowrap text-left",
          :style => "padding-left: 0.4em !important;border-radius:6px;"
        ) do
          concat(
            content_tag(
              :span, 
              :class => "d-inline-flex"
            ) do
              concat(
                content_tag(:span, hosts.first[:display_name], :class => "text-truncate g-width-9em")
              )
            end
          )
          .concat(
            content_tag(
              :span,
              hosts.size,
              :class => "ml-auto badge badge-pill badge-gray-999",
              :data => { :datasize => hosts.size },
              :style => "margin-left: 1px;"
            )
          )
        end
      )
      .concat(
        content_tag(
          :div, :class => "dropdown-menu"
        ) do
          concat(
            content_tag(
              :div, :class => "balloon-menu"
            ) do
              concat(
                content_tag(:div, nil, :class => "balloon-arrow")
              )
              .concat(
                content_tag(
                  :div, :class => "dropdown-item"
                ) do
                  concat(
                    content_tag(
                      :div, :class => "list-group"
                    ) do
                      hosts.each do |host|
                        concat(
                          content_tag(
                            :div, :class => "list-group-item text-truncate g-max-width-12em"
                          ) do
                            concat(
                              content_tag(
                                :span,
                                :data => {
                                  :toggle => "tooltip",
                                  :html => "true",
                                  :title => "ホスト名: #{host[:hostname]}<br/>表示名: #{host[:ext_name] ? host[:ext_name] : "(未>設定)"}",                             
                                  :placement => "right",
                                  :boundary => "window"
                                } 
                              ) do
                                concat(
                                  link_to(
                                    host[:display_name],
                                    { :controller => :sidfm_products, :action => :index },
                                    :class => 'search-host-by-display-name',
                                    :data => { :displayname => host[:display_name] }
                                  ) 
                                ) 
                              end 
                            ) 
                          end 
                        ) 
                      end 
                    end
                  ) 
                end 
              ) 
            end 
          ) 
        end 
      ) 
    end 
  end

  # カテゴリ一覧のdropdown(データは非同期で取得しない)
  def create_categories_balloon(system_categories, custom_categories, prior_category, additional_catname_data_attr = {})
    all_cats = prior_category == 'custom' ? (custom_categories + system_categories) : (system_categories + custom_categories)

    content = content_tag(
      :div,
      :class => "balloon-dropdown"
    ) do
      concat(
        button_tag(
          :type => "button",
          :class => "create-balloon dropdown-without-arrow btn btn-xs btn-light-nb2 text-nowrap g-width-9em my-1 dropdown-toggle caret-off"
        ) do
          concat(
            content_tag(
              :span,
              :class => "d-inline-flex",
              :style => "width:6em!important;"
            ) do
              concat(
                content_tag(
                  :span,
                  "#{all_cats.first.name}",
                  :style => "overflow:hidden;"
                )
              )
            end
          )
          .concat(
            content_tag(
              :span,
              "#{all_cats.size}",
              :class => "badge badge-pill badge-gray-999",
              :data => { :datasize => all_cats.size },
              :style => "margin-left: 1px;"
            )
          )
        end
      )
      .concat(
        content_tag(
          :div,
          :class => "dropdown-menu"
        ) do
          concat(
            content_tag(
              :div,
              :class => "balloon-menu"
            ) do 
              concat(
                content_tag(:div, nil, :class => "balloon-arrow")
              )
              .concat(
                content_tag(
                  :div,
                  :class => "dropdown-item"
                ) do
                  concat(
                    content_tag(
                      :div,
                      :class => "list-group"
                    ) do
                      all_cats.each do |cat|
                        concat(
                          content_tag(
                            :div,
                            :class => "list-group-item text-truncate g-max-width-12em"
                          ) do
                            concat(
                              link_to(
                                cat.name,
                                '#',
                                :class => 'd-block ignore-click category-change text-truncate dropdown-max-width2',
                                :data => {
                                  :value => cat.id,
                                  :categoryid => cat.id,
                                  :toggle => 'tooltip',
                                  :placement => 'right',
                                  :title => cat.desc,
                                  :boundary => 'window'
                                }.merge(additional_catname_data_attr)
                              )
                            )
                          end
                        )
                      end
                    end
                  )
                end
              )
            end
          )
        end
      )
    end
  end

  # データ一覧のドロップダウン
  def create_summary_items_balloon(items, item_size, display_name, **opts)
    content = content_tag(
      :div,
      :class => "d-inline-block balloon-dropdown"
    ) do
      concat(
        button_tag(
          :type => "button",
          :class => "create-balloon text-left dropdown-without-arrow btn btn-xs btn-light-nb2 align-middle my-1 text-nowrap #{opts[:button_class] ? "#{opts[:button_class]}" : "g-width-12em"}",
          :data => opts[:button_data],
        ) do
          concat(
            content_tag(
              :span,
              :class => "d-inline-flex balloon-text-area",
              :style => (opts[:button_inner_text_style] || "width:9em!important;")
            ) do
              concat(
                content_tag(
                  :span,
                  "#{display_name}",
                  :style => "overflow:hidden;",
                  :data => { :displaytext => display_name }
                )
              )
            end
          )
          .concat(
            content_tag(
              :span,
              "#{item_size}",
              :class => "badge badge-pill badge-gray-999",
              :style => "margin-left: 1px;", 
              :data => { :itemsize => item_size }
            )
          )
        end
      )
      .concat(
        content_tag(
          :div, :class => "dropdown-menu"
        ) do
          concat(
            content_tag(
              :div, :class => "balloon-menu", :style => opts[:balloon_menu_style]
            ) do
              concat(
                content_tag(:div, nil, :class => "balloon-arrow")
              )
              .concat(
                content_tag(
                 :div, :class => "dropdown-item"
                ) do
                  concat(
                    content_tag(
                      :div, :class => "list-group", :data => { :itemlist => true }
                    ) do
                      items.each do |item|
                        concat(yield(item)) if block_given?
                      end
                    end
                  )
                end
              )
            end
          )
        end
      )
    end
  end

end
