#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmLproductsHelper

  # サマリー表のチェックボックス
  def create_summary_lproduct_chkbox(lproduct_id, checked)
    content = check_box_tag(
      nil,
      lproduct_id,
      checked.include?(lproduct_id),
      :class => 'SelectLproduct',
    )
  end

  # サマリー表の優先順位
  def create_summary_lproduct_priority(priority)
    content_tag(:span, priority)
  end

  # サマリー表のプロダクト名
  def create_summary_lproduct_name(lproduct)
    content = lproduct.flg_enable ?
      link_to(
        lproduct.name,
        { :controller => :sidfm_informations, :action => :product, :id => "lproduct#{lproduct.id}" },
      ) :
      content_tag(:span, lproduct.name)
  end

  # サマリー表のバージョン
  def create_summary_lproduct_versions(versions)
    content =
      if versions.size > 1 then
        content_tag(
          :div,
          :class => "d-inline-block balloon-dropdown"
        ) do
          concat(
            button_tag(
              :type => "button",
              :class => "create-balloon dropdown-without-arrow btn btn-xs btn-light-nb2 g-font-size-1x text-nowrap text-left",
              :style => "padding-left: 0.4em !important;border-radius:6px;"
            ) do
              concat(
                content_tag(
                  :span, :class => "d-inline-flex"
                ) do
                  concat( content_tag(:span, versions.first[:version], :class => "text-truncate g-width-8em") )       
                end
              )
              .concat(
                content_tag(
                  :span,
                  "#{versions.size}",
                  :class => "ml-auto badge badge-pill badge-gray-999",
                  :style => "margin-left: 1px;"
                )
              )
            end
          )
          .concat(
            content_tag(
              :div, :class => "dropdown-menu"
            ) do
              concat(
                content_tag(
                  :div, :class => "balloon-menu"
                ) do
                  concat(
                    content_tag(:div, nil, :class => "balloon-arrow")
                  )
                  .concat(
                    content_tag(
                      :div, :class => "dropdown-item"
                    ) do
                      concat(
                        content_tag(
                          :div, :class => "list-group"
                        ) do
                          versions.each do |version|
                            concat(
                              content_tag(
                                :div, :class => "list-group-item text-truncate g-max-width-10em"
                              ) do
                                concat(
                                  content_tag(
                                    :span,
                                    :data => {
                                      :toggle => "tooltip",
                                      :title => "#{version[:version]}",
                                      :placement => "right",
                                      :boundary => "window"
                                    }
                                  ) do
                                    concat(
                                      link_to(
                                        version[:version],
                                        '#', #{ :controller => :sidfm_lproducts, :action => :index },
                                        :class => 'search-versions-by-version',
                                        :data => {:version => version[:version] }
                                      )
                                    )
                                  end
                                )
                              end
                            )
                          end
                        end
                      )
                    end
                  )
                end
              )
            end
          )
        end
      elsif versions.size == 1 then
        link_to(
          versions.first[:version],
          '#', #{ :controller => :sidfm_lproducts, :action => :index },
          :class => 'g-pl-5 search-versions-by-version',
          :data => { :version => versions.first[:version] }
        )
      else
        content_tag(
          :div,
          "登録されているバージョンはありません",
          :class => "g-line-height-1_2 g-font-size-0_93x g-pl-5"
        )
      end
  end

  # サマリー表のEOL日付
  def create_summary_lproduct_eol_date(lproduct)
    content_tag(
      :span,
      "#{display_lproduct_date(lproduct.eol_date)}",
      :class => "d-inline-block#{lproduct.is_support_end ? ' text-danger' : '' }"
    )
  end

  # サマリー表の有効無効
  def create_summary_lproduct_status(flg_enable)
    content = flg_enable ?
      content_tag(:span, "有効", :class => "badge badge-success g-font-size-0_93x font-weight-normal text-nowrap") :
      content_tag(:span, "無効", :class => "badge badge-secondary g-font-size-0_93x font-weight-normal text-nowrap") 
  end
 
  # サマリー表の操作
  def create_summary_lproduct_edit(lproduct, versions)
    versions.each{|v| v.delete(:fullname)}
    content = content_tag(
      :div,
      :class => "d-flex justify-content-center"
    ) do
      concat(
        button_tag(
          '編集',
          :type => 'button',
          :class => 'mr-1 btn btn-sm btn-edit edit-lproduct text-nowrap',
          :data => {
            :edit => 'lproduct',
            :lproductid => lproduct.id,
            :lproductname => lproduct.name,
            :priority => lproduct.priority,
            :eoldate => (lproduct.eol_date ? SidfmdLproduct.date_to_string(lproduct.eol_date) : nil),
            :lproductstatus => lproduct.flg_enable,
            :versions => versions.to_json
          }
        )
      )
      .concat(
        form_tag(
          { :controller => :sidfm_lproducts, :action => :delete },
          :method => 'post',
          :id => "form-delete-#{lproduct.id}"
        ) do
          concat(
            button_tag(
              "削除",
              :type => "button",
              :class => "btn btn-sm btn-danger delete-lproduct text-nowrap",
              :data => { :lprodid => "#{lproduct.id}" }
            )
          )
          .concat( hidden_field_tag("form[delete_id]", "#{lproduct.id}") )
        end
      )
    end
  end

end
