#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdIssueSri < ActiveRecord::Base
  belongs_to :sidfmd_issue

  scope :enable_hosts, -> { joins(:sidfmd_issue).joins(:sidfmd_issue => [ :sidfmd_host ]).where(:sidfmd_hosts => { :flg_delete => false }) }
  scope :enable_category, -> { joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => SidfmdCategoriesHost.enable_hosts.enable_category_hosts.select(:sidfmd_host_id) }) }

  RISK_NONE = '無'.freeze
  VALUE_NONE = 0

  RISK_LOW = '低'.freeze
  VALUE_LOW = 1

  RISK_MEDIUM = '中'.freeze
  VALUE_MEDIUM = 2

  RISK_HIGH = '高'.freeze
  VALUE_HIGH = 3

  RISK_CRITICAL = '重大'.freeze
  VALUE_CRITICAL = 4

  RISK_FIXED = '済'.freeze
  VALUE_FIXED = 5

  LABELS = [ :none, :low, :medium, :high, :critical ].map(&:freeze).freeze
  VALUES = [ VALUE_CRITICAL, VALUE_HIGH, VALUE_MEDIUM, VALUE_LOW, VALUE_NONE ].freeze
  RISKS = [  RISK_NONE, RISK_LOW, RISK_MEDIUM, RISK_HIGH, RISK_CRITICAL ].freeze

  def self.label2value(label)
    idx = LABELS.index(label)
    idx ? idx : 0
  end

  def max_sri
  end

  # 各ホストのSRI値を、[ sidfmd_host_id, risk_id ] の形で配列として返す。
  # [ [ sidfmd_host_id, risk_id ], ... ]
  def self.sri_count
    joins(:sidfmd_issue).group(:sidfmd_host_id, :risk_id).count
  end

  # 各ホストの最大SRI値を、{ sidfmd_host_id => risk_id, ... } の形でハッシュとして返す。
  def self.maxsri_count
    ret = {}
    hsc = sri_count
    hsc.each_key do |k|
      if ret.has_key?(k[0]) then
        ret[k[0]] = k[1] if ret[k[0]] > k[1]
      else
        ret[k[0]] = k[1]
      end
    end
    ret
  end
  
  # 最大SRI毎のホスト数を返す。
  # 最大SRIは、ホスト毎に最小値（危険度高）を求めてその値(risk_id)を使用する。
  # { risk_id => ホスト数, ... }
  def self.maxsri_all_count
    ret = { 1 => 0, 2 => 0, 3 => 0, 4 => 0 }
    mc = maxsri_count
    mc.each_value do |v|
      ret[v] += 1
    end
    ret
  end

end
