# -*- coding: utf-8 -*-

require 'syslog_ruby'

class SidfmdRuleASendSyslog < ActiveRecord::Base
  belongs_to :sidfmd_rule_action

  PROTOCOL_UDP = 'udp'.freeze
  PROTOCOL_TCP = 'tcp'.freeze
  PROTOCOLS = [ PROTOCOL_UDP, PROTOCOL_TCP ].freeze

  FACILITIES = SyslogRuby::Facility.constants.find_all {|x| /^LOG/ !~ x.to_s }.map(&:to_s).map(&:freeze).freeze
  SEVERITIES = SyslogRuby::Severity.constants.find_all {|x| /^(LOG|WARNING)/ !~ x.to_s }.map(&:to_s).map(&:freeze).freeze

  #syslogの操作では使用できないキーワード一覧
  UNUSABLE_KEYWORDS = [
    '%RULE_TIMING%', '%RULE_COND%', '%RULE_ACTION%', '%HOSTINFO_ALL%', '%HOST_LINKS_ALL%',
    '%VULINFO_ALL%', '%VULTITLE_ALL%', '%VUL_LINKS_ALL%', '%SIDfm_LINKS_ALL%', '%MS_LINKS_ALL%',
    '%APP_ALL%', '%HOSTINFO_MATCH%', '%HOST_LINKS_MATCH%', '%VULINFO_MATCH%', '%VULTITLE_MATCH%', '%VUL_LINKS_MATCH%',
    '%SIDfm_LINKS_MATCH%', '%MS_LINKS_MATCH%', '%APP_MATCH%',
    '%VULSYNC_REGI_ALL%', '%VULSYNC_RENEW_ALL%', '%VULSYNC_DEL_ALL%',
    '%VULSYNC_REGI_MATCH%', '%VULSYNC_RENEW_MATCH%', '%VULSYNC_DEL_MATCH%'
  ]

  def self.validate_keywords(text)
    keywords = SidfmdMailer.get_keywords(text)
    if (UNUSABLE_KEYWORDS - keywords.keys).size != UNUSABLE_KEYWORDS.size then
      return false
    else
      return true
    end
  end

  IP_ADDRESS_IDX = '%ip_address%'
  #送信先ホストは、ホスト名とIPアドレスがあり、UI上で表示する要素がホスト名とIPアドレスで異なるため、IPアドレスの場合は、
  #DBに保存する際にIP_ADDRESS_IDXを接頭語に入れるようにしてあります。
  #編集用のparameter設定時に、IP_ADDRESS_IDXが値にあるかどうかでホスト名かIPアドレスかを判断して設定しています。
  def self.get_address(target)
    return target.sub("#{IP_ADDRESS_IDX}", '')
  end
 
end
