#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdVmApikey < ActiveRecord::Base
  belongs_to :user
 
  NAME_LABEL = 'APIキー名'
  APIKEY_LABEL = 'APIキー'
  PERIOD_LABEL = '有効期間'
  USER_LABEL = 'ユーザ'
  ALLOW_UPDATE_LABEL = '更新許可'

  # api_keyを生成する
  # 生成したapi_keyを返す
  # SidfmdRegkey#generate_apikeyと同じ
  def generate_apikey
    # 重複回避
    salt = Sidfm.salt
    flg = true
    api_key = nil
    while flg do
      api_key = SofTekUtils.hexdigest(post: salt)
      unless self.class.find_by_api_key(api_key) then
        self.update_attribute :api_key, api_key
        flg = false
      end
    end
    api_key
  end

  # APIキーの登録
  def self.create_apikey(name, user_id)
    self.create(:name => name, :user_id => user_id)
  end

  # APIキーのキーを削除
  def remove_apikey
    self.update_attribute :api_key, nil
  end

  # キー名の更新
  def update_name(name)
    self.update_attribute :name, name
  end

  # 更新許可の更新
  def update_allow_update(is_allow_update)
    self.update_attribute :allow_update, is_allow_update
  end

  # 有効期間の更新
  def update_period(begin_date, end_date)
    self.update_attributes :begin_date => begin_date, :end_date => end_date
  end

  # ユーザの更新
  def update_user(user_id)
    self.update_attribute :user_id, user_id
  end

  # APIキーの削除
  def self.delete_apikey(id)
    self.transaction do
      apikey = self.lock.find_by_id(id.to_i)
      apikey.destroy if apikey
    end
  end

  # APIキーの一括削除
  def self.bulk_delete_apikey(ids)
    self.transaction do
      self.where(:id => ids).destroy_all
    end
  end

  # 指定したユーザのAPIキーの更新許可を変更
  def self.update_user_apikeys_allow_update(user_ids, allow_update = false)
    self.where(:user_id => user_ids).update_all(allow_update: allow_update)
  end

  # 指定したユーザのAPIキーを削除
  def self.delete_user_apikeys(user_ids)
    self.where(:user_id => user_ids).destroy_all
  end

  # 日付を文字列に変換
  # UI等で、日付をここの表示方法で統一する
  def self.date_to_string(date)
    date.strftime('%Y/%m/%d')
  end

  # ユーザがアクセス・操作可能が可能かどうかを返す
  def is_available_user?(user_id:, user_category_ids:, sidfm_roles:)
    ret = true
    if !sidfm_roles.include?(:host_admin) then
      if self.user_id != user_id.to_i then
        ret = false
      end
    else
      unless sidfm_roles.include?(:admin) then
        #ret = SidfmdCategoriesUser.where(:sidfmd_category_id => user_category_ids)
        #  .where(:user_id => self.user_id)
        #  .exists?
        ret = self.user.categories.where(:id => user_category_ids).exists?
      end
    end
    ret
  end

  # 更新許可を「許可」にできるかどうかを返す
  # 引数のuser_idは、変更先ユーザを指定する場合に渡す
  def is_allowed_allow_update_true?(user_id = nil)
    ret = false
    user = User.find_by(:id => (user_id ? user_id : self.user_id))
    if user then
      ret = !user.roles.where(:name => "SIDfmVM閲覧者").exists?
    end
    ret
  end

  # APIキーで認証を行う。
  # 認証OKならSidfmdVmApikeyのインスタンス、認証NGならnilを返す。
  def self.auth(apikey)
    return nil unless apikey
    today = Date.current
    ret = self
      .where(:api_key => apikey)
      .where("begin_date IS NULL OR begin_date <= ?", today)
      .where("end_date IS NULL OR end_date >= ?", today)
    ret.first
  end

end
