/* coding: utf-8 */
/* Copyright (C) 2018 SofTek Systems, Inc. */

function BootStrapTableCellStyle(value, row, index, field) {
    switch (field){
        case 'chkbox':
            return { classes: 'chk-box' };
        case 'name':
            return { classes: 'cat-name' };
        case 'description':
            return { classes: 'cat-description word-break-all' };
        case 'hostlim':
            return { classes: 'cat-hostlim' };
        case 'class':
            return { classes: 'cat-class' };
        case 'operation':
            return { classes: 'cat-operation position-relative' };
        default:
            return { classes: '' };
    }
}

function BootStrapTableRowStyle(row, index){
    return {
        classes: 'list_alternative'
    }
}

$(function() {

    //インポートエラーがある場合は、エラー内容を見せるために、画面の上部にスクロール位置を設定する。    
    if( $.cookie('import_err') ){
        $('html,body').animate({ scrollTop: 0 }, '1');
    }
    $.removeCookie('import_err');

    const summaryTableId = 'categories-summary-table';
    const summaryTableElm = $('#' + summaryTableId);
    const pageName = 'categories';

    function getCategoryRelatedCtagsinfo(catIds){
        return new Promise((resolve, reject) => {
            $.post(
                $('#api-category-related-ctags-path').val(),
                { ids: JSON.stringify(catIds) },
                function(data){
                    let msg = '';
                    if( data.is_related_ctag && data.is_ctag_related_issue){
                        msg = '削除するカテゴリに紐づくカスタムタグ、及びそのカスタムタグに紐づく脆弱性があります。\n' +
                              'カテゴリの削除を実行すると、そのカスタムタグ、及びカスタムタグと脆弱性の紐付きも削除されます。';
                    }else if( data.is_related_ctag ){
                        msg = '削除するカテゴリに紐づくカスタムタグがあります。\n' +
                              'カテゴリの削除を実行すると、そのカスタムタグも削除されます。';
                    }
                    resolve(msg);
                }
            ).fail(function(){
                alert('エラーが発生しました。');
                $('#pages_form').submit();
            });
        });
    }

    function checkRelatedCatGroups(catIds, actionName){
        return new Promise((resolve, reject) => {
            $.post(
                $('#api-check-related-cat-groups-path').val(),
                { action_name: actionName, ids: JSON.stringify(catIds) },
                function(data){
                    let msg = '';
                    if( data.delete_cat_groups && data.delete_cat_groups.length ){
                        msg = '削除するカテゴリのみが設定されているカテゴリグループがあります。\n' +
                              'カテゴリの削除を実行すると、そのカテゴリグループも削除されます。';
                    }
                    resolve(msg);
                }
            ).fail(function(){
                alert('エラーが発生しました。');
                $('#pages_form').submit();
            });
        });
    }

    // カテゴリ削除 
    // 削除時に、関連しているルールをチェック
    const enableDeleteCategory = () => {
        $(`#${summaryTableId} [data-delcategory]`).on('click', function(){
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                    additionalAlertMessage: '削除に失敗しました。'
                });
                if( roleOrCategoryError ) return;
                var button = $(this);
                var conf_text = button.attr('data-conftext');
                var path = $('#sidfm_internal_api_check_related_rules_path').val();
                var del_category_id = button.attr('data-delcategory');
                const confRelatedCtagMsg = await getCategoryRelatedCtagsinfo([del_category_id]);
                const confCatGroupsMsg = await checkRelatedCatGroups([del_category_id], 'delete');
                let relatedDataConftext = '';
                if( confRelatedCtagMsg && confCatGroupsMsg ) {
                    relatedDataConftext = confRelatedCtagMsg + '\nまた、' + confCatGroupsMsg + 'よろしいですか?';
                }else if( confRelatedCtagMsg ){
                    relatedDataConftext = confRelatedCtagMsg + '\nよろしいですか?';
                }else if( confCatGroupsMsg ){
                    relatedDataConftext = confCatGroupsMsg + '\nよろしいですか?';
                }
                if( relatedDataConftext ){
                    let conf = confirm(relatedDataConftext);
                    if( !conf ){
                        return;
                    }else{
                        conf_text = '';
                    }
                }
                $('body').append(create_loading_window());
                $('#submit-related-rules-item-form').attr({
                    'data-categoryid': del_category_id
                });
      
                var is_related_hosts = false;
                var is_related_users = false;
                var is_related_rules = false;
                var cat_name = button.data('categoryname');
      
                var check_rules = $.ajax({
                    url: path,
                    type: 'POST',
                    data: { 'page_name': 'category', 'delete_id': del_category_id, 'action_idx': 'delete' },
                }).done(function(data){
                    if(Object.keys(data['rule_messages']).length > 0){
                        is_related_rules = true;
                        display_related_rules(data, false);
                        $('#related-rule-info-area').show();
                    }else{
                        $('#related-rule-info-area').hide();
                    }
                });
                var check_hosts = $.ajax({
                    url: $('#check_category_related_hosts_path').val(),
                    type: 'POST',
                    data: { 'category_id': del_category_id }
                }).done(function(hosts){
                    if( hosts.length > 0 ){
                         is_related_hosts = true;
                        $('#related-host-info-area-tbody > *').remove();
                        $('#related-host-info-area').show();
                        var host_elms = hosts.map(function(host){
                            return $('<tr>').attr('class', 'list_alternative').append(
                                $('<td>').append(
                                    $('<div>').attr({
                                        'data-toggle': 'tooltip',
                                        'data-title': ('ホスト名: ' + host.hostname + '<br/>表示名: ' + (host.ext_name ? host.ext_name : '(未設定)')),
                                        'data-html': 'true',
                                        'data-boundary': 'window',
                                        'data-placement': 'left',
                                    }).text(host.ext_name ? host.ext_name : host.hostname)
                                )
                            );
                        });
                        $('#related-host-info-area-tbody').append(host_elms);
                        $('#related-host-info-area-tbody').find('[data-toggle="tooltip"]').tooltip();
                    }else{
                        $('#related-host-info-area').hide();
                    }
                });
      
                var check_users = $.ajax({
                    url: $('#check_category_related_users_path').val(),
                    type: 'POST',
                    data: { 'category_id': del_category_id }
                }).done(function(users){
                    if( users.length > 0 ){
                        is_related_users = true;
                        $('#related-user-info-area').show();
                        $('#related-user-info-area-tbody > *').remove();
                        var user_elms = users.map(function(user){
                            return $('<tr>').attr('class', 'list_alternative').append(
                                $('<td>').append(
                                    $('<div>').text(user.login)
                                ),
                                $('<td>').append(
                                    $('<div>').text(user.lastname + ' ' + user.firstname)
                                )
                            );
                        });
                        $('#related-user-info-area-tbody').append(user_elms);
                    }else{
                        $('#related-user-info-area').hide();
                    }
                });
      
                $.when(check_rules, check_hosts, check_users).done(function(){
                    $('#checking-window').remove();
                    if( is_related_rules || is_related_users || is_related_hosts){
                        $('#related-host-info-content, #related-user-info-content, #related-rule-info-content').hide();
                        $('#related-host-info-btn-icon, #related-user-info-btn-icon, #related-rule-info-btn-icon').removeClass('fa-angle-up')
                                                                                                                  .addClass('fa-angle-down')
                        var title_items = [];
                        if( is_related_hosts ){
                            title_items.push('ホスト');
                        }
                        if( is_related_users ){
                            title_items.push('ユーザ');
                        }
                        if( is_related_rules ){
                            title_items.push('ルール');
                        }
                        $('#modalConfRelatedRules-title').text('「' + cat_name + '」' +  'に関連する' + title_items.join('/') + 'の情報');
                        $('#modalConfRelatedRules').modal('show');
                    }else{
                        if( conf_text ){
                            var conf = confirm(conf_text);
                            if(!conf){
                                return false;
                            }
                        }
                        execute_delete_category(del_category_id);
                    }
                });
            })(); 
        });
    };

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
    const bindSummaryTableRowEvents = () => {
        enable_bootstrap_actions('#' + summaryTableId);
        enable_dropdown_balloon(summaryTableId);
        enableDeleteCategory(); //カテゴリ削除
    };
    enable_dropdown_balloon(); //ページ上部の「こちら」の説明に使用

    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all-cat-ids',
        chkboxSelector: `#${summaryTableId} .categorySelect`,
        allChkboxSelector: `#${summaryTableId} .allSelect`,
        noMatchesText: 'カテゴリが登録されておりません。'
    });

    set_style_for_ActionArea_on_filter_dropdown_event();
    sticky_control('SummaryTable', pageName);
    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) });
 
    // 入力用ダイアログを開く処理
    $('#modal-form').on('show.bs.modal', function(event) {
        var button = $(event.relatedTarget);
        var func = button.data('func');
        var category = parseInt(button.data('category'));
        var title = button.data('title');
        var userid = button.data('userid');
        if (typeof userid === 'undefined' || (userid == "")) {
            userid = '0';
        }
        var name = button.data('name');
        var desc = button.data('desc');
        var users = button.data('users');

        var modal = $(this);
        modal.find('.modal-title .text-truncate').text(title);
        $('#form_userid').val(userid);
        $('#form_name').val(name);
        $('#form_desc').val(desc);
     
        if (func === "new") {
          //$('#form_is_user').prop('checked', 'checked');
          //$('#form_users_selected').val(JSON.stringify([]));
            $('#form_hostlim_field').val(0).prop('disabled', true);
            $('#form_limit_hostlim').prop('checked', false);
            $('#form_unlimit_hostlim').prop('checked', true);
            $('#form_func').val('new');
            $('#form_submit').val('登録');
            $('#form').attr('action', $('#url_new').val());
        } else {
            var hostlim = button.data('hostlim');
            const isLimitHost = hostlim != null;
            $('#form_hostlim_field').val(isLimitHost ? hostlim : 0).prop('disabled', !isLimitHost);
            $('#form_limit_hostlim').prop('checked', isLimitHost);
            $('#form_unlimit_hostlim').prop('checked', !isLimitHost);
            $('#form_id').val(category);
            //$('#form_is_user').prop('checked', parseInt(userid) > 0 ? 'checked' : '');
	          //$('#form_users_selected').val(JSON.stringify(users));
	          $('#form_func').val('mod');
            $('#form_submit').val('更新');
            $('#form').attr('action', $('#url_mod').val());
            $('#prev-name').val(name);
            $('#prev-desc').val(desc);
            $('#prev-hostlim').val(String(hostlim));
        }
        $('#modal-form').modal('show');

	//var temp_users = JSON.parse($('#temp_users').val());
	//var form_users_selected = JSON.parse($('#form_users_selected').val());

	//$('#selected_users > option').remove();
	//$('#unselected_users > option').remove();
	/*for (var i = 0; i < temp_users.length; i++) {
	    $($.inArray(temp_users[i]["id"], form_users_selected) != -1 ? '#selected_users' : '#unselected_users').append($('<option></option>', { value: temp_users[i]["id"], text: temp_users[i]["lastname"] + ' ' + temp_users[i]["firstname"]}));


	}*/
    });

    $('[name="form[switch_hostlim]"]').on('change', function(){
        const isLimit = $(this).val() === 'true';
        $('#form_hostlim_field').prop('disabled', !isLimit);
    });

    // 選択されたユーザを選択済に移動する。
    // 反映にはsubmitが必要。
    /*$('#select-user').on('click', function() {
	return move_selected_items('#form_users_selected',
				   '#unselected_users',
				   '#selected_users', false);
    });*/

    // 選択されたユーザを未選択に移動する。
    // 反映にはsubmitが必要。
    /*$('#unselect-user').on('click', function() {
	return move_selected_items('#form_users_selected',
				   '#selected_users',
				   '#unselected_users', true);
    });*/

    // submit処理
    // 簡易validation含む。完全なvalidationはModel or Controllerで。
    $('#form_submit').on('click', function() {
        const func = $('#form_func').val();
        const value = $('#form_name').val();
        const isLimitHostNum = $('[name="form[switch_hostlim]"]:checked').val() === 'true';
        const LimitHostNum = isLimitHostNum ? $('#form_hostlim_field').val() : null;
        if (value.length === 0) {
            alert('カテゴリ名が入力されていません。');
            return false;
        }
        if( value[0].match(/\s/) || value[value.length - 1].match(/\s/) ){//カテゴリの先頭と末尾に空白を許可しない
            alert('カテゴリ名の先頭と末尾には、空白文字を使用できません。');
            $('#form_name').val(value.trim());
            return false;
        }
        var names = JSON.parse($('#category_names').val());
        /*if (func == 'new' && names) {
            if ($.inArray($('#form_name').val(), names) >= 0) {
                alert('同一の名称のカテゴリが既に存在します。');
                return false;
            }
        }*/
        if ($('#form_desc').val() == '') {
            alert('概要が入力されていません。');
            return false;
        }
        if( isLimitHostNum && (!LimitHostNum.length || LimitHostNum.match(/[^0-9]/)) ){
            alert('ホスト登録上限数を制限する場合は、入力欄に0以上の数値(整数)を入力して下さい。');
            return false;
        }
        //return confirm('登録します。よろしいでしょうか？');
        if( func == 'mod' ){
            var name = $('#form_name').val();
            var desc = $('#form_desc').val();
            const prevLimitHostNum = JSON.parse($('#prev-hostlim').val());
            if( $('#prev-name').val() === name && $('#prev-desc').val() === desc && LimitHostNum == prevLimitHostNum){
                alert('変更がありません。');
                return false;
            }
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                    additionalAlertMessage: '更新に失敗しました。'
                });
                var param = {
                    'form[id]': $('#form_id').val(),
                    'form[name]': name,
                    'form[desc]': desc,
                    'form[hostlim]': LimitHostNum
                };
                var path = $('#form').attr('action');
                $.ajax({
                    url: path,
                    type: 'POST',
                    data: param
                }).done(function(res){
                    if( res['err_message'].length > 0 ){
                        if( res['err_message'] === 'カテゴリの更新に失敗しました。' || res['err_message'] === '更新対象のカテゴリが存在しません。'){
                            alert(res['err_message'] + '画面をリロードします。');
                            $('#pages_form').submit();
                        }else{
                            alert(res['err_message']);
                            return false;
                        }
                    }else{
                        update_view(res['host_ids'], res['html_ids'], false, dynamicSummaryTable);
                    }
                }).fail(function(){
                    alert('カテゴリの更新に失敗しました。');
                    $('#pages_form').submit();
                });
            })();
        }else{
            //新規の場合
            //ホスト数は、変更の場合はajax時にチェックされる
            const maxHostlim = parseInt($('#max-hostlim').val());
            if( isLimitHostNum && parseInt(LimitHostNum) > maxHostlim ){
                alert(`ホスト登録上限数に設定可能な値は、最大${maxHostlim}(ライセンスのホスト数上限値)です。`);
                return false;
            }
            const paramsData = {
                func,
                name: value,
                desc: $('#form_desc').val()
            };
            if( !$('#form_hostlim_field').prop('disabled') ){
                paramsData.hostlim = $('#form_hostlim_field').val();
            };
            dynamicSummaryTable.executeProcess({
                paramsData,
                path: $('#form').attr('action'),
                additionalAlertMessage: '登録に失敗しました。'
            });
        }
        return false;
    });

  function toggle_category_related_info_content(target_content_id, target_icon_id){
    var target_content = $('#' + target_content_id);
    var target_icon = $('#' + target_icon_id);
    if( target_content.is(':visible') ){
        target_content.hide();
        target_icon.removeClass('fa-angle-up').addClass('fa-angle-down');
    }else{
        target_content.show();
        target_icon.removeClass('fa-angle-down').addClass('fa-angle-up');
    }
  }

  $('#toggle-related-host-info-content-btn').on('click', function(){
      toggle_category_related_info_content('related-host-info-content', 'related-host-info-btn-icon');
  });

  $('#toggle-related-user-info-content-btn').on('click', function(){
      toggle_category_related_info_content('related-user-info-content', 'related-user-info-btn-icon');
  });

  $('#toggle-related-rule-info-content-btn').on('click', function(){
      toggle_category_related_info_content('related-rule-info-content', 'related-rule-info-btn-icon');
  });

  function execute_delete_category(category_id){
      (async () => {
          const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
              additionalAlertMessage: '削除に失敗しました。'
          });
          var path = $('#sidfm_categories_delete_path').val();
          $.ajax({
              type: 'POST',
              url: path,
              data: { 'id': category_id }
          }).done(function(result){
              if( result['err_message'].length > 0 ){
                  alert(result['err_message']);
                  location.reload();
              }else{
                  $('#pages_form').attr('action', $('#sidfm_categories_path').val());
                  update_view(result['host_ids'], result['html_ids'], false, dynamicSummaryTable);
              }
          });
      })();
  }
  
  //関連するルールを確認するmodalでOKボタンを押した時にformをsubmitする
  $('#submit-related-rules-item-form').on('click', function(){
        var button = $(this);
        var category_id = button.attr('data-categoryid');
        execute_delete_category(category_id);
  });

  $('.balloon-dropdown').on('shown.bs.dropdown', function(){
        setTimeout(function(){ $('.balloon-card:visible').css({'white-space': 'normal', 'word-break': 'normal'}) }, 10);
  });
 
  //enterでformが送信されるのを防ぐ 
  $('.modal input').on('keydown', function(e) {
      if( (e.which && e.which === 13) || (e.keyCode && e.keyCode === 13) ){
          return false;
      } else {
          return true;
      }
  });

  //export
  $('#export-custom-categories').on('click', function(){
      json_tsv_summary_table_progress_download(
          'tsv',
          $('#sidfm_categories_export_path').val(),
          { dynamicSummaryTable }
      );
  });

  $('#exec-import').on('click', function(){
      var progress_import_path = $('#progress-import-path').val();
      var form_data = new FormData();
      var file = $('#categories_file').get(0).files[0];
      if( !file ){
          alert('ファイルを選択して下さい。');
          return false;
      }
      form_data.append('categories_file', file);

      var progress_window = get_progress_bar();
      $('body').append(progress_window);

      (async () => {
          const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
              additionalAlertMessage: 'インポートに失敗しました。'
          });
          if( roleOrCategoryError ) return;
          var percentage = 0;
          var total_size = 0;
          var executed_size = 0;
          var is_err = false;

          var progress_import = function(){
              $.ajax({
                    url: progress_import_path,
                    type: 'GET',
                }).done(function(res){
                    if( res['is_exception'] ){
                        alert('エラーが発生したため、インポートを中止しました。\n画面をリロードします。');
                        $.cookie('import_err', 'true');
                        $('#pages_form').submit();
                        return false;
                    }
                    if( res['err'] ){
                        is_err = true;
                    }
                    if( res['executed_size'] > 0 ){
                        executed_size += res['executed_size'];
                        var percentage = (executed_size / total_size * 100);
                        if( percentage >= 99 && percentage < 100 ){
                            percentage = 99;
                        }else{
                            percentage = Math.round(percentage);
                        }
                        $('#progress-bar').attr('aria-valuenow', percentage).css('width', percentage + '%').text(percentage + '%');
                        progress_import();
                    }else{
                        if( is_err ){
                            alert('インポートしたファイルにエラーがあるため、インポートを中止しました。\n画面をリロードします。');
                            $.cookie('import_err', 'true');
                            $('#pages_form').submit();
                            return false;
                        }
                        setTimeout(function(){//probress barのbarが満たされるまで待つ
                            location.href = $('#import-path').val();
                        }, 500);
                    }
                }).fail(function(){
                    alert('ファイルのインポートに失敗しました。画面をリロードします。');
                    $('#pages_form').submit();
                });
          };

          //最初にファイルを読み込む
          $.ajax({
              url: $('#read-import-file-path').val(),
              type: 'POST',
              data: form_data,
              processData: false,
              contentType: false
          }).done(function(res){
              if( res['err_message'] ){
                  alert(res['err_message'] + '画面をリロードします。');
                  location.reload();
              }
              if( res['err'] ){
                  alert('ファイルのインポートに失敗しました。画面をリロードします。');
                  location.reload();
              }else{
                  total_size = res['total_size'];
              }
              progress_import();
          }).fail(function(){
              alert('ファイルのインポートに失敗しました。画面をリロードします。');
              location.reload();
          });
      })();
  });

  $('#modalConfRelatedRules').on('hide.bs.modal', function(){
      $(this).find('.Related-rule-table').scrollTop(0);
  });

    activate_sidebar_item('sidebar-item-category-manip')
});


