/* coding: utf-8 */

//BootStrapTableのtdのスタイル。画面ごとに関数を作って設定する
function BootStrapTableCellStyle(value, row, index, field) {
    switch (field){
        case 'chkbox':
            return { classes: 'summary-chk-box-td' };
        case 'content':
            return { classes: 'summary-apikey-td' };
        case 'operation':
            return { classes: 'summary-apikey-action-td overflow-auto' };
        default:
            return { classes: '' };
    }
}

$(function() {
    activate_sidebar_item('sidebar-item-probeapikey-manip');

    const summaryTableId = 'probe-apikeys-summary-table';
    const summaryTableElm = $('#' + summaryTableId);
    const pageName = 'probe-apikey';
    const searchCategoriesPath = $('#sidfm_internal_api_search_categories_path').val();

    const createSearchedCatElms = (categories) => {
        return categories.map(category => {
            const hitStr = category.name;
            return $('<div>').attr({ 
                       'data-categoryname': category.name,
                       'data-categoryid': category.id,
                       'data-toggle': 'tooltip',
                       'data-placement': 'right',
                       'data-boundary': 'window',
                       'data-html': true,
                       'class': 'searched-category-name',
                   }).text(hitStr);
        });
    };

    //削除
    const enableDeleteApikey = () => {
        $('#' + summaryTableId + ' [data-delete="apikey"]').on('click', function(){
            let conf = confirm($(this).attr('data-confirmtext'));
            if( conf ){
                dynamicSummaryTable.executeProcess({
                    paramsData: { id: $(this).attr('data-id') },
                    path: $('#apikey-delete-path').val(),
                    additionalAlertMessage: '削除に失敗しました。'
                });
            }
            return false;
        });
    };

    //更新
    const setModifyName = (triggerElm) => {
        const data = JSON.parse(triggerElm.attr('data-content'));
        $('#modify-apikey-name-field').val(data.name);
        $('#modify-apikey-item-name').show();
    };

    const setModifyCategoryItem = (triggerElm) => {
        const data = JSON.parse(triggerElm.attr('data-content'));
        const categoryData = data.category;
        $('#modify-apikey-search-category-field').val(categoryData.categoryname).attr('data-categoryid', categoryData.category_id);
        $('#modify-apikey-item-category').show();
    };

    const toggleModifyApikeyPeriodUnlimitElm = () => {
        $('#modify-apikey-period-begindate-field, #modify-apikey-period-enddate-field').prop('disabled', $('#modify-apikey-period-unlimit').prop('checked'));
    };

    const setModifyPeriodItem = (triggerElm) => {
        const data = JSON.parse(triggerElm.attr('data-content'));
        $('#modify-apikey-period-unlimit').prop('checked', !data.begin_date && !data.end_date);
        $('#modify-apikey-period-begindate-field').val(data.begin_date);
        $('#modify-apikey-period-enddate-field').val(data.end_date);
        toggleModifyApikeyPeriodUnlimitElm();
        $('#modify-apikey-item-period').show();
    };

    const updateApikey = (triggerElm) => {
        let conf = confirm(triggerElm.attr('data-confirmtext'));
        if( !conf ) return;
        if( conf ){
            const paramsData = {
                target: 'api_key',
                id: triggerElm.attr('data-id'),
            };
            if( triggerElm.attr('data-removekey') ){
                paramsData.remove_api_key = true;
            }else{
                paramsData.update_api_key = true;
            }
            dynamicSummaryTable.executeProcess({
                paramsData,
                path: $('#apikey-modify-path').val(),
                additionalAlertMessage: (paramsData.update_api_key ? '変更に失敗しました。' : '削除に失敗しました。')
            });
        }
        return false; 
    };

    const enableUpdateApikeyItems = () => {
        $('#' + summaryTableId + ' [data-updateapikey="true"]').on('click', function(){
            const updateItem = $(this).attr('data-update');
            if( updateItem == 'key' ){
                updateApikey($(this));
                return;
            }
            const modal = $('#modalModifyApikey');
            modal.find('.modify-apikey-item').hide();
            $('#modify-apikey-id').val($(this).attr('data-id'));
            const itemNames = JSON.parse($('#apikey-item-names').val());
            modal.find('.modal-title').text(itemNames[updateItem] + '変更');
            switch (updateItem){
                case 'name':
                    setModifyName($(this));
                    break;
                case 'category':
                    setModifyCategoryItem($(this));
                    break;
                case 'period':
                    setModifyPeriodItem($(this));
                    break;
            };
            modal.modal('show');
        });
    };

    $('#modify-apikey-period-unlimit').on('change', toggleModifyApikeyPeriodUnlimitElm);

    const modifyApikeySearchCategoryInput = $('#modify-apikey-search-category-field');
    const modifySearchedCatsArea = $('#modify-apikey-searched-categories');
    const resetModifySearchCategories = () => {
        modifySearchedCatsArea.find('[data-toggle="tooltip"]').tooltip('hide');
        modifySearchedCatsArea.html('');
        modifyApikeySearchCategoryInput.removeAttr('data-categoryid');
    };

    const bindModifySearchedCategoryEvent = (elm) => {
        elm.tooltip();
        elm.on('click', function(){
            resetModifySearchCategories();
            modifyApikeySearchCategoryInput.val($(this).attr('data-categoryname')).attr('data-categoryid', $(this).attr('data-categoryid'));
        });
    };

    modifyApikeySearchCategoryInput.on('input', function(){
        const str = $(this).val();
        if( str ){
            $.post(searchCategoriesPath, { name: str }, function(categories){
                resetModifySearchCategories();
                if(categories.length && modifyApikeySearchCategoryInput.val()){
                    modifySearchedCatsArea.append(createSearchedCatElms(categories));
                    bindModifySearchedCategoryEvent(modifySearchedCatsArea.find('[data-categoryid]'));
                }
            });
        }else{
            resetModifySearchCategories();
        }
    });


    const validateModifyApikeyValue = (target) => {
        let errMessage = '';
        if( target === 'name' ){
            if( !$('#modify-apikey-name-field').val() ){
                errMessage = '1文字以上入力して下さい';
            }
        }else if( target === 'category' ){
            if( !modifyApikeySearchCategoryInput.attr('data-categoryid') ){
                errMessage = 'カテゴリが選択されておりません。\n表示されたカテゴリから選択して下さい。';
            }
        }else if( target === 'period' ){
            if( !$('#modify-apikey-period-unlimit').prop('checked') ){
                let beginDate = get_date($('#modify-apikey-period-begindate-field').val());
                let endDate = get_date($('#modify-apikey-period-enddate-field').val());
                if( beginDate === false || endDate === false ){
                    errMessage = '日付は、西暦4桁、及び月と日のある日付を入力して下さい。\n※数字は半角で入力して下さい。';
                }else if( !$('#modify-apikey-period-begindate-field').val() && !$('#modify-apikey-period-enddate-field').val() ){
                    errMessage = '有効期限が無期限でない場合は、日付を入力して下さい';
                }else if( beginDate && endDate ){
                    beginDate = new Date(beginDate);
                    endDate = new Date(endDate);
                    if( beginDate > endDate ){
                        errMessage = '開始日は、終了日以前の日付にして下さい';
                    }   
                }
            }
        }
        return errMessage;
    };

    $('#exec-modify-apikey').on('click', function(){
        const button = $(this);
        button.addClass('pointer-events-none');
        const paramsData = { id: $('#modify-apikey-id').val() };
        $('#modalModifyApikey [name]:visible').each((idx, elm) => {
            elm = $(elm);
            switch (elm.attr('name')){
                case 'name':
                    paramsData.name = elm.val();
                    paramsData.target = 'name';
                    break;
                case 'category':
                    paramsData.category_id = elm.attr('data-categoryid');
                    paramsData.target = 'category';
                    break;
                case 'begindate':
                    if( !elm.prop('disabled') ){
                        paramsData.begin_date = elm.val();
                        paramsData.target = 'period'; //有効期限変更
                    }
                    break;
                case 'enddate':
                    if( !elm.prop('disabled') ){
                        paramsData.end_date = elm.val();
                        paramsData.target = 'period'; //有効期限変更
                    }
                    break;
                case 'is_unlimit_period':
                    paramsData.is_unlimit_period = elm.prop('checked');
                    paramsData.target = 'period'; //有効期限変更
                    break;
            };
        });
        const errMsg = validateModifyApikeyValue(paramsData.target);
        if( !errMsg ){
            const exec = () => {
                dynamicSummaryTable.executeProcess({
                    paramsData,
                    path: $('#apikey-modify-path').val(),
                    additionalAlertMessage: '変更に失敗しました。'
                });
                button.removeClass('pointer-events-none');
            };
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                    additionalAlertMessage: '削除に失敗しました。'
                });
                if( roleOrCategoryError ) return;
                exec();
            })();
        }else{
            alert(errMsg);
            button.removeClass('pointer-events-none');
        }
    });

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
    const bindSummaryTableRowEvents = () => {
        enable_bootstrap_actions('#' + summaryTableId);
        enableUpdateApikeyItems();
        enableDeleteApikey();
    };

    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all-apikey-ids',
        chkboxSelector: `#${summaryTableId} .apikeySelect`,
        allChkboxSelector: `#${summaryTableId} .allApikeySelect`,
        bulkBtnSelector: '.action-btn',
    });

    set_style_for_ActionArea_on_filter_dropdown_event();
    sticky_control(summaryTableId, pageName);
    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) });

    //フィルタ
    const summaryTableFilter = new SummaryTableFilter({dynamicSummaryTable});

    summaryTableFilter.enableFilterText(
       '#filter-name',
       '#reset-filter-name',
       { fieldSelector: '#filter-name-field' },
       'name'
    ); //APIキー名

    //summaryTableFilter.enableFilterText(
    //   '#filter-categoryname',
    //   '#reset-filter-categoryname',
    //   { fieldSelector: '#filters-categoryname-field' },
    //   'categoryname'
    //); //カテゴリ名
    summaryTableFilter.enableFilterChangeLink('.category-change'); //カテゴリ検索

    summaryTableFilter.enableFilterDate2('#exec-filter-date2', '#reset-filter-date2', 'period'); //有効期間検索
    summaryTableFilter.enableResetAllFilters('#filter-clear'); //全リセット

    const newApikeyNameElm = $('#create-new-apikey-name-field');
    const searchedCatsArea = $('#create-new-apikey-searched-categories');
    const searchCatsInput = $('#create-new-apikey-search-category-field');
    const newApikeyCatIdElm = $('#create-new-apikey-categoryid');

    const toggleExecButton = () => {
        let isPass = true;
        if( !newApikeyNameElm.val() ){
            isPass = false;
        }else if( searchCatsInput.length && !searchCatsInput.val() ){
            isPass = false;
        }else if( !newApikeyCatIdElm.val() ){
            isPass = false;
        }
        $('#exec-regist-new-apikey').prop('disabled', !isPass);
    };

    const resetSearchCategories = () => {
        searchedCatsArea.find('[data-toggle="tooltip"]').tooltip('hide');
        searchedCatsArea.html('');
        newApikeyCatIdElm.val('');
    };

    $('#create-new-apikey').on('click', function(){
        if( searchCatsInput.length ){
            searchCatsInput.val('');
            resetSearchCategories();
        }
        newApikeyNameElm.val('');
        toggleExecButton();
        $('#modalCreateNewApikey').modal('show');
    });

    newApikeyNameElm.on('input', function(){
        toggleExecButton();
    });

    const bindSearchedCatEvent = (elm) => {
        elm.tooltip();
        elm.on('click', function(){
            resetSearchCategories();
            searchCatsInput.val($(this).attr('data-categoryname'));
            newApikeyCatIdElm.val($(this).attr('data-categoryid'));
            toggleExecButton();
        });
    };

    searchCatsInput.on('input', function(){
        const str = $(this).val();
        if( str ){
            $.post(searchCategoriesPath, { name: str }, function(categories){
                resetSearchCategories();
                if(categories.length && searchCatsInput.val()){
                    searchedCatsArea.append(createSearchedCatElms(categories));
                    bindSearchedCatEvent(searchedCatsArea.find('[data-categoryid]'));
                    toggleExecButton();
                }
            });
        }else{
            resetSearchCategories();
        }
    });

    //登録実行
    $('#exec-regist-new-apikey').on('click', function(){
        const button = $(this);
        button.prop('disabled', true);
        const name = newApikeyNameElm.val();
        if( !name ){
            alert('APIキー名を入力して下さい。');
            button.prop('disabled', false);
            return;
        }
        if( searchCatsInput.length ){
            if( !newApikeyCatIdElm.val() ){
                alert('カテゴリが選択されておりません。\nカテゴリを検索後、表示されたカテゴリから選択して下さい。');
                button.prop('disabled', false);
                return;
            }
        }
        dynamicSummaryTable.executeProcess({
            paramsData: { name, category_id: newApikeyCatIdElm.val() },
            path: $('#apikey-create-path').val(),
            additionalAlertMessage: '登録に失敗しました。'
        });
    });

    //クリップボードにコピー
    const clipboard = new ClipboardJS('.copy-apikey');

    //クリップボードがサポートされていなければアラートを表示 
    $(document).on('click', '.copy-apikry', function(e){
        e.stopPropagation();
        $('#tooltip-copy-btn').tooltip('hide');
        if(clipboard_supported === false) {
            alert('このブラウザではコピー機能はサポートされておりません');
            return false;
        }
    });

    clipboard.on('success', function(e){
        let trigger = $(e.trigger);
        trigger.tooltip('show');
        setTimeout(function (){
            trigger.tooltip('hide');
            e.clearSelection();
        }, 1000);
    });

    clipboard.on('error', function(e) {
        alert('コピーに失敗しました');
        e.clearSelection();
    });

    //一括操作
    const createBulkTrElm = (name) => {
        let tr = $('<tr>');
        tr.append(
            $('<td>').attr('class', 'text-left text-break').text(name)
        );
        return tr;
    };

    //一括削除
    $('#bulk-delete-button').on('click', function(){
        let checked = JSON.parse($('#pages_checked').val());
        $('#bulk-delete-ids').val(JSON.stringify(checked));
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '削除に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            $.post( $('#api-get-apikey-names-path').val(), { ids: JSON.stringify(checked) }, function(names){
                $('#modalBulkDeleteTbody *').remove();
                const sliceNum = 20;
                let nameElms = [];
                const target = $('#modalBulkDeleteTbody');
                names.slice(0, sliceNum).forEach( name => nameElms.push(createBulkTrElm(name)) );
                target.append(nameElms);
                $('#modalBulkDelete').modal('show');
                setTimeout(function(){
                    names.slice(sliceNum).forEach( name => target.append(createBulkTrElm(name)) );
                });
            });
        })();
    });

    $('#exec-bulk-delete').on('click', function(){
        let conf = confirm('一括削除します。よろしいですか?');
        if( !conf ) return;
        dynamicSummaryTable.executeProcessWithProgressBar({
            ids: JSON.parse($('#bulk-delete-ids').val()),
            paramsData: {},
            idsParamsName: 'ids',
            path: $('#apikey-bulk-delete-path').val(),
        });
        return false;
    });
});