#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class CreateSidfmdLproducts < ActiveRecord::Migration
  def change
    # ローカルプロダクト
    # SidfmdProduct相当。
    create_table :sidfmd_lproducts do |t|
      # 有効・無効。無効なら表示しない。
      t.boolean :flg_enable, :null => false, :default => true, :index => true
      # プロダクト名称
      t.text :name, :null => false, :index => true
      # プロダクト一覧での順序を示す値。
      # プロダクト名称でのソートでは対応できない場合に設定する。
      # priority->nameの順でソートすること。
      t.integer :priority, :null => false, :default => 0, :index => true

      t.timestamps
    end

    # ローカルプロダクトのバージョン
    # SidfmdVersion相当。
    create_table :sidfmd_lversions do |t|
      # ローカルプロダクトへの参照
      t.references :sidfmd_lproduct, :null => false, :index => true
      # 有効・無効。無効なら表示しない。
      t.boolean :flg_enable, :null => false, :default => true, :index => true
      # バージョン名称
      t.text :version, :null => false, :index => true
      # プロダクト名を含めたフルネーム
      t.text :fullname, :null => false, :index => true
      # バージョン一覧での順序を示す値。
      # バージョン名称でのソートでは対応できない場合に設定する。
      # priority->nameの順でソートすること。
      # なお、この値についてはプロダクト毎に独立であるように設定・使用すること。
      t.integer :priority, :null => false, :default => 0, :index => true

      t.timestamps
    end

    # ホストとローカルプロダクトの関連付け
    # SidfmdHostApp相当。
    create_table :sidfmd_host_lapps do |t|
      # ホストへの参照
      t.references :sidfmd_host, :null => false, :index => true
      # ローカルプロダクトへの参照
      t.references :sidfmd_lproduct, :null => false, :index => true
      # ローカルプロダクトのバージョンへの参照
      # なお、バージョンではなくプロダクト全体への参照を示す場合は、
      # nullを設定すること。
      t.references :sidfmd_lversion, :null => true, :index => true

      t.timestamps
    end

  end
end
