# -*- coding: utf-8 -*-

class CreateCtags < ActiveRecord::Migration
  def change
    # カスタムタグ
    create_table :sidfmd_ctags do |t|
      # 所属するカスタムカテゴリ。nullはカスタムカテゴリに関係なく全てで利用可能なカスタムタグを意味する。
      t.references :sidfmd_category, null: true, index: true
      # カスタムタグ名称。モデルで重複チェックすること。
      t.text :keyword, null: false, index: true
      # 文字色。"#fff"等。
      t.text :color, null: false
      # 背景色。"#000"等。
      t.text :bgcolor, null: false
      # 優先度。値が大きいものが表示で優先される。
      t.integer :priority, null: false, default: 0, index: true
    end

    # カスタムタグと脆弱性チケットの関連付け
    create_table :sidfmd_issue_ctags do |t|
      t.references :sidfmd_issue, null: false, index: true
      t.references :sidfmd_ctag, null: false, index: true
    end
    # SidfmdVulViewに関連づけられるカスタムタグ
    create_table :sidfmd_vv_ctags do |t|
      t.references :sidfmd_vul_view, null: false, index: true
      t.references :sidfmd_host, null: false, index: true
      t.references :sidfmd_ctag, null: false, index: true
    end
    # SidfmdHostViewに関連づけられるカスタムタグ
    create_table :sidfmd_hv_ctags do |t|
      t.references :sidfmd_host_view, null: false, index: true
      t.integer :html_id, null: false, index: true
      t.references :sidfmd_ctag, null: false, index: true
    end
    # ルール条件「カスタムタグ」
    create_table :sidfmd_rule_c_ctags do |t|
      t.references :sidfmd_rule_cond, null: false, index: true
    end
    # ルール条件「カスタムタグ」の従属テーブル
    create_table :sidfmd_rule_cm_ctags do |t|
      t.references :sidfmd_rule_c_ctag, null: false, index: true
      t.references :sidfmd_ctag, null: false, index: true
      # 設定内容: trueなら指定したタグがあれば成立、falseなら指定したタグがあれば非成立
      t.boolean :is_enable, null: false, default: true
    end
    # ルール操作「カスタムタグ設定」
    create_table :sidfmd_rule_a_ctags do |t|
      t.references :sidfmd_rule_action, null: false, index: true
    end
    # ルール操作「カスタムタグ設定」の従属テーブル
    create_table :sidfmd_rule_am_ctags do |t|
      t.references :sidfmd_rule_a_ctag, null: false, index: true
      t.references :sidfmd_ctag, null: false, index: true
      # 設定内容: trueなら指定したタグを設定、falseなら指定したタグを除去
      # 指定されたタグ以外は操作しない。
      t.boolean :is_enable, null: false, default: true
    end
  end

end
