require_dependency 'account_controller'

module SidfmPatches
  module AccountControllerPatch
    extend ActiveSupport::Concern

    included do 
      alias_method :logout, :logout_without_home_page
      alias_method :login, :login_with_history
    end
 
    #セッションが切れた後に、Redmine管理者以外はredmineのページに遷移させずにVMページに遷移させる
    def logout_without_home_page
      if User.current.anonymous?
        redirect_to home_url
      elsif request.post?
        logout_user
        redirect_to home_url
      else
        redirect_to sidfm_index_path
      end
    end

    def login_with_history
      if request.post?
        authenticate_user
        user = User.current
        SidfmdLoginHistory.create(:login_date => Time.now, :user_id => user.id, :login => user.login)
      else
        if User.current.logged?
          redirect_back_or_default home_url, :referer => true
        end
      end
    rescue AuthSourceException => e
      logger.error "An error occurred when authenticating #{params[:username]}: #{e.message}"
      render_error :message => e.message
    end

  end
end

ActionDispatch::Reloader.to_prepare do
  unless AccountController.included_modules.include?(SidfmPatches::AccountControllerPatch)
    AccountController.send(:include, SidfmPatches::AccountControllerPatch)
  end
end

