#!/bin/bash

cmd="sudo docker exec sidfmd"

function apply(){
  $cmd cp /etc/httpd/conf.d/ssl.conf.custom /etc/httpd/conf.d/ssl.conf
  if $cmd test -f /etc/pki/tls/certs/server-chain.crt;then
    $cmd sed -i s/#SSLCertificateChainFile/SSLCertificateChainFile/g /etc/httpd/conf.d/ssl.conf
  fi
  $cmd /vm_manage server disable
  sleep 5
  $cmd /vm_manage server enable
}

function copy_cert(){
  sudo docker cp $1 sidfmd:/etc/pki/tls/certs/server.crt
}

function copy_key(){
  sudo docker cp $1 sidfmd:/etc/pki/tls/private/server.key
}

function copy_chain(){
  sudo docker cp $1 sidfmd:/etc/pki/tls/certs/server-chain.crt
}

function clean(){
  $cmd rm -f /etc/pki/tls/certs/server.crt
  $cmd rm -f /etc/pki/tls/private/server.key
  $cmd rm -f /etc/pki/tls/certs/server-chain.crt
  $cmd cp /etc/httpd/conf.d/ssl.conf.default /etc/httpd/conf.d/ssl.conf
  $cmd /vm_manage server disable
  sleep 5
  $cmd /vm_manage server enable
}

function usage(){
  echo -e "usage: $0"
  echo -e "  -cert <cert file> \tinstall server certificate."
  echo -e "  -key <key file> \tinstall server key."
  echo -e "  -chain <chain file> \tinstall server certificate chain. (optional)"
  echo -e "  -apply \t\tapply configuration & restart service."
  echo -e "  -clean \t\tdelete installed certificate, key, chain."
  exit 1
}

case $1 in
  "-cert")
    if [ ! -f "$2" ];then
      usage
    else
      copy_cert $2
    fi
  ;;
  "-key")
    if [ ! -f "$2" ];then
      usage
    else
      copy_key $2
    fi
  ;;
  "-chain")
    if [ ! -f "$2" ];then
      usage
    else
      copy_chain $2
    fi
  ;;
  "-clean")
    clean
  ;;
  "-apply")
    apply
  ;;
  *)
    usage
  ;;
esac
exit 0
